@@ -8,11 +8,11 @@
 #define AMAROK_ACTIONCLASSES_H
 
 #include "engineobserver.h"
+#include "prettypopupmenu.h"
 #include "sliderwidget.h"
 
 #include <kaction.h>
 #include <kactionclasses.h>
-#include <kpopupmenu.h>
 #include <qguardedptr.h>
 
 class KActionCollection;
@@ -21,7 +21,7 @@ class KHelpMenu;
 
 namespace amaroK
 {
-    class Menu : public KPopupMenu
+    class Menu : public PrettyPopupMenu
     {
         Q_OBJECT
         public:
@@ -32,11 +32,9 @@ namespace amaroK
                 ID_CONF_DECODER,
                 ID_SHOW_VIS_SELECTOR,
                 ID_SHOW_COVER_MANAGER,
-                ID_SHOW_SCRIPT_SELECTOR,
-                ID_SHOW_SCRIPT_CONSOLE,
-                ID_SHOW_EFFECTS,
+                ID_CONFIGURE_EQUALIZER,
                 ID_SHOW_WIZARD,
-                ID_SHOW_MENUBAR
+                ID_RESCAN_COLLECTION
             };
 
         public slots:
@@ -60,14 +58,24 @@ namespace amaroK
     };
 
 
-    class PlayPauseAction : public KAction, public EngineObserver
+    class PlayPauseAction : public KToggleAction, public EngineObserver
     {
         public:
             PlayPauseAction( KActionCollection* );
-            ~PlayPauseAction();
-            virtual void engineStateChanged( Engine::State );
+            virtual void engineStateChanged( Engine::State, Engine::State = Engine::Empty );
     };
 
+    class AnalyzerContainer : public QWidget
+    {
+        public:
+            AnalyzerContainer( QWidget *parent );
+        protected:
+            virtual void resizeEvent( QResizeEvent* );
+            virtual void mousePressEvent( QMouseEvent* );
+        private:
+            void changeAnalyzer();
+            QWidget *m_child;
+    };
 
     class AnalyzerAction : public KAction
     {
@@ -81,57 +89,64 @@ namespace amaroK
     {
         public:
             VolumeAction( KActionCollection* );
-            ~VolumeAction();
-
             virtual int plug( QWidget *, int index = -1 );
-
         private:
             void engineVolumeChanged( int value );
-
-            QGuardedPtr<amaroK::Slider> m_slider;
+            QGuardedPtr<amaroK::VolumeSlider> m_slider;
     };
 
 
     class ToggleAction : public KToggleAction
     {
         public:
-            ToggleAction( const QString &text, void ( *f ) ( bool ), KActionCollection* const ac, const char *name )
-                    : KToggleAction( text, 0, ac, name )
-                    , m_function( f )
-            {}
+            ToggleAction( const QString &text, void ( *f ) ( bool ), KActionCollection* const ac, const char *name );
 
-            virtual void setChecked( bool b )
-            {
-                const bool announce = b != isChecked();
+            virtual void setChecked( bool b );
 
-                m_function( b );
-                KToggleAction::setChecked( b );
-                if( announce ) emit toggled( b ); //KToggleAction doesn't do this for us. How gay!
-            }
+            virtual void setEnabled( bool b );
 
         private:
             void ( *m_function ) ( bool );
     };
 
+    class SelectAction : public KSelectAction
+    {
+        public:
+            SelectAction( const QString &text, void ( *f ) ( int ), KActionCollection* const ac, const char *name );
 
-    class RandomAction : public ToggleAction
+            virtual void setCurrentItem( int n );
+
+            virtual void setEnabled( bool b );
+
+            virtual void setIcons( QStringList icons );
+
+            QStringList icons() const;
+
+            QString currentIcon() const;
+
+        private:
+            void ( *m_function ) ( int );
+            QStringList m_icons;
+    };
+
+
+    class RandomAction : public SelectAction
     {
         public:
             RandomAction( KActionCollection *ac );
+            virtual void setCurrentItem( int n );
     };
 
-
-    class RepeatTrackAction : public ToggleAction
+    class FavorAction : public SelectAction
     {
         public:
-            RepeatTrackAction( KActionCollection *ac );
+            FavorAction( KActionCollection *ac );
     };
 
-
-    class RepeatPlaylistAction : public ToggleAction
+    class RepeatAction : public SelectAction
     {
         public:
-            RepeatPlaylistAction( KActionCollection *ac );
+            RepeatAction( KActionCollection *ac );
     };
 
     class BurnMenu : public KPopupMenu
@@ -162,6 +177,35 @@ namespace amaroK
             virtual int plug( QWidget*, int index = -1 );
     };
 
+    class StopMenu : public KPopupMenu
+    {
+            Q_OBJECT
+
+        public:
+            enum MenuIds {
+                NOW,
+                AFTER_TRACK,
+                AFTER_QUEUE
+            };
+
+            static KPopupMenu *instance();
+
+        private slots:
+            void slotAboutToShow();
+            void slotActivated( int index );
+
+        private:
+            StopMenu();
+    };
+
+
+    class StopAction : public KAction
+    {
+        public:
+            StopAction( KActionCollection* );
+            virtual int plug( QWidget*, int index = -1 );
+    };
+
 } /* namespace amaroK */
 
 
