@@ -3,6 +3,7 @@
                             -------------------
    begin                : Sat Oct 11 2003
    copyright            : (C) 2003 by Stanislav Karchebny
+                          (C) 2004 Christian Muehlhaeuser
                           (C) 2005 Ian Monroe
                           (C) 2005 Seb Ruiz
    email                : berkus@users.sf.net
@@ -24,9 +25,11 @@
 #include "debug.h"
 #include "collectiondb.h"
 #include "contextbrowser.h"
+#include "devicemanager.h"
 #include "engine/enginebase.h"
 #include "enginecontroller.h"
 #include "equalizersetup.h"
+#include "htmlview.h"
 #include "mediabrowser.h"
 #include "osd.h"
 #include "playlist.h"
@@ -60,9 +63,14 @@ namespace amaroK
         }
     }
 
+    QString DcopPlayerHandler::version()
+    {
+        return APP_VERSION;
+    }
+
     bool DcopPlayerHandler::dynamicModeStatus()
     {
-        return AmarokConfig::dynamicMode();
+        return amaroK::dynamicMode();
     }
 
     bool DcopPlayerHandler::equalizerEnabled()
@@ -73,6 +81,11 @@ namespace amaroK
             return false;
     }
 
+    bool DcopPlayerHandler::osdEnabled()
+    {
+        return AmarokConfig::osdEnabled();
+    }
+
     bool DcopPlayerHandler::isPlaying()
     {
         return EngineController::engine()->state() == Engine::Playing;
@@ -85,12 +98,12 @@ namespace amaroK
 
     bool DcopPlayerHandler::repeatPlaylistStatus()
     {
-        return AmarokConfig::repeatPlaylist();
+        return amaroK::repeatPlaylist();
     }
 
     bool DcopPlayerHandler::repeatTrackStatus()
     {
-        return AmarokConfig::repeatTrack();
+        return amaroK::repeatTrack();
     }
 
     int DcopPlayerHandler::getVolume()
@@ -110,6 +123,13 @@ namespace amaroK
         return score;
     }
 
+    int DcopPlayerHandler::rating()
+    {
+        const MetaBundle &bundle = EngineController::instance()->bundle();
+        int rating = CollectionDB::instance()->getSongRating( bundle.url().path() );
+        return rating;
+    }
+
     int  DcopPlayerHandler::status()
     {
         // <0 - error, 0 - stopped, 1 - paused, 2 - playing
@@ -172,7 +192,7 @@ namespace amaroK
 
     QString DcopPlayerHandler::currentTime()
     {
-        return MetaBundle::prettyLength( EngineController::engine() ->position() / 1000 );
+        return MetaBundle::prettyLength( EngineController::engine() ->position() / 1000 ,true );
     }
 
     QString DcopPlayerHandler::encodedURL()
@@ -213,7 +233,7 @@ namespace amaroK
     QString DcopPlayerHandler::setContextStyle(const QString& msg)
     {
         AmarokConfig::setContextBrowserStyleSheet( msg );
-        ContextBrowser::instance()->setStyleSheet();
+        ContextBrowser::instance()->reloadStyleSheet();
 
         if ( QFile::exists( amaroK::saveLocation( "themes/" + msg + '/' ) + "stylesheet.css" ) )
             return "Context browser theme '"+msg+"' applied.";
@@ -233,7 +253,10 @@ namespace amaroK
 
     QString DcopPlayerHandler::track()
     {
-        return EngineController::instance()->bundle().track();
+        if ( EngineController::instance()->bundle().track() != 0 )
+            return QString::number( EngineController::instance()->bundle().track() );
+        else
+            return QString();
     }
 
     QString DcopPlayerHandler::type()
@@ -243,7 +266,7 @@ namespace amaroK
 
     QString DcopPlayerHandler::year()
     {
-        return EngineController::instance()->bundle().year();
+        return QString::number( EngineController::instance()->bundle().year() );
     }
 
     void DcopPlayerHandler::configEqualizer()
@@ -253,11 +276,6 @@ namespace amaroK
             EqualizerSetup::instance()->raise();
     }
 
-    void DcopPlayerHandler::enableDynamicMode( bool enable )
-    {
-        static_cast<KToggleAction*>(amaroK::actionCollection()->action( "dynamic_mode" ))->setChecked( enable );
-    }
-
     void DcopPlayerHandler::enableOSD(bool enable)
     {
         amaroK::OSD::instance()->setEnabled( enable );
@@ -266,7 +284,8 @@ namespace amaroK
 
     void DcopPlayerHandler::enableRandomMode( bool enable )
     {
-        static_cast<KToggleAction*>(amaroK::actionCollection()->action( "random_mode" ))->setChecked( enable );
+        static_cast<KSelectAction*>(amaroK::actionCollection()->action( "random_mode" ))
+            ->setCurrentItem( enable ? AmarokConfig::EnumRandomMode::Tracks : AmarokConfig::EnumRandomMode::Off );
     }
 
     void DcopPlayerHandler::enableRepeatPlaylist( bool enable )
@@ -281,12 +300,12 @@ namespace amaroK
 
     void DcopPlayerHandler::mediaDeviceMount()
     {
-        MediaDevice::instance()->mount();
+        MediaBrowser::instance()->currentDevice()->connectDevice();
     }
 
     void DcopPlayerHandler::mediaDeviceUmount()
     {
-        MediaDevice::instance()->umount();
+        MediaBrowser::instance()->currentDevice()->disconnectDevice();
     }
 
     void DcopPlayerHandler::mute()
@@ -321,14 +340,14 @@ namespace amaroK
 
     void DcopPlayerHandler::queueForTransfer( KURL url )
     {
-        MediaDevice::instance()->addURL( url );
+        MediaBrowser::queue()->addURL( url );
+        MediaBrowser::queue()->URLsAdded();
     }
 
     void DcopPlayerHandler::seek(int s)
     {
-        EngineBase* const engine = EngineController::engine();
-        if ( s > 0 && engine->state() != Engine::Empty )
-            engine ->seek( s * 1000 );
+        if ( s > 0 && EngineController::engine()->state() != Engine::Empty )
+            EngineController::instance()->seek( s * 1000 );
     }
 
     void DcopPlayerHandler::seekRelative(int s)
@@ -357,6 +376,9 @@ namespace amaroK
     {
         EngineController::engine()->setEqualizerEnabled( active );
         AmarokConfig::setEqualizerEnabled( active );
+
+        if( EqualizerSetup::isInstantiated() )
+            EqualizerSetup::instance()->setActive( active );
     }
 
     void DcopPlayerHandler::setEqualizerPreset( QString name )
@@ -386,11 +408,27 @@ namespace amaroK
         CollectionDB::instance()->setSongPercentage(url, score);
     }
 
+    void DcopPlayerHandler::setRating( int rating )
+    {
+        const QString &url = EngineController::instance()->bundle().url().path();
+        CollectionDB::instance()->setSongRating(url, rating);
+    }
+
+    void DcopPlayerHandler::setRatingByPath( const QString &url, int rating )
+    {
+        CollectionDB::instance()->setSongRating(url, rating);
+    }
+
     void DcopPlayerHandler::setVolume(int volume)
     {
         EngineController::instance()->setVolume(volume);
     }
 
+    void DcopPlayerHandler::setVolumeRelative(int ticks)
+    {
+        EngineController::instance()->increaseVolume(ticks);
+    }
+
     void DcopPlayerHandler::showBrowser( QString browser )
     {
         if ( browser == "context" )
@@ -417,7 +455,7 @@ namespace amaroK
 
     void DcopPlayerHandler::transferDeviceFiles()
     {
-        MediaDevice::instance()->transferFiles();
+        MediaBrowser::instance()->currentDevice()->transferFiles();
     }
 
     void DcopPlayerHandler::volumeDown()
@@ -439,7 +477,10 @@ namespace amaroK
         //the amarok binary anyway --mxcl
         debug() << "Startup ID: " << args.first() << endl;
         kapp->setStartupId( args.first().local8Bit() );
+#ifdef Q_WS_X11
+        // currently X11 only
         KStartupInfo::appStarted();
+#endif
         args.pop_front();
 
         const int argc = args.count() + 1;
@@ -476,7 +517,7 @@ namespace amaroK
 
     int  DcopPlaylistHandler::getActiveIndex()
     {
-        return Playlist::instance()->currentTrackIndex();
+        return Playlist::instance()->currentTrackIndex( false );
     }
 
     int  DcopPlaylistHandler::getTotalTrackCount()
@@ -512,11 +553,10 @@ namespace amaroK
 
     void DcopPlaylistHandler::playMedia( const KURL &url )
     {
-        /* ContextBrowser::openURLRequest is necessary for konquisidebar */
-        ContextBrowser::instance()->openURLRequest( url );
+        Playlist::instance()->insertMedia( url, Playlist::DirectPlay | Playlist::Unique);
     }
 
-    void DcopPlaylistHandler::popupMessage(const QString& msg)
+    void DcopPlaylistHandler::popupMessage( const QString& msg )
     {
         StatusBar::instance()->longMessageThreadSafe( msg );
     }
@@ -530,6 +570,17 @@ namespace amaroK
         }
     }
 
+    void DcopPlaylistHandler::removeByIndex( int index )
+    {
+        PlaylistItem* const item =
+            static_cast<PlaylistItem*>( Playlist::instance()->itemAtIndex( index ) );
+
+        if ( item ) {
+            Playlist::instance()->removeItem( item );
+            delete item;
+        }
+    }
+
     void DcopPlaylistHandler::repopulate()
     {
         Playlist::instance()->repopulate();
@@ -584,6 +635,11 @@ namespace amaroK
         PlaylistBrowser::instance()->addPlaylist( url );
     }
 
+    int DcopPlaylistBrowserHandler::loadPlaylist( const QString &playlist )
+    {
+        return PlaylistBrowser::instance()->loadPlaylist( playlist );
+    }
+
 /////////////////////////////////////////////////////////////////////////////////////
 // class DcopContextBrowserHandler
 /////////////////////////////////////////////////////////////////////////////////////
@@ -593,11 +649,6 @@ namespace amaroK
         , QObject( kapp )
     {}
 
-    void DcopContextBrowserHandler::showHome()
-    {
-        ContextBrowser::instance()->showHome();
-    }
-
     void DcopContextBrowserHandler::showCurrentTrack()
     {
         ContextBrowser::instance()->showCurrentTrack();
@@ -613,6 +664,11 @@ namespace amaroK
         ContextBrowser::instance()->showWikipedia();
     }
 
+    void DcopContextBrowserHandler::showLyrics( const QCString& lyrics )
+    {
+        ContextBrowser::instance()->lyricsResult( lyrics );
+    }
+
 /////////////////////////////////////////////////////////////////////////////////////
 // class DcopCollectionHandler
 /////////////////////////////////////////////////////////////////////////////////////
@@ -658,12 +714,22 @@ namespace amaroK
         return final;
     }
 
+    bool DcopCollectionHandler::isDirInCollection( const QString& path )
+    {
+        return CollectionDB::instance()->isDirInCollection( path );
+    }
+
+    bool DcopCollectionHandler::moveFile( const QString &oldURL, const QString &newURL, bool overwrite )
+    {
+        return CollectionDB::instance()->moveFile( oldURL, newURL, overwrite );
+    }
+
     QStringList DcopCollectionHandler::query( const QString& sql )
     {
         return CollectionDB::instance()->query( sql );
     }
 
-   QStringList DcopCollectionHandler::similarArtists( int artists )
+    QStringList DcopCollectionHandler::similarArtists( int artists )
     {
         return CollectionDB::instance()->similarArtists( EngineController::instance()->bundle().artist(), artists );
     }
@@ -683,6 +749,22 @@ namespace amaroK
         CollectionDB::instance()->scanMonitor();
     }
 
+    void DcopCollectionHandler::disableAutoScoring( bool disable )
+    {
+        CollectionDB::instance()->disableAutoScoring( disable );
+    }
+
+    void DcopCollectionHandler::newUniqueIdForFile( const QString &path )
+    {
+        CollectionDB::instance()->newUniqueIdForFile( path );
+    }
+
+    void DcopCollectionHandler::newUniqueIdForFiles( const QStringList &list )
+    {
+        foreach( list )
+            CollectionDB::instance()->newUniqueIdForFile( *it );
+    }
+
 /////////////////////////////////////////////////////////////////////////////////////
 // class DcopScriptHandler
 /////////////////////////////////////////////////////////////////////////////////////
@@ -744,6 +826,86 @@ namespace amaroK
         }
         return stringList;
     }
+
+/////////////////////////////////////////////////////////////////////////////////////
+// class DcopDevicesHandler
+/////////////////////////////////////////////////////////////////////////////////////
+
+    DcopDevicesHandler::DcopDevicesHandler()
+        : DCOPObject( "devices" )
+        , QObject( kapp )
+    {}
+
+    void DcopDevicesHandler::mediumAdded(QString name)
+    {
+        DeviceManager::instance()->mediumAdded(name);
+    }
+
+    void DcopDevicesHandler::mediumRemoved(QString name)
+    {
+        DeviceManager::instance()->mediumRemoved(name);
+    }
+
+    void DcopDevicesHandler::mediumChanged(QString name)
+    {
+        DeviceManager::instance()->mediumChanged(name);
+    }
+
+    QStringList DcopDevicesHandler::showDeviceList()
+    {
+        return DeviceManager::instance()->getDeviceStringList();
+    }
+
+/////////////////////////////////////////////////////////////////////////////////////
+// class DcopDevicesHandler
+/////////////////////////////////////////////////////////////////////////////////////
+
+    DcopMediaBrowserHandler::DcopMediaBrowserHandler()
+        : DCOPObject( "mediabrowser" )
+        , QObject( kapp )
+    {}
+
+    void DcopMediaBrowserHandler::deviceConnect()
+    {
+        MediaBrowser::instance()->currentDevice()->connectDevice();
+    }
+
+    void DcopMediaBrowserHandler::deviceDisconnect()
+    {
+        MediaBrowser::instance()->currentDevice()->disconnectDevice();
+    }
+
+    QStringList DcopMediaBrowserHandler::deviceList()
+    {
+        return MediaBrowser::instance()->deviceNames();
+    }
+
+    void DcopMediaBrowserHandler::deviceSwitch( QString name )
+    {
+        MediaBrowser::instance()->deviceSwitch( name );
+    }
+
+    void DcopMediaBrowserHandler::queue( KURL url )
+    {
+        MediaBrowser::queue()->addURL( url );
+        MediaBrowser::queue()->URLsAdded();
+    }
+
+    void DcopMediaBrowserHandler::queueList( KURL::List urls )
+    {
+        MediaBrowser::queue()->addURLs( urls );
+    }
+
+    void DcopMediaBrowserHandler::transfer()
+    {
+        MediaBrowser::instance()->currentDevice()->transferFiles();
+    }
+
+    void DcopMediaBrowserHandler::transcodingFinished( QString src, QString dest )
+    {
+        MediaBrowser::instance()->transcodingFinished( src, dest );
+    }
+
 } //namespace amaroK
 
 #include "amarokdcophandler.moc"
