@@ -21,8 +21,11 @@ email                : markey@web.de
 #include "app.h"
 #include "config.h"
 #include "configdialog.h"
-#include "debug.h"
+#include "contextbrowser.h"
 #include "collectionbrowser.h"
+#include "dbsetup.h"             //firstRunWizard()
+#include "debug.h"
+#include "devicemanager.h"
 #include "effectwidget.h"
 #include "enginebase.h"
 #include "enginecontroller.h"
@@ -45,6 +48,7 @@ email                : markey@web.de
 #include <iostream>
 
 #include <kconfigdialogmanager.h>
+#include <kcombobox.h>           //firstRunWizard()
 #include <kcmdlineargs.h>        //initCliArgs()
 #include <kcursor.h>             //amaroK::OverrideCursor
 #include <kedittoolbar.h>        //slotConfigToolbars()
@@ -54,8 +58,11 @@ email                : markey@web.de
 #include <kkeydialog.h>          //slotConfigShortcuts()
 #include <klocale.h>
 #include <kmessagebox.h>         //applySettings(), genericEventHandler()
+#include <krun.h>                //amaroK::invokeBrowser()
 #include <kstandarddirs.h>
 #include <kurldrag.h>            //genericEventHandler()
+#include <kaboutdata.h>
+#include <kio/job.h>
 
 #include <qevent.h>              //genericEventHandler()
 #include <qeventloop.h>          //applySettings()
@@ -69,13 +76,21 @@ email                : markey@web.de
 
 // For the HyperThreading fix
 #ifdef __linux__
-    #include <features.h>
-    #if defined(__GLIBC_PREREQ) && __GLIBC_PREREQ(2,3)
+    #ifdef SCHEDAFFINITY_SUPPORT
         #include <errno.h>
         #include <sched.h>
-    #endif //__GLIBC_PREREQ && __GLIBC_PREREQ(2,3)
+    #endif //SCHEDAFFINITY_SUPPORT
 #endif //__linux__
 
+int App::mainThreadId = 0;
+
+LIBAMAROK_EXPORT KAboutData aboutData( "amarok",
+    I18N_NOOP( "amaroK" ), APP_VERSION,
+    I18N_NOOP( "The audio player for KDE" ), KAboutData::License_GPL,
+    I18N_NOOP( "(C) 2002-2003, Mark Kretschmann\n(C) 2003-2006, The amaroK Development Squad" ),
+    I18N_NOOP( "IRC:\nirc.freenode.net - #amarok, #amarok.de\n\nFeedback:\namarok-devel@lists.sourceforge.net\n\n(Build Date: " __DATE__ ")" ),
+             ( "http://amarok.kde.org" ) );
+
 App::App()
         : KApplication()
         , m_pPlayerWindow( 0 ) //will be created in applySettings()
@@ -83,7 +98,8 @@ App::App()
     DEBUG_BLOCK
 
     const KCmdLineArgs* const args = KCmdLineArgs::parsedArgs();
-    bool restoreSession = args->count() == 0 || args->isSet( "append" ) || args->isSet( "enqueue" );
+    bool restoreSession = args->count() == 0 || args->isSet( "append" ) || args->isSet( "enqueue" )
+                                || amaroK::config()->readBoolEntry( "AppendAsDefault", false );    
 
     QPixmap::setDefaultOptimization( QPixmap::MemoryOptim );
 
@@ -93,7 +109,9 @@ App::App()
     new amaroK::DcopPlaylistBrowserHandler();
     new amaroK::DcopContextBrowserHandler();
     new amaroK::DcopCollectionHandler();
+    new amaroK::DcopMediaBrowserHandler();
     new amaroK::DcopScriptHandler();
+    new amaroK::DcopDevicesHandler();
 
     // Remember old folder setup, so we can detect changes after the wizard was used
     const QStringList oldCollectionFolders = AmarokConfig::collectionFolders();
@@ -107,6 +125,8 @@ App::App()
 
     fixHyperThreading();
 
+    m_pDeviceManager = DeviceManager::instance();
+
     m_pGlobalAccel    = new KGlobalAccel( this );
     m_pPlaylistWindow = new PlaylistWindow();
     m_pTray           = new amaroK::TrayIcon( m_pPlaylistWindow );
@@ -131,8 +151,7 @@ App::App()
     applySettings( true );
 
     // Start ScriptManager. Must be created _after_ PlaylistWindow.
-    if ( amaroK::config( "ScriptManager" )->readBoolEntry( "Auto Run" ) )
-        ScriptManager::instance();
+    ScriptManager::instance();
 
     //notify loader application that we have started
     std::cout << "STARTUP\n" << std::flush;
@@ -153,22 +172,24 @@ App::App()
         EngineController::instance()->restoreSession();
     }
 
-    // Refetch covers every 80 days or delete every 90 days to comply with Amazon license
+    // Refetch covers every 80 days to comply with Amazon license
     #ifdef AMAZON_SUPPORT
     new RefreshImages();
-    pruneCoverImages();
     #endif
 
     // Trigger collection scan if folder setup was changed by wizard
     if ( oldCollectionFolders != AmarokConfig::collectionFolders() )
-        CollectionDB::instance()->startScan();
+        QTimer::singleShot( 0, CollectionDB::instance(), SLOT( startScan() ) );
     // If database version is updated, the collection needs to be rescanned.
     // Works also if the collection is empty for some other reason
     // (e.g. deleted collection.db)
     else if ( CollectionDB::instance()->isEmpty() )
     {
-        CollectionDB::instance()->startScan();
+        QTimer::singleShot( 0, CollectionDB::instance(), SLOT( startScan() ) );
     }
+    else if ( AmarokConfig::monitorChanges() )
+        QTimer::singleShot( 0, CollectionDB::instance(), SLOT( scanModifiedDirs() ) );
+
 
     handleCliArgs();
 }
@@ -197,11 +218,13 @@ App::~App()
     amaroK::config()->writeEntry( "HiddenOnExit", mainWindow()->isHidden() );
 
     CollectionDB::instance()->stopScan();
-    ThreadWeaver::deleteInstance(); //waits for jobs to finish
+    ThreadWeaver::instance()->abortAllJobsNamed( "CreateMood" ); //these take forever
 
     delete m_pPlayerWindow;   //sets some XT keys
     delete m_pPlaylistWindow; //sets some XT keys
 
+    ThreadWeaver::deleteInstance(); //waits for jobs to finish
+
     // this must be deleted before the connection to the Xserver is
     // severed, or we risk a crash when the QApplication is exited,
     // I asked Trolltech! *smug*
@@ -215,6 +238,39 @@ App::~App()
 }
 
 
+#include <dcopref.h>
+#include <qstringlist.h>
+
+namespace
+{
+    // grabbed from KsCD source, kompatctdisk.cpp
+    QString urlToDevice(const QString& device)
+    {
+        KURL deviceUrl(device);
+        if (deviceUrl.protocol() == "media" || deviceUrl.protocol() == "system")
+        {
+            DCOPRef mediamanager( "kded", "mediamanager" );
+            DCOPReply reply = mediamanager.call( "properties(QString)", deviceUrl.fileName() );
+            QStringList properties = reply;
+
+            if (!reply.isValid() || properties.count() < 6)
+            {
+                debug() << "Invalid reply from mediamanager" << endl;
+                return QString::null;
+            }
+            else
+            {
+                debug() << "Reply from mediamanager " << properties[5] << endl;
+                return properties[5];
+            }
+        }
+
+        return device;
+    }
+
+}
+
+
 void App::handleCliArgs() //static
 {
     static char cwd[PATH_MAX];
@@ -234,9 +290,10 @@ void App::handleCliArgs() //static
             list << args->url( i );
 
         int options;
+        bool appendAsDefault = amaroK::config()->readBoolEntry( "AppendAsDefault", false );
         if( args->isSet( "queue" ) )
            options = Playlist::Queue;
-        else if( args->isSet( "append" ) || args->isSet( "enqueue" ) ) {
+        else if( args->isSet( "append" ) || args->isSet( "enqueue" ) || appendAsDefault ) {
            options = Playlist::Append;
            if( args->isSet( "play" ) )
               options |= Playlist::DirectPlay;
@@ -264,6 +321,18 @@ void App::handleCliArgs() //static
         EngineController::instance()->next();
     else if ( args->isSet( "previous" ) )
         EngineController::instance()->previous();
+    else if (args->isSet("cdplay")) {
+        QString device = args->getOption("cdplay");
+        KURL::List urls;
+        if (EngineController::engine()->getAudioCDContents(device, urls)) {
+            Playlist::instance()->insertMedia(
+                urls, Playlist::Replace|Playlist::DirectPlay);
+        } else { // Default behaviour
+            debug() <<
+                "Sorry, the engine doesn't support direct play from AudioCD..."
+                    << endl;
+        }
+    }
 
     if ( args->isSet( "toggle-playlist-window" ) )
         pApp->m_pPlaylistWindow->showHide();
@@ -278,8 +347,6 @@ void App::handleCliArgs() //static
 
 void App::initCliArgs( int argc, char *argv[] ) //static
 {
-    extern class KAboutData aboutData; //defined in amarokcore/main.cpp
-
     static KCmdLineOptions options[] =
         {
             { "+[URL(s)]", I18N_NOOP( "Files/URLs to open" ), 0 },
@@ -305,11 +372,12 @@ void App::initCliArgs( int argc, char *argv[] ) //static
             { "wizard", I18N_NOOP( "Run first-run wizard" ), 0 },
             { "engine <name>", I18N_NOOP( "Use the <name> engine" ), 0 },
             { "cwd <directory>", I18N_NOOP( "Base for relative filenames/URLs" ), 0 },
+            { "cdplay <device>", I18N_NOOP("Play an AudioCD from <device>"), 0 },
             { 0, 0, 0 }
         };
 
     KCmdLineArgs::reset();
-    KCmdLineArgs::init( argc, argv, &aboutData ); //calls KApplication::addCmdLineOptions()
+    KCmdLineArgs::init( argc, argv, &::aboutData ); //calls KApplication::addCmdLineOptions()
     KCmdLineArgs::addCmdLineOptions( options );   //add our own options
 }
 
@@ -328,6 +396,8 @@ void App::initGlobalShortcuts()
                             ec, SLOT( playPause() ), true, true );
     m_pGlobalAccel->insert( "stop", i18n( "Stop" ), 0, KKey("WIN+v"), 0,
                             ec, SLOT( stop() ), true, true );
+    m_pGlobalAccel->insert( "stop_after_global", i18n( "Stop Playing After Current Track" ), 0, KKey("WIN+CTRL+v"), 0,
+                            Playlist::instance()->qscrollview(), SLOT( toggleStopAfterCurrentTrack() ), true, true );
     m_pGlobalAccel->insert( "next", i18n( "Next Track" ), 0, KKey("WIN+b"), 0,
                             ec, SLOT( next() ), true, true );
     m_pGlobalAccel->insert( "prev", i18n( "Previous Track" ), 0, KKey("WIN+z"), 0,
@@ -349,6 +419,17 @@ void App::initGlobalShortcuts()
     m_pGlobalAccel->insert( "mute", i18n( "Mute Volume" ), 0, KKey("WIN+m"), 0,
                             ec, SLOT( mute() ), true, true );
 
+    m_pGlobalAccel->insert( "rating1", i18n( "Rate Current Track: 1" ), 0, KKey("WIN+1"), 0,
+                            this, SLOT( setRating1() ), true, true );
+    m_pGlobalAccel->insert( "rating2", i18n( "Rate Current Track: 2" ), 0, KKey("WIN+2"), 0,
+                            this, SLOT( setRating2() ), true, true );
+    m_pGlobalAccel->insert( "rating3", i18n( "Rate Current Track: 3" ), 0, KKey("WIN+3"), 0,
+                            this, SLOT( setRating3() ), true, true );
+    m_pGlobalAccel->insert( "rating4", i18n( "Rate Current Track: 4" ), 0, KKey("WIN+4"), 0,
+                            this, SLOT( setRating4() ), true, true );
+    m_pGlobalAccel->insert( "rating5", i18n( "Rate Current Track: 5" ), 0, KKey("WIN+5"), 0,
+                            this, SLOT( setRating5() ), true, true );
+
     m_pGlobalAccel->setConfigGroup( "Shortcuts" );
     m_pGlobalAccel->readSettings( kapp->config() );
     m_pGlobalAccel->updateConnections();
@@ -375,18 +456,18 @@ void App::initGlobalShortcuts()
 
 void App::fixHyperThreading()
 {
-    // Workaround for stability issues with HyperThreading CPU's, @see BUG 99199.
-    // First we detect the presence of HyperThreading. If active, we bind amarokapp
-    // to the first CPU only (hard affinity).
-    //
-    // @see http://www-128.ibm.com/developerworks/linux/library/l-affinity.html
-    // @see http://www.linuxjournal.com/article/6799
-    // (articles on processor affinity with the linux kernel)
+    /** Workaround for stability issues with HyperThreading CPU's, @see BUG 99199.
+     * First we detect the presence of HyperThreading. If active, we bind amarokapp
+     * to the first CPU only (hard affinity).
+     *
+     * @see http://www-128.ibm.com/developerworks/linux/library/l-affinity.html
+     * @see http://www.linuxjournal.com/article/6799
+     * (articles on processor affinity with the linux kernel)
+     */
 
     DEBUG_BLOCK
 
     #ifdef __linux__
-    debug() << "SCHEDAFFINITY_SUPPORT enabled. Testing to see if HT fix should be enabled..." << endl;
     QString line;
     uint cpuCount = 0;
     QFile cpuinfo( "/proc/cpuinfo" );
@@ -404,24 +485,24 @@ void App::fixHyperThreading()
         debug() << "CPU with active HyperThreading detected. Enabling WORKAROUND.\n";
 
         // If the library is new enough try and call sched_setaffinity.
-        #if defined(__GLIBC_PREREQ) && __GLIBC_PREREQ(2,3) && defined(CPU_ZERO) && defined(CPU_SET)
+        #ifdef SCHEDAFFINITY_SUPPORT
         cpu_set_t mask;
         CPU_ZERO( &mask ); // Initializes all the bits in the mask to zero
         CPU_SET( 0, &mask ); // Sets only the bit corresponding to cpu
-        #ifdef SCHEDAFFINITY_SUPPORT
+        #ifdef SCHEDAFFINITY_3PARAMS
         if ( sched_setaffinity( 0, sizeof(mask), &mask ) == -1 )
-        #else  //SCHEDAFFINITY_SUPPORT
+        #else  //SCHEDAFFINITY_3PARAMS
         if ( sched_setaffinity( 0, &mask ) == -1 )
-        #endif //SCHEDAFFINITY_SUPPORT
+        #endif //SCHEDAFFINITY_3PARAMS
         {
             warning() << "sched_setaffinity() call failed with error code: " << errno << endl;
             QTimer::singleShot( 0, this, SLOT( showHyperThreadingWarning() ) );
             return;
         }
-        #else //defined(__GLIBC_PREREQ) && __GLIBC_PREREQ(2,3)
-             warning()<<"glibc too old (<2.3) for sched_setaffinity" << endl;
+        #else //SCHEDAFFINITY_SUPPORT
+             warning()<<"glibc failed checks for sched_setaffinity" << endl;
         QTimer::singleShot( 0, this, SLOT( showHyperThreadingWarning() ) );
-        #endif //defined(__GLIBC_PREREQ) && __GLIBC_PREREQ(2,3)
+        #endif //SCHEDAFFINITY_SUPPORT
     }
     else { debug() << "Fix not enabled" << endl; }
     #else //__linux__
@@ -514,6 +595,7 @@ void App::applySettings( bool firstTime )
             //we need to explicately set the PL button if it's the first time
             if( !firstTime ) m_pPlayerWindow->show();
 
+
             connect( m_pPlayerWindow, SIGNAL(playlistToggled( bool )), m_pPlaylistWindow, SLOT(showHide()) );
 
             //TODO get this to work!
@@ -526,13 +608,16 @@ void App::applySettings( bool firstTime )
             m_pTray->engineStateChanged(EngineController::instance()->engine()->state(), EngineController::instance()->engine()->state());
             m_pTray->engineNewMetaData(EngineController::instance()->bundle(), false);
 
+            //make player window minimal if it was last time
+            if( AmarokConfig::playerWindowMinimalView() ){
+                m_pPlayerWindow->setMinimalView( true );
+            }
         }
         else
             //this is called in the PlayerWindow ctor, hence the else
             m_pPlayerWindow->applySettings();
 
     } else if( m_pPlayerWindow ) {
-
         delete m_pTray; m_pTray = new amaroK::TrayIcon( m_pPlaylistWindow );
         m_pTray->engineStateChanged(EngineController::instance()->engine()->state(), EngineController::instance()->engine()->state());
         m_pTray->engineNewMetaData(EngineController::instance()->bundle(), false);
@@ -540,7 +625,7 @@ void App::applySettings( bool firstTime )
 
         //Set the caption correctly.
         if ( !EngineController::instance()->bundle().prettyTitle().isEmpty() )
-            m_pPlaylistWindow->setCaption( "amaroK - " + EngineController::instance()->bundle().veryNiceTitle() );
+            m_pPlaylistWindow->setCaption( i18n("amaroK - %1").arg( EngineController::instance()->bundle().veryNiceTitle() ) );
         else
             m_pPlaylistWindow->setCaption( "amaroK" );
 
@@ -552,26 +637,14 @@ void App::applySettings( bool firstTime )
            playlistWindow()->createGUI();
     }
 
-
     playlistWindow()->applySettings();
     Scrobbler::instance()->applySettings();
     amaroK::OSD::instance()->applySettings();
     CollectionDB::instance()->applySettings();
-    amaroK::StatusBar::instance()->setShown( AmarokConfig::showStatusBar() );
     m_pTray->setShown( AmarokConfig::showTrayIcon() );
     TrackToolTip::instance()->addToWidget( m_pTray );
 
 
-    // we check > 0 because textCodecForIndex( 0 ) crashes amaroK for unknown
-    // reasons, also now we assign index 0 to "" in the config combobox
-    if( AmarokConfig::recodeID3v1Tags() )
-    if( AmarokConfig::recodeEncoding() > 0 )
-        TagLib::ID3v1::Tag::setStringHandler( new ID3v1StringHandler( AmarokConfig::recodeEncoding() ) );
-    else
-        // otherwise the locale encoding is used
-        TagLib::ID3v1::Tag::setStringHandler( new ID3v1StringHandler( QTextCodec::codecForLocale() ) );
-
-
     //on startup we need to show the window, but only if it wasn't hidden on exit
     //and always if the trayicon isn't showing
     if( firstTime && !amaroK::config()->readBoolEntry( "HiddenOnExit", false ) || !AmarokConfig::showTrayIcon() )
@@ -599,11 +672,16 @@ void App::applySettings( bool firstTime )
         engine->setEqualizerEnabled( AmarokConfig::equalizerEnabled() );
         if ( AmarokConfig::equalizerEnabled() )
             engine->setEqualizerParameters( AmarokConfig::equalizerPreamp(), AmarokConfig::equalizerGains() );
+
+        amaroK::actionCollection()->action("play_audiocd")->setEnabled( EngineController::hasEngineProperty( "HasKIO" ) || EngineController::hasEngineProperty("HasCDDA"));
     } //</Engine>
 
     { //<Collection>
-
+        CollectionView::instance()->renderView(true);
     } //</Collection>
+    { //<Context>
+        ContextBrowser::instance()->renderView();
+    } //</Context>
 
     {   // delete unneeded cover images from cache
         const QString size = QString::number( AmarokConfig::coverPreviewSize() ) + '@';
@@ -623,12 +701,10 @@ void App::applySettings( bool firstTime )
 void
 App::applyColorScheme()
 {
-    DEBUG_BLOCK
-
     QColorGroup group;
     using amaroK::ColorScheme::AltBase;
     int h, s, v;
-    QWidget* const browserBar = (QWidget*)playlistWindow()->child( "BrowserBar" );
+    QWidget* const browserBar = static_cast<QWidget*>( playlistWindow()->child( "BrowserBar" ) );
 
     if( AmarokConfig::schemeKDE() )
     {
@@ -820,7 +896,7 @@ bool amaroK::genericEventHandler( QWidget *recipient, QEvent *e )
                       "Use <B>Quit</B> from the menu, or the amaroK tray-icon to exit the application.</qt>" ),
                 i18n( "Docking in System Tray" ), "hideOnCloseInfo" );
         }
-        else kapp->quit();
+        else pApp->quit();
 
         break;
 
@@ -842,13 +918,14 @@ void App::engineStateChanged( Engine::State state, Engine::State oldState )
             m_pPlaylistWindow->setCaption( kapp->makeStdCaption( i18n("Playlist") ) );
         else m_pPlaylistWindow->setCaption( "amaroK" );
         TrackToolTip::instance()->clear();
+        amaroK::OSD::instance()->setImage( KIconLoader().iconPath( "amarok", -KIcon::SizeHuge ) );
         break;
 
     case Engine::Playing:
         if ( oldState == Engine::Paused )
-            amaroK::OSD::instance()->OSDWidget::show( i18n("Play" ) );
+            amaroK::OSD::instance()->OSDWidget::show( i18n( "state, as in playing", "Play" ) );
         if ( !bundle.prettyTitle().isEmpty() )
-            m_pPlaylistWindow->setCaption( "amaroK - " + bundle.veryNiceTitle() );
+            m_pPlaylistWindow->setCaption( i18n("amaroK - %1").arg( bundle.veryNiceTitle() ) );
         break;
 
     case Engine::Paused:
@@ -870,7 +947,7 @@ void App::engineNewMetaData( const MetaBundle &bundle, bool /*trackChanged*/ )
 {
     amaroK::OSD::instance()->show( bundle );
     if ( !bundle.prettyTitle().isEmpty() )
-        m_pPlaylistWindow->setCaption( "amaroK - " + bundle.veryNiceTitle() );
+        m_pPlaylistWindow->setCaption( i18n("amaroK - %1").arg( bundle.veryNiceTitle() ) );
 
     TrackToolTip::instance()->setTrack( bundle );
 }
@@ -882,9 +959,6 @@ void App::engineTrackPositionChanged( long position, bool /*userSeek*/ )
 
 void App::engineVolumeChanged( int newVolume )
 {
-    //get the amarok icon to show in osd if not playing - else show album cover.
-    QImage osdImage = QImage::QImage();
-    Engine::Playing ? osdImage = QImage::QImage()  :  osdImage = QImage( KIconLoader().iconPath( "amarok", -KIcon::SizeHuge ) );
     amaroK::OSD::instance()->OSDWidget::show( newVolume ? i18n("Volume: %1%").arg( newVolume ) : i18n("Mute") );
 }
 
@@ -897,9 +971,9 @@ void App::slotConfigEqualizer() //SLOT
 
 void App::slotConfigAmarok( const QCString& page )
 {
-    DEBUG_FUNC_INFO
+    DEBUG_THREAD_FUNC_INFO
 
-    AmarokConfigDialog* dialog = (AmarokConfigDialog*) KConfigDialog::exists( "settings" );
+    AmarokConfigDialog* dialog = static_cast<AmarokConfigDialog*>( KConfigDialog::exists( "settings" ) );
 
     if( !dialog )
     {
@@ -946,7 +1020,6 @@ void App::slotConfigToolBars()
     }
 }
 
-
 void App::firstRunWizard()
 {
     ///show firstRunWizard
@@ -967,13 +1040,11 @@ void App::firstRunWizard()
             AmarokConfig::setShowPlayerWindow( true );
             //FIXME the statusbar is now quite essential and also without it
             // the popup messages break. Fix in 1.2.1
-            AmarokConfig::setShowStatusBar( /*false*/ true );
             break;
 
         case FirstRunWizard::Compact:
             amaroK::config()->writeEntry( "XMLFile", "amarokui.rc" );
             AmarokConfig::setShowPlayerWindow( false );
-            AmarokConfig::setShowStatusBar( true );
             break;
         }
 
@@ -985,49 +1056,68 @@ void App::firstRunWizard()
              oldCollectionFolders != AmarokConfig::collectionFolders() )
             CollectionDB::instance()->startScan();
 
+        AmarokConfig::setDatabaseEngine(
+            QString::number( amaroK::databaseTypeCode( wizard.dbSetup7->databaseEngine->currentText() ) ) );
+
         config->updateSettings();
     }
 }
 
-
-void App::pruneCoverImages()
+void App::setUseScores( bool use )
 {
-#ifdef AMAZON_SUPPORT
-    const int MAX_DAYS = 90;
+    AmarokConfig::setUseScores( use );
+    emit useScores( use );
+}
 
-    QDir covers( amaroK::saveLocation( "albumcovers/large/" ) );
-    QDir coverCache( amaroK::saveLocation( "albumcovers/cache/" ) );
+void App::setUseRatings( bool use )
+{
+    AmarokConfig::setUseRatings( use );
+    emit useRatings( use );
+}
 
-    QFileInfoList list( *covers.entryInfoList( QDir::Files ) );
-    QFileInfoList listCache( *coverCache.entryInfoList( QDir::Files ) );
-
-    // Merge both lists
-    for ( uint i = 0; i < listCache.count(); i++ )
-        list.append( listCache.at( i ) );
-
-    // Prune files
-    uint count = 0;
-    uint pruneCount = 0;
-    const QDate currentDate = QDate::currentDate();
-    for ( uint i = 0; i < list.count(); ++i, ++count )
-        if ( list.at( i )->created().date().daysTo( currentDate ) > MAX_DAYS ) {
-            pruneCount++;
-            QFile::remove( list.at( i )->absFilePath() );
-        }
-
-    if ( pruneCount )
-        amaroK::StatusBar::instance()->longMessage( i18n(
-                "amaroK has had to delete some of the images that you downloaded from Amazon. "
-                "Amazon's licensing policy require us to remove content within 90 days of its download. "
-                "You can read more about Amazon's policies "
-                "<a href='http://www.amazon.com/webservices'>here</a>. "
-                "Please note that this only affects covers downloaded using the Cover Manager's fetch covers feature and the covers downloaded using the context menu in the Context Browser, not manually added covers." ), KDE::StatusBar::Information );
+KIO::Job *App::trashFiles( const KURL::List &files )
+{
+#if KDE_IS_VERSION( 3, 3, 91 )
+    KIO::Job *job = KIO::trash( files, true /*show progress*/ );
+    amaroK::StatusBar::instance()->newProgressOperation( job ).setDescription( i18n("Moving files to trash") );
+    connect( job, SIGNAL( result( KIO::Job* ) ), this, SLOT( slotTrashResult( KIO::Job* ) ) );
+    return job;
+#else
+    return 0;
 #endif
 }
 
+void App::setRating( int n )
+{
+    n *= 2;
+
+    if( EngineController::instance()->engine()->state() == Engine::Playing ||
+        EngineController::instance()->engine()->state() == Engine::Paused  ||
+        EngineController::instance()->engine()->state() == Engine::Idle    )
+    {
+        CollectionDB::instance()->setSongRating( EngineController::instance()->playingURL().path(), n );
+        amaroK::OSD::instance()->OSDWidget::show( i18n("Rating: %1").arg( QString().fill( '*', n/2 ) ) );
+    }
+    else if( PlaylistWindow::self()->isReallyShown() && Playlist::instance()->qscrollview()->hasFocus() )
+        Playlist::instance()->setSelectedRatings( n );
+}
+
+void App::slotTrashResult( KIO::Job *job )
+{
+    if( job->error() )
+        job->showErrorDialog( PlaylistWindow::self() );
+}
+
 QWidget *App::mainWindow() const
 {
-   return AmarokConfig::showPlayerWindow() ? (QWidget*)m_pPlayerWindow : (QWidget*)m_pPlaylistWindow;
+    return AmarokConfig::showPlayerWindow() ? static_cast<QWidget*>( m_pPlayerWindow )
+                                            : static_cast<QWidget*>( m_pPlaylistWindow );
+}
+
+void App::quit()
+{
+    emit prepareToQuit();
+    KApplication::quit();
 }
 
 namespace amaroK
@@ -1052,6 +1142,14 @@ namespace amaroK
         return kapp->config();
     }
 
+    bool invokeBrowser( const QString& url )
+    {
+        const QString cmd = "%1 \"%2\"";
+        int pid = KRun::runCommand( cmd.arg( AmarokConfig::externalBrowser(), url ) );
+
+        return pid > 0;
+    }
+
     namespace ColorScheme
     {
         QColor Base;
@@ -1075,6 +1173,106 @@ namespace amaroK
     {
         return KGlobal::dirs()->saveLocation( "data", QString("amarok/") + directory, true );
     }
+
+    QString cleanPath( const QString &path, bool onlyASCII )
+    {
+        QString result = path;
+        // german umlauts
+        result.replace( QChar(0x00e4), "ae" ).replace( QChar(0x00c4), "Ae" );
+        result.replace( QChar(0x00f6), "oe" ).replace( QChar(0x00d6), "Oe" );
+        result.replace( QChar(0x00dc), "ue" ).replace( QChar(0x00fc), "Ue" );
+        result.replace( QChar(0x00df), "ss" );
+
+        // some strange accents
+        result.replace( QChar(0x00e7), "c" ).replace( QChar(0x00c7), "C" );
+        result.replace( QChar(0x00fd), "y" ).replace( QChar(0x00dd), "Y" );
+        result.replace( QChar(0x00f1), "n" ).replace( QChar(0x00d1), "N" );
+
+        // accented vowels
+        QChar a[] = { 'a', 0xe0,0xe1,0xe2,0xe3,0xe5, 0 };
+        QChar A[] = { 'A', 0xc0,0xc1,0xc2,0xc3,0xc5, 0 };
+        QChar E[] = { 'e', 0xe8,0xe9,0xea,0xeb, 0 };
+        QChar e[] = { 'E', 0xc8,0xc9,0xca,0xcb, 0 };
+        QChar i[] = { 'i', 0xec,0xed,0xee,0xef, 0 };
+        QChar I[] = { 'I', 0xcc,0xcd,0xce,0xcf, 0 };
+        QChar o[] = { 'o', 0xf2,0xf3,0xf4,0xf5,0xf8, 0 };
+        QChar O[] = { 'O', 0xd2,0xd3,0xd4,0xd5,0xd8, 0 };
+        QChar u[] = { 'u', 0xf9,0xfa,0xfb, 0 };
+        QChar U[] = { 'U', 0xd9,0xda,0xdb, 0 };
+        QChar nul[] = { 0 };
+        QChar *replacements[] = { a, A, e, E, i, I, o, O, u, U, nul };
+
+        for( uint i = 0; i < result.length(); i++ )
+        {
+            QChar c = result.ref( i );
+            for( uint n = 0; replacements[n][0] != QChar(0); n++ )
+            {
+                for( uint k=0; replacements[n][k] != QChar(0); k++ )
+                {
+                    if( replacements[n][k] == c )
+                    {
+                        c = replacements[n][0];
+                    }
+                }
+            }
+            if( onlyASCII && ( c > QChar(0x7f) || c == QChar(0) ) )
+            {
+                c = '_';
+            }
+            result.ref( i ) = c;
+        }
+        return result;
+    }
+
+    QString vfatPath( const QString &path )
+    {
+        QString s = path;
+
+        for( uint i = 0; i < s.length(); i++ )
+        {
+            QChar c = s.ref( i );
+            if( c < QChar(0x20)
+                    || c=='*' || c=='?' || c=='<' || c=='>'
+                    || c=='|' || c=='"' || c==':' || c=='/'
+                    || c=='\\' )
+                c = '_';
+            s.ref( i ) = c;
+        }
+
+        uint len = s.length();
+        if( len == 3 || (len > 3 && s[3] == '.') )
+        {
+            QString l = s.left(3).lower();
+            if( l=="aux" || l=="con" || l=="nul" || l=="prn" )
+                s = "_" + s;
+        }
+        else if( len == 4 || (len > 4 && s[4] == '.') )
+        {
+            QString l = s.left(3).lower();
+            QString d = s.mid(3,1);
+            if( (l=="com" || l=="lpt") &&
+                    (d=="0" || d=="1" || d=="2" || d=="3" || d=="4" ||
+                     d=="5" || d=="6" || d=="7" || d=="8" || d=="9") )
+                s = "_" + s;
+        }
+
+        while( s.startsWith( "." ) )
+            s = s.mid(1);
+
+        while( s.endsWith( "." ) )
+            s = s.left( s.length()-1 );
+
+        s = s.left(255);
+        len = s.length();
+        if( s[len-1] == ' ' )
+            s[len-1] = '_';
+
+        return s;
+    }
+
+    void setUseScores( bool use ) { App::instance()->setUseScores( use ); }
+    void setUseRatings( bool use ) { App::instance()->setUseRatings( use ); }
+    KIO::Job *trashFiles( const KURL::List &files ) { return App::instance()->trashFiles( files ); }
 }
 
 #include "app.moc"
