@@ -18,13 +18,17 @@
 #ifndef AMAROK_APP_H
 #define AMAROK_APP_H
 
+#include "amarok_export.h"
 #include "engineobserver.h" //baseclass
 #include <kapplication.h>   //baseclass
+#include <kurl.h>
 
 namespace amaroK {
     class TrayIcon;
 }
 
+namespace KIO { class Job; }
+
 class KActionCollection;
 class KConfig;
 class KGlobalAccel;
@@ -32,22 +36,31 @@ class MetaBundle;
 class PlayerWidget;
 class Playlist;
 class PlaylistWindow;
+class DeviceManager;
 
-
-class App : public KApplication, public EngineObserver
+class LIBAMAROK_EXPORT App : public KApplication, public EngineObserver
 {
     Q_OBJECT
     public:
         App();
        ~App();
 
+        static App *instance() { return static_cast<App*>( qApp ); }
+
         static void handleCliArgs();
         static void initCliArgs( int argc, char *argv[] );
 
+        static int mainThreadId;
+
         PlaylistWindow *playlistWindow() const { return m_pPlaylistWindow; }
 
         // FRIENDS ------
         friend class PlaylistWindow; //requires access to applySettings()
+
+    signals:
+        void useScores( bool use );
+        void useRatings( bool use );
+        void prepareToQuit();
     protected: /* for OSD, tray, and dcop */
         void engineStateChanged( Engine::State state, Engine::State oldState = Engine::Empty );
         void engineNewMetaData( const MetaBundle &bundle, bool trackChanged );
@@ -56,6 +69,11 @@ class App : public KApplication, public EngineObserver
 
     private slots:
         void showHyperThreadingWarning();
+        void setRating1() { setRating( 1 ); }
+        void setRating2() { setRating( 2 ); }
+        void setRating3() { setRating( 3 ); }
+        void setRating4() { setRating( 4 ); }
+        void setRating5() { setRating( 5 ); }
 
     public slots:
         void applySettings( bool firstTime = false );
@@ -65,6 +83,13 @@ class App : public KApplication, public EngineObserver
         void slotConfigToolBars();
         void slotConfigEqualizer();
         void firstRunWizard();
+        void setUseScores( bool use );
+        void setUseRatings( bool use );
+        KIO::Job *trashFiles( const KURL::List &files );
+        void quit();
+
+    private slots:
+        void slotTrashResult( KIO::Job *job );
 
     private:
         /** Workaround for HyperThreading CPU's, @see BUG 99199 */
@@ -76,14 +101,14 @@ class App : public KApplication, public EngineObserver
         /** returns the leading window, either playerWindow or playlistWindow */
         QWidget *mainWindow() const;
 
-        /** Removes all covers images older than 90 days, to comply with amazon.com license */
-        void pruneCoverImages();
+        void setRating( int n );
 
         // ATTRIBUTES ------
         KGlobalAccel        *m_pGlobalAccel;
         PlayerWidget        *m_pPlayerWindow;
         PlaylistWindow      *m_pPlaylistWindow;
         amaroK::TrayIcon    *m_pTray;
+        DeviceManager       *m_pDeviceManager;
 };
 
 #define pApp static_cast<App*>(kapp)
