@@ -13,7 +13,9 @@ email                : markey@web.de
  *                                                                         *
  ***************************************************************************/
 
+#include "amarok.h"
 #include "amarokconfig.h"
+#include "collectiondb.h"
 #include "config.h" // Has USE_MYSQL
 #include "configdialog.h"
 #include "contextbrowser.h"
@@ -21,6 +23,8 @@ email                : markey@web.de
 #include "debug.h"
 #include "directorylist.h"
 #include "enginecontroller.h"
+#include "mediabrowser.h"
+#include "mediumpluginmanager.h"
 #include "Options1.h"
 #include "Options2.h"
 #include "Options4.h"
@@ -28,11 +32,10 @@ email                : markey@web.de
 #include "Options7.h"
 #include "Options8.h"
 #include "osd.h"
+#include "playlistwindow.h"
 #include "plugin/pluginconfig.h"
 #include "pluginmanager.h"
-#include "collectiondb.h"
 
-#include <qcombobox.h>
 #include <qgroupbox.h>
 #include <qlabel.h>
 #include <qlayout.h>
@@ -51,8 +54,23 @@ email                : markey@web.de
 #include <kiconloader.h>
 #include <klineedit.h>
 #include <klocale.h>
+#include <kpushbutton.h>
 #include <kstandarddirs.h>
 
+namespace amaroK {
+    int databaseTypeCode( const QString type )
+    {
+        // can't use kconfigxt for the database comboxbox since we need the DBConnection id and not the index
+        int dbType = DbConnection::sqlite;
+        if ( type == "MySQL")
+            dbType = DbConnection::mysql;
+        else if ( type == "Postgresql" )
+            dbType = DbConnection::postgresql;
+        return dbType;
+    }
+}
+
+
 //////////////////////////////////////////////////////////////////////////////////////////
 // PUBLIC
 //////////////////////////////////////////////////////////////////////////////////////////
@@ -72,11 +90,11 @@ AmarokConfigDialog::AmarokConfigDialog( QWidget *parent, const char* name, KConf
     QVBox    *opt6 = new QVBox;
             m_opt7 = new Options7( 0, "Collection" );
     Options8 *opt8 = new Options8( 0, "Scrobbler" );
+    QVBox    *opt9 = new QVBox;
 
     // Sound System
     opt6->setName( "Engine" );
-    opt6->setSpacing( 12 );
-    opt6->setMargin( 11 );
+    opt6->setSpacing( KDialog::spacingHint() );
     QWidget *groupBox, *aboutEngineButton;
     groupBox            = new QGroupBox( 2, Qt::Horizontal, i18n("Sound System"), opt6 );
     m_engineConfigFrame = new QGroupBox( 1, Qt::Horizontal, opt6 );
@@ -86,23 +104,21 @@ AmarokConfigDialog::AmarokConfigDialog( QWidget *parent, const char* name, KConf
     QToolTip::add( m_soundSystem, i18n("Click to select the sound system to use for playback.") );
     QToolTip::add( aboutEngineButton, i18n("Click to get the plugin information.") );
 
+    /// Populate the engine selection combo box
     KTrader::OfferList offers = PluginManager::query( "[X-KDE-amaroK-plugintype] == 'engine'" );
     KTrader::OfferList::ConstIterator end( offers.end() );
     for( KTrader::OfferList::ConstIterator it = offers.begin(); it != end; ++it ) {
+        // Don't list the <no engine> (void engine) entry if it's not currenty active,
+        // cause there's no point in choosing this engine (it's a dummy, after all).
+        if( (*it)->property( "X-KDE-amaroK-name" ).toString() == "void-engine"
+            && AmarokConfig::soundSystem() != "void-engine" ) continue;
+
         m_soundSystem->insertItem( (*it)->name() );
         // Save name properties in QMap for lookup
         m_pluginName[(*it)->name()] = (*it)->property( "X-KDE-amaroK-name" ).toString();
         m_pluginAmarokName[(*it)->property( "X-KDE-amaroK-name" ).toString()] = (*it)->name();
     }
 
-    // KConfigXT doesn't like Comboboxes, still, 2 versions after
-    // it was created. How shit. Hence we can't make recodeEncoding
-    // a string (which we need to because by index is not consistent)
-    QTextCodec *codec;
-    m_opt1->kcfg_RecodeEncoding->insertItem( i18n("Locale Encoding") );
-    for( int i = 1; (codec = QTextCodec::codecForIndex( i )); i++ )
-        m_opt1->kcfg_RecodeEncoding->insertItem( codec->name() );
-
     // Collection
 #if !defined(USE_MYSQL) && !defined(USE_POSTGRESQL)
     m_opt7->databaseBox->hide();
@@ -114,19 +130,44 @@ AmarokConfigDialog::AmarokConfigDialog( QWidget *parent, const char* name, KConf
     //type="string" also fixes this bug, but means the password is stored in plain
     //text. This is a temporary fix so that the majority of users get a fixed Apply
     //button.
-    delete m_opt7->dbSetupFrame->kcfg_MySqlPassword;
+    delete m_opt7->dbSetupFrame->kcfg_MySqlPassword2;
 #endif
     m_opt7->collectionFoldersBox->setColumns( 1 );
     new CollectionSetup( m_opt7->collectionFoldersBox ); //TODO this widget doesn't update the apply/ok buttons
 
+    // Media Devices
+    opt9->setName( "Media Devices" );
+    opt9->setSpacing( KDialog::spacingHint() );
+    QVBox *topbox = new QVBox( opt9 );
+    topbox->setSpacing( KDialog::spacingHint() );
+    QGroupBox *mediaBox  = new QGroupBox( 2, Qt::Horizontal, i18n("Media Devices"), topbox );
+    mediaBox->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Minimum );
+    QVBox *vbox = new QVBox( mediaBox );
+    vbox->setSpacing( KDialog::spacingHint() );
+    m_deviceManager = new MediumPluginManager( vbox );
+
+    QHBox *hbox = new QHBox( topbox );
+    hbox->setSpacing( KDialog::spacingHint() );
+    hbox->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Minimum );
+    KPushButton *autodetect = new KPushButton( i18n( "Autodetect Devices" ), hbox );
+    autodetect->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed ) );
+    connect( autodetect, SIGNAL(clicked()), m_deviceManager, SLOT(redetectDevices()) );
+    KPushButton *add = new KPushButton( i18n( "Add Device..." ), hbox );
+    add->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed ) );
+    connect( add, SIGNAL(clicked()), m_deviceManager, SLOT(newDevice()) );
+
+    QFrame *frame = new QFrame( topbox );
+    frame->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
+
     // add pages
     addPage( m_opt1, i18n( "General" ), "misc", i18n( "Configure General Options" ) );
-    addPage( m_opt2,   i18n( "Appearance" ), "colors", i18n( "Configure amaroK's Appearance" ) );
+    addPage( m_opt2, i18n( "Appearance" ), "colors", i18n( "Configure amaroK's Appearance" ) );
     addPage( m_opt4, i18n( "Playback" ), "kmix", i18n( "Configure Playback" ) );
     addPage( opt5,   i18n( "OSD" ), "tv", i18n( "Configure On-Screen-Display" ) );
     addPage( opt6,   i18n( "Engine" ), "amarok", i18n( "Configure Engine" ) );
-    addPage( m_opt7, i18n( "Collection" ), "collection", i18n( "Configure Collection" ) );
+    addPage( m_opt7, i18n( "Collection" ), amaroK::icon( "collection" ), i18n( "Configure Collection" ) );
     addPage( opt8,   i18n( "last.fm" ), "audioscrobbler", i18n( "Configure last.fm Support" ) );
+    addPage( opt9,   i18n( "Media Devices" ), amaroK::icon( "device" ), i18n( "Configure Portable Player Support" ) );
 
     // Show information labels (must be done after insertions)
     QObjectList *list = queryList( "QLabel", "infoPixmap" );
@@ -140,15 +181,20 @@ AmarokConfigDialog::AmarokConfigDialog( QWidget *parent, const char* name, KConf
         static_cast<QLabel*>(label)->setMaximumWidth( 250 );
     delete list;
 
+    connect( m_deviceManager, SIGNAL(changed()), SLOT(updateButtons()) );
     connect( m_soundSystem, SIGNAL(activated( int )), SLOT(updateButtons()) );
     connect( aboutEngineButton, SIGNAL(clicked()), SLOT(aboutEngine()) );
     connect( opt5, SIGNAL(settingsChanged()), SLOT(updateButtons()) ); //see options5.ui.h
     connect( m_opt2->styleComboBox, SIGNAL( activated( int ) ), SLOT( updateButtons() ) );
+    connect( m_opt7->dbSetupFrame->databaseEngine, SIGNAL( activated( int ) ), SLOT( updateButtons() ) );
+    connect( m_opt1->kComboBox_browser, SIGNAL( activated( int ) ), SLOT( updateButtons() ) );
+    connect( m_opt1->kLineEdit_customBrowser, SIGNAL( textChanged( const QString& ) ), SLOT( updateButtons() ) );
 }
 
 AmarokConfigDialog::~AmarokConfigDialog()
 {
     delete m_engineConfig;
+    delete m_deviceManager;
 }
 
 
@@ -178,6 +224,16 @@ void AmarokConfigDialog::showPage( const QCString& page )
 // PROTECTED SLOTS
 //////////////////////////////////////////////////////////////////////////////////////////
 
+/**
+ * Update the buttons.
+ * REIMPLEMENTED
+ */
+
+void AmarokConfigDialog::updateButtons()
+{
+    KConfigDialog::updateButtons();
+}
+
 /**
  * Update the settings from the dialog.
  * Example use: User clicks Ok or Apply button in a configure dialog.
@@ -185,7 +241,7 @@ void AmarokConfigDialog::showPage( const QCString& page )
  */
 void AmarokConfigDialog::updateSettings()
 {
-    OSDPreviewWidget *osd = (OSDPreviewWidget*)child( "osdpreview" );
+    OSDPreviewWidget *osd = static_cast<OSDPreviewWidget*>( child( "osdpreview" ) );
     AmarokConfig::setOsdAlignment( osd->alignment() );
     AmarokConfig::setOsdYOffset( osd->y() );
     amaroK::OSD::instance()->applySettings();
@@ -194,6 +250,8 @@ void AmarokConfigDialog::updateSettings()
 
     if ( m_engineConfig ) m_engineConfig->save();
 
+    AmarokConfig::setExternalBrowser( externalBrowser() );
+
     // When sound system has changed, update engine config page
     if ( m_soundSystem->currentText() != m_pluginAmarokName[AmarokConfig::soundSystem()] ) {
         AmarokConfig::setSoundSystem( m_pluginName[m_soundSystem->currentText()] );
@@ -204,22 +262,24 @@ void AmarokConfigDialog::updateSettings()
     if ( m_opt2->styleComboBox->currentText() != AmarokConfig::contextBrowserStyleSheet() ) {
         //can't use kconfigxt for the style comboxbox's since we need the string, not the index
         AmarokConfig::setContextBrowserStyleSheet( m_opt2->styleComboBox->currentText() );
-        ContextBrowser::instance()->setStyleSheet();
+        ContextBrowser::instance()->reloadStyleSheet();
     }
 
-    // can't use kconfigxt for the database comboxbox since we need the DBConnection id and not the index
-    QString dbType = QString::number(DbConnection::sqlite);
-    if (m_opt7->dbSetupFrame->databaseEngine->currentText() == "MySQL")
+    int dbType = amaroK::databaseTypeCode( m_opt7->dbSetupFrame->databaseEngine->currentText() );
+    if ( dbType != AmarokConfig::databaseEngine().toInt() ) {
+        AmarokConfig::setDatabaseEngine( QString::number( dbType ) );
+        emit settingsChanged();
+    }
+
+    m_deviceManager->finished();
+
+    if( MediaBrowser::isAvailable() )
     {
-        dbType = QString::number(DbConnection::mysql);
-    }
-    else if (m_opt7->dbSetupFrame->databaseEngine->currentText() == "Postgresql")
-    {
-        dbType = QString::number(DbConnection::postgresql);
-    }
-    if ( dbType != AmarokConfig::databaseEngine() ) {
-        AmarokConfig::setDatabaseEngine( dbType );
+        PlaylistWindow::self()->addBrowser( "MediaBrowser", MediaBrowser::instance(), i18n( "Media Device" ), amaroK::icon( "device" ) );
     }
+
+    amaroK::setUseScores( m_opt1->kcfg_UseScores->isChecked() );
+    amaroK::setUseRatings( m_opt1->kcfg_UseRatings->isChecked() );
 }
 
 
@@ -258,13 +318,16 @@ void AmarokConfigDialog::updateWidgetsDefault()
  */
 bool AmarokConfigDialog::hasChanged()
 {
-    OSDPreviewWidget *osd = (OSDPreviewWidget*) child( "osdpreview" );
+    OSDPreviewWidget *osd = static_cast<OSDPreviewWidget*>( child( "osdpreview" ) );
 
     return  m_soundSystem->currentText() != m_pluginAmarokName[AmarokConfig::soundSystem()] ||
             osd->alignment() != AmarokConfig::osdAlignment() ||
             osd->alignment() != OSDWidget::Center && osd->y() != AmarokConfig::osdYOffset() ||
             m_opt2->styleComboBox->currentText() != AmarokConfig::contextBrowserStyleSheet() ||
-            m_engineConfig && m_engineConfig->hasChanged();
+            amaroK::databaseTypeCode(  m_opt7->dbSetupFrame->databaseEngine->currentText()  ) != AmarokConfig::databaseEngine().toInt() ||
+            m_engineConfig && m_engineConfig->hasChanged() ||
+            m_deviceManager && m_deviceManager->hasChanged() ||
+            externalBrowser() != AmarokConfig::externalBrowser();
 }
 
 
@@ -323,14 +386,22 @@ void AmarokConfigDialog::soundSystemChanged()
     m_opt4->kcfg_Crossfade->setEnabled( hasCrossfade );
     m_opt4->kcfg_CrossfadeLength->setEnabled( hasCrossfade && crossfadeOn );
     m_opt4->crossfadeLengthLabel->setEnabled( hasCrossfade && crossfadeOn );
-    //will enable after 1.3.4 string freeze is over
-    //m_opt4->kcfg_CrossfadeManualOnly->setEnabled( hasCrossfade && crossfadeOn );
+    m_opt4->kcfg_CrossfadeManualOnly->setEnabled( hasCrossfade && crossfadeOn );
 
     if (!hasCrossfade)
     {
         m_opt4->radioButtonNormalPlayback->setChecked( true );
     }
-
 }
 
+QString AmarokConfigDialog::externalBrowser() const
+{
+    return   m_opt1->kComboBox_browser->isEnabled() ?
+                 m_opt1->kComboBox_browser->currentText() == i18n( "Default KDE Browser" ) ?
+                 "kfmclient openURL" :
+                 m_opt1->kComboBox_browser->currentText().lower() :
+             m_opt1->kLineEdit_customBrowser->text().lower();
+}
+
+
 #include "configdialog.moc"
