@@ -1,20 +1,22 @@
 // (c) Pierpaolo Di Panfilo 2004
+// (c) 2005 Isaiah Damron <xepo@trifault.net>
 // See COPYING file for licensing information
 
 #include "amarok.h"
 #include "amarokconfig.h"
 #include "browserToolBar.h"
 #include "clicklineedit.h"
+#include "debug.h"
+#include "collectionbrowser.h" //manipulateThe()
 #include "collectiondb.h"
 #include "config.h"
 #include "coverfetcher.h"
 #include "covermanager.h"
-#include "debug.h"
+#include "pixmapviewer.h"
 
 #include <qdesktopwidget.h>  //ctor: desktop size
 #include <qfile.h>
 #include <qfontmetrics.h>    //paintItem()
-#include <qhbox.h>
 #include <qimage.h>
 #include <qlabel.h>
 #include <qlayout.h>
@@ -26,6 +28,7 @@
 #include <qprogressdialog.h>
 #include <qrect.h>
 #include <qstringlist.h>
+#include <qtooltip.h>
 #include <qtimer.h>    //search filter timer
 #include <qtooltip.h>
 #include <qvbox.h>
@@ -46,11 +49,12 @@
 #include <ksqueezedtextlabel.h> //status label
 #include <kstandarddirs.h>   //KGlobal::dirs()
 #include <kstatusbar.h>
+#include <kstringhandler.h>    //paintItem
 #include <ktoolbar.h>
 #include <ktoolbarbutton.h>    //clear filter button
 #include <kurl.h>
 #include <kurldrag.h>
-
+#include <kwin.h>
 
 static QString artistToSelectInInitFunction;
 CoverManager *CoverManager::s_instance = 0;
@@ -70,23 +74,42 @@ CoverManager::CoverManager()
 
     // Sets caption and icon correctly (needed e.g. for GNOME)
     kapp->setTopWidget( this );
-    this->setCaption( kapp->makeStdCaption( i18n("Cover Manager") ) );
-    this->setWFlags( WDestructiveClose );
-    this->setMargin( 4 );
+    setCaption( kapp->makeStdCaption( i18n("Cover Manager") ) );
+    setWFlags( WDestructiveClose );
+    setMargin( 4 );
 
     //artist listview
     m_artistView = new KListView( this );
     m_artistView->addColumn(i18n( "Albums By" ));
     m_artistView->setFullWidth( true );
-    m_artistView->setSorting( -1 );    //no sort
+    m_artistView->setSorting( 0 );
     m_artistView->setMinimumWidth( 180 );
-    KListViewItem *item = new KListViewItem( m_artistView, i18n( "All Albums" ) );
-    item->setPixmap( 0, SmallIcon("cdrom_unmount") );
+    KListViewItem *item = 0;
 
     //load artists from the collection db
     const QStringList artists = CollectionDB::instance()->artistList( false, false );
-    foreach( artists )  {
-        item = new KListViewItem( m_artistView, item, *it );
+    foreach( artists )
+    {
+        QString artist = *it;
+        if( artist.startsWith( "The " ) )
+            CollectionView::instance()->manipulateThe( artist, true );
+
+        item = new KListViewItem( m_artistView, item, artist );
+        item->setPixmap( 0, SmallIcon("personal") );
+    }
+    m_artistView->sort();
+
+    m_artistView->setSorting( -1 );
+    KListViewItem *last = item;
+    item = new KListViewItem( m_artistView, 0, i18n( "All Albums" ) );
+    item->setPixmap( 0, SmallIcon("cdrom_unmount") );
+
+    QueryBuilder qb;
+    qb.addReturnValue( QueryBuilder::tabAlbum, QueryBuilder::valName );
+    qb.setOptions( QueryBuilder::optOnlyCompilations | QueryBuilder::optRemoveDuplicates );
+    qb.setLimit( 0, 1 );
+    if ( qb.run().count() ) {
+        item = new KListViewItem( m_artistView, last, i18n( "Various Artists" ) );
         item->setPixmap( 0, SmallIcon("personal") );
     }
 
@@ -100,7 +123,7 @@ CoverManager::CoverManager()
         QHBox *searchBox = new QHBox( hbox );
         KToolBar* searchToolBar = new Browser::ToolBar( searchBox );
         KToolBarButton *button = new KToolBarButton( "locationbar_erase", 0, searchToolBar );
-        m_searchEdit = new ClickLineEdit( i18n( "Filter here..." ), searchToolBar );
+        m_searchEdit = new ClickLineEdit( i18n( "Enter search terms here" ), searchToolBar );
         m_searchEdit->setFrame( QFrame::Sunken );
 
         searchToolBar->setStretchableWidget( m_searchEdit );
@@ -123,11 +146,12 @@ CoverManager::CoverManager()
     #ifdef AMAZON_SUPPORT
     // amazon locale menu
     m_amazonLocaleMenu = new KPopupMenu( this );
-    m_amazonLocaleMenu->insertItem( i18n("International"), International );
-    m_amazonLocaleMenu->insertItem( i18n("France"), France );
-    m_amazonLocaleMenu->insertItem( i18n("Germany"), Germany );
-    m_amazonLocaleMenu->insertItem( i18n("Japan"), Japan);
-    m_amazonLocaleMenu->insertItem( i18n("United Kingdom"), UK );
+    m_amazonLocaleMenu->insertItem( i18n("International"), CoverFetcher::International );
+    m_amazonLocaleMenu->insertItem( i18n("Canada"), CoverFetcher::Canada );
+    m_amazonLocaleMenu->insertItem( i18n("France"), CoverFetcher::France );
+    m_amazonLocaleMenu->insertItem( i18n("Germany"), CoverFetcher::Germany );
+    m_amazonLocaleMenu->insertItem( i18n("Japan"), CoverFetcher::Japan);
+    m_amazonLocaleMenu->insertItem( i18n("United Kingdom"), CoverFetcher::UK );
     connect( m_amazonLocaleMenu, SIGNAL( activated(int) ), SLOT( changeLocale(int) ) );
     #endif
 
@@ -140,17 +164,7 @@ CoverManager::CoverManager()
     toolBar->insertButton( babelfish, 2, m_amazonLocaleMenu, true, i18n( "Amazon Locale" ) );
 
     QString locale = AmarokConfig::amazonLocale();
-
-         if( locale == "fr" ) m_currentLocale = France;
-    else if( locale == "de" ) m_currentLocale = Germany;
-    else if( locale == "jp" ) m_currentLocale = Japan;
-    else if( locale == "uk" ) m_currentLocale = UK;
-    else {
-        // make sure we handle old config files correctly
-        locale = "us";
-        m_currentLocale = International;
-    }
-
+    m_currentLocale = CoverFetcher::localeStringToID( locale );
     m_amazonLocaleMenu->setItemChecked( m_currentLocale, true );
 
     //fetch missing covers button
@@ -178,16 +192,24 @@ CoverManager::CoverManager()
 
 
     // signals and slots connections
-    connect( m_artistView, SIGNAL(selectionChanged( QListViewItem* ) ), SLOT(slotArtistSelected( QListViewItem* )) );
-    connect( m_coverView, SIGNAL(rightButtonPressed( QIconViewItem*, const QPoint& )), SLOT(showCoverMenu( QIconViewItem*, const QPoint& )) );
-    connect( m_coverView, SIGNAL(executed( QIconViewItem* )), SLOT(coverItemExecuted( QIconViewItem* )) );
-    connect( m_timer, SIGNAL(timeout()), SLOT(slotSetFilter()) );
-    connect( m_searchEdit, SIGNAL(textChanged( const QString& )), SLOT(slotSetFilterTimeout()) );
+    connect( m_artistView, SIGNAL(selectionChanged( QListViewItem* ) ),
+                           SLOT(slotArtistSelected( QListViewItem* )) );
+    connect( m_coverView,  SIGNAL(rightButtonPressed( QIconViewItem*, const QPoint& )),
+                           SLOT(showCoverMenu( QIconViewItem*, const QPoint& )) );
+    connect( m_coverView,  SIGNAL(executed( QIconViewItem* )),
+                           SLOT(coverItemExecuted( QIconViewItem* )) );
+    connect( m_timer,      SIGNAL(timeout()),
+                           SLOT(slotSetFilter()) );
+    connect( m_searchEdit, SIGNAL(textChanged( const QString& )),
+                           SLOT(slotSetFilterTimeout()) );
 
     #ifdef AMAZON_SUPPORT
-    connect( CollectionDB::instance(), SIGNAL(coverFetched( const QString&, const QString& )), SLOT(coverFetched( const QString&, const QString& )) );
-    connect( CollectionDB::instance(), SIGNAL(coverRemoved( const QString&, const QString& )), SLOT(coverRemoved( const QString&, const QString& )) );
-    connect( CollectionDB::instance(), SIGNAL(coverFetcherError( const QString& )), SLOT(coverFetcherError()) );
+    connect( CollectionDB::instance(), SIGNAL(coverFetched( const QString&, const QString& )),
+                                       SLOT(coverFetched( const QString&, const QString& )) );
+    connect( CollectionDB::instance(), SIGNAL(coverRemoved( const QString&, const QString& )),
+                                       SLOT(coverRemoved( const QString&, const QString& )) );
+    connect( CollectionDB::instance(), SIGNAL(coverFetcherError( const QString& )),
+                                       SLOT(coverFetcherError()) );
     #endif
 
     m_currentView = AllAlbums;
@@ -229,22 +251,30 @@ void CoverManager::init()
 }
 
 
+CoverViewDialog::CoverViewDialog(const QString& artist, const QString& album, QWidget *parent)
+    : QDialog(parent, 0, false, WDestructiveClose | WType_TopLevel | WNoAutoErase)
+    , m_pixmap(CollectionDB::instance()->albumImage( artist, album, 0 ))
+{
+    KWin::setType( winId(), NET::Utility );
+    kapp->setTopWidget( this );
+    setCaption( kapp->makeStdCaption( i18n("%1 - %2").arg( artist, album ) ) );
+
+    m_layout = new QHBoxLayout( this );
+    m_layout->setAutoAdd(true);
+    m_pixmapViewer = new PixmapViewer( this, m_pixmap );
+
+    setFixedSize( m_pixmapViewer->maximalSize() );
+}
+
+
 void CoverManager::viewCover( const QString& artist, const QString& album, QWidget *parent ) //static
 {
     //QDialog means "escape" works as expected
-    QDialog *dialog = new QDialog( parent, 0, false, WDestructiveClose | WType_TopLevel );
-    kapp->setTopWidget( dialog );
-    dialog->setCaption( kapp->makeStdCaption( artist + " - " + album ) );
-
-    QPixmap pixmap( CollectionDB::instance()->albumImage( artist, album, 0 ) );
-    QLabel* label = new QLabel( dialog );
-    label->setFrameShape( QFrame::NoFrame );
-    dialog->setFixedSize( pixmap.size() );
-    label->setFixedSize( pixmap.size() );
-    label->setPixmap( pixmap );
-
+    QDialog *dialog = new CoverViewDialog( artist, album, parent );
     dialog->show();
 }
+
+
 QString CoverManager::amazonTld() //static
 {
     if (AmarokConfig::amazonLocale() == "us")
@@ -253,10 +283,13 @@ QString CoverManager::amazonTld() //static
         return "co.jp";
             else if (AmarokConfig::amazonLocale() == "uk")
         return "co.uk";
+            else if (AmarokConfig::amazonLocale() == "ca")
+        return "ca";
             else
         return AmarokConfig::amazonLocale();
 }
 
+
 void CoverManager::fetchMissingCovers() //SLOT
 {
     #ifdef AMAZON_SUPPORT
@@ -323,12 +356,23 @@ void CoverManager::slotArtistSelected( QListViewItem *item ) //SLOT
     if( item->depth() ) //album item
         return;
 
+    QString artist = item->text(0);
+
+    if( artist.endsWith( ", The" ) )
+        CollectionView::instance()->manipulateThe( artist, false );
+
     m_coverView->clear();
     m_coverItems.clear();
 
+    // reset current view mode state to "AllAlbum" which is the default on artist change in left panel
+    m_currentView = AllAlbums;
+    m_viewMenu->setItemChecked( AllAlbums, true );
+    m_viewMenu->setItemChecked( AlbumsWithCover, false );
+    m_viewMenu->setItemChecked( AlbumsWithoutCover, false );
+
     QProgressDialog progress( this, 0, true );
     progress.setLabelText( i18n("Loading Thumbnails...") );
-    progress.QDialog::setCaption( "..." );
+    progress.QDialog::setCaption( i18n("...") );
 
     //NOTE we MUST show the dialog, otherwise the closeEvents get processed
     // in the processEvents() calls below, GRUMBLE! Qt sux0rs
@@ -353,17 +397,17 @@ void CoverManager::slotArtistSelected( QListViewItem *item ) //SLOT
     qb.setOptions( QueryBuilder::optNoCompilations );
 
     if ( item != m_artistView->firstChild() )
-        qb.addMatches( QueryBuilder::tabArtist, item->text( 0 ) );
+        qb.addMatches( QueryBuilder::tabArtist, artist );
 
     albums = qb.run();
 
-    //also retrieve compilations when we're showing all items (first treenode)
-    if ( item == m_artistView->firstChild() )
+    //also retrieve compilations when we're showing all items (first treenode) or
+    //"Various Artists" (last treenode)
+    if ( item == m_artistView->firstChild() || item == m_artistView->lastChild() )
     {
         QStringList cl;
 
         qb.clear();
-        qb.addReturnValue( QueryBuilder::tabDummy, QueryBuilder::valDummy );
         qb.addReturnValue( QueryBuilder::tabAlbum,  QueryBuilder::valName );
 
         qb.excludeMatch( QueryBuilder::tabAlbum, i18n( "Unknown" ) );
@@ -372,8 +416,10 @@ void CoverManager::slotArtistSelected( QListViewItem *item ) //SLOT
         qb.setOptions( QueryBuilder::optOnlyCompilations );
         cl = qb.run();
 
-        for ( uint i = 0; i < cl.count(); i++ )
+        for ( uint i = 0; i < cl.count(); i++ ) {
+            albums.append( i18n( "Various Artists" ) );
             albums.append( cl[ i ] );
+        }
     }
 
     QApplication::restoreOverrideCursor();
@@ -446,7 +492,7 @@ void CoverManager::showCoverMenu( QIconViewItem *item, const QPoint &p ) //SLOT
         menu.insertItem( SmallIconSet("editdelete"), i18n("&Unset Cover Image"), DELETE );
 
         menu.setItemEnabled( SHOW, item->hasCover() );
-        menu.setItemEnabled( DELETE, item->hasCover() );
+        menu.setItemEnabled( DELETE, item->canRemoveCover() );
     }
 
     switch( menu.exec(p) ) {
@@ -476,7 +522,6 @@ void CoverManager::showCoverMenu( QIconViewItem *item, const QPoint &p ) //SLOT
     #undef item
 }
 
-
 void CoverManager::coverItemExecuted( QIconViewItem *item ) //SLOT
 {
     #define item static_cast<CoverViewItem*>(item)
@@ -508,7 +553,7 @@ void CoverManager::slotSetFilter() //SLOT
     m_coverView->setAutoArrange( false );
     for( QIconViewItem *item = m_coverItems.first(); item; item = m_coverItems.next() ) {
         CoverViewItem *coverItem = static_cast<CoverViewItem*>(item);
-        if( coverItem->album().contains( m_filter, FALSE ) || coverItem->artist().contains( m_filter, FALSE ) )
+        if( coverItem->album().contains( m_filter, false ) || coverItem->artist().contains( m_filter, false ) )
             m_coverView->insertItem( item, m_coverView->lastItem() );
     }
     m_coverView->setAutoArrange( true );
@@ -543,7 +588,7 @@ void CoverManager::changeView( int id  ) //SLOT
         bool show = false;
         CoverViewItem *coverItem = static_cast<CoverViewItem*>(item);
         if( !m_filter.isEmpty() ) {
-            if( !coverItem->album().contains( m_filter, FALSE ) && !coverItem->artist().contains( m_filter, FALSE ) )
+            if( !coverItem->album().contains( m_filter, false ) && !coverItem->artist().contains( m_filter, false ) )
                 continue;
         }
 
@@ -567,24 +612,8 @@ void CoverManager::changeView( int id  ) //SLOT
 
 void CoverManager::changeLocale( int id ) //SLOT
 {
-    switch ( id )
-    {
-        case International:
-            AmarokConfig::setAmazonLocale( "us" );
-            break;
-        case France:
-            AmarokConfig::setAmazonLocale( "fr" );
-            break;
-        case Germany:
-            AmarokConfig::setAmazonLocale( "de" );
-            break;
-        case Japan:
-            AmarokConfig::setAmazonLocale( "jp" );
-            break;
-        case UK:
-            AmarokConfig::setAmazonLocale( "uk" );
-            break;
-    }
+    QString locale = CoverFetcher::localeIDToString( id );
+    AmarokConfig::setAmazonLocale( locale );
     m_amazonLocaleMenu->setItemChecked( m_currentLocale, false );
     m_amazonLocaleMenu->setItemChecked( id, true );
     m_currentLocale = id;
@@ -702,7 +731,7 @@ void CoverManager::deleteSelectedCovers()
                                   "Are you sure you want to delete these %n covers from the Collection?",
                                   selected.count() ),
                             QString::null,
-                            i18n("&Remove") );
+                            KStdGuiItem::del() );
 
     if ( button == KMessageBox::Continue ) {
         for ( CoverViewItem* item = selected.first(); item; item = selected.next() ) {
@@ -754,9 +783,10 @@ void CoverManager::updateStatusBar()
             QStringList values = QStringList::split( " @@@ ", m_fetchCovers[0], true );    //get artist and album name
             if ( values.count() >= 2 )
             {
-                text = i18n( "Fetching cover for " );
-                if ( !values[0].isEmpty() ) text += values[0] + " - ";
-                text += values[1] + "...";
+                if( values[0].isEmpty() )
+                    text = i18n( "Fetching cover for %1..." ).arg( values[1] );
+                else
+                    text = i18n( "Fetching cover for %1 - %2..." ).arg( values[0], values[1] );
             }
         }
         else if( m_fetchingCovers ) {
@@ -764,7 +794,7 @@ void CoverManager::updateStatusBar()
             if( m_coversFetched )
                 text += i18n( "1 fetched", "%n fetched", m_coversFetched );
             if( m_coverErrors ) {
-                if( m_coversFetched ) text += " - ";
+            if( m_coversFetched ) text += i18n(" - ");
                 text += i18n( "1 not found", "%n not found", m_coverErrors );
             }
             if( m_coversFetched + m_coverErrors == 0 )
@@ -783,7 +813,7 @@ void CoverManager::updateStatusBar()
         //album info
         for( QIconViewItem *item = m_coverView->firstItem(); item; item = item->nextItem() ) {
             totalCounter++;
-            if( !((CoverViewItem*)item)->hasCover() )
+            if( !static_cast<CoverViewItem*>( item )->hasCover() )
                 missingCounter++;    //counter for albums without cover
         }
 
@@ -792,7 +822,12 @@ void CoverManager::updateStatusBar()
         else if( m_artistView->selectedItem() ) {
             text = i18n( "1 album", "%n albums", totalCounter );
             if( m_artistView->selectedItem() != m_artistView->firstChild() ) //showing albums by an artist
-                text += i18n( " by " ) + m_artistView->selectedItem()->text(0);
+            {
+                QString artist = m_artistView->selectedItem()->text(0);
+                if( artist.endsWith( ", The" ) )
+                    CollectionView::instance()->manipulateThe( artist, false );
+                text += i18n( " by " ) + artist;
+            }
         }
 
         if( missingCounter )
@@ -806,6 +841,12 @@ void CoverManager::updateStatusBar()
     m_statusLabel->setText( text );
 }
 
+void CoverManager::setStatusText( QString text )
+{
+    m_oldStatusText = m_statusLabel->text();
+    m_statusLabel->setText( text );
+}
+
 //////////////////////////////////////////////////////////////////////
 //    CLASS CoverView
 /////////////////////////////////////////////////////////////////////
@@ -821,6 +862,13 @@ CoverView::CoverView( QWidget *parent, const char *name, WFlags f )
     arrangeItemsInGrid();
     setAutoArrange( true );
     setItemsMovable( false );
+
+    // as long as QIconView only shows tooltips when the cursor is over the
+    // icon (and not the text), we have to create our own tooltips
+    setShowToolTips( false );
+
+    connect( this, SIGNAL( onItem( QIconViewItem * ) ), SLOT( setStatusText( QIconViewItem * ) ) );
+    connect( this, SIGNAL( onViewport() ), CoverManager::instance(), SLOT( updateStatusBar() ) );
 }
 
 
@@ -830,8 +878,8 @@ QDragObject *CoverView::dragObject()
     if( !item )
        return 0;
 
-    const QString sql = "SELECT tags.url FROM tags, album WHERE album.name LIKE '%1' AND tags.album = album.id ORDER BY tags.track;";
-    const QStringList values = CollectionDB::instance()->query( sql.arg( item->album() ) );
+    const QString sql = "SELECT tags.url FROM tags, album WHERE album.name %1 AND tags.album = album.id ORDER BY tags.track;";
+    const QStringList values = CollectionDB::instance()->query( sql.arg( CollectionDB::likeCondition( item->album() ) ) );
 
     KURL::List urls;
     for( QStringList::ConstIterator it = values.begin(), end = values.end(); it != end; ++it )
@@ -847,15 +895,34 @@ QDragObject *CoverView::dragObject()
     return drag;
 }
 
+void CoverView::setStatusText( QIconViewItem *item )
+{
+    #define item static_cast<CoverViewItem *>( item )
+    if ( !item )
+        return;
+
+    bool sampler = false;
+    //compilations have valDummy for artist.  see QueryBuilder::addReturnValue(..) for explanation
+    //FIXME: Don't rely on other independent code, use an sql query
+    if( item->artist().isEmpty() ) sampler = true;
+
+    QString tipContent = i18n( "%1 - %2" ).arg( sampler ? i18n("Various Artists") : item->artist() )
+                                          .arg( item->album() );
+
+    CoverManager::instance()->setStatusText( tipContent );
+
+    #undef item
+}
+
 //////////////////////////////////////////////////////////////////////
 //    CLASS CoverViewItem
 /////////////////////////////////////////////////////////////////////
 
-CoverViewItem::CoverViewItem( QIconView *parent, QIconViewItem *after, QString artist, QString album )
+CoverViewItem::CoverViewItem( QIconView *parent, QIconViewItem *after, const QString &artist, const QString &album )
     : KIconViewItem( parent, after, album )
     , m_artist( artist )
     , m_album( album )
-    , m_coverImagePath( CollectionDB::instance()->albumImage( m_artist, m_album, 0 ) )
+    , m_coverImagePath( CollectionDB::instance()->albumImage( m_artist, m_album, 0, &m_embedded ) )
     , m_coverPixmap( 0 )
 {
     setDragEnabled( true );
@@ -870,7 +937,7 @@ bool CoverViewItem::hasCover() const
 
 void CoverViewItem::loadCover()
 {
-    m_coverImagePath = CollectionDB::instance()->albumImage( m_artist, m_album );
+    m_coverImagePath = CollectionDB::instance()->albumImage( m_artist, m_album, 1, &m_embedded );
     m_coverPixmap = QPixmap( m_coverImagePath );  //create the scaled cover
 
     repaint();
@@ -916,15 +983,7 @@ void CoverViewItem::paintItem(QPainter* p, const QColorGroup& cg)
     int nameWidth = fm.width( str );
     if( nameWidth > textRect().width() )
     {
-        QString nameJustify = "...";
-        int i = 0;
-        while ( fm.width( nameJustify + str[ i ] ) < textRect().width() )
-            nameJustify += str[ i++ ];
-        nameJustify.remove( 0, 3 );
-        if ( nameJustify.isEmpty() )
-            nameJustify = str.left( 1 );
-        nameJustify += "...";
-        str = nameJustify;
+        str = KStringHandler::rPixelSqueeze( str, p->fontMetrics(), textRect().width() );
     }
     p->setPen( cg.text() );
     p->drawText( textRect(), Qt::AlignCenter, str );
@@ -946,8 +1005,8 @@ void CoverViewItem::dropped( QDropEvent *e, const QValueList<QIconDragItem> & )
        if( hasCover() ) {
            int button = KMessageBox::warningContinueCancel( iconView(),
                             i18n( "Are you sure you want to overwrite this cover?"),
-                            QString::null,
-                            i18n("&Overwrite Confirmation") );
+                            i18n("Overwrite Confirmation"),
+                            i18n("&Overwrite") );
            if( button == KMessageBox::Cancel )
                return;
        }
