@@ -25,6 +25,7 @@ class HelixSimplePlayerAudioStreamInfoResponse;
 #include <pthread.h>
 #include <vector>
 #include <config.h>
+#include <iostream>
 using std::vector;
 
 #define MAX_PATH PATH_MAX
@@ -52,17 +53,26 @@ class IHXAudioStreamInfoResponse;
 class IHXCommonClassFactory;
 class IHXPluginEnumerator;
 class IHXPlugin2Handler;
+class IHXAudioDeviceManager;
+class IHXAudioHookManager;
 class IHXPreferences;
+class HSPAudioDevice;
+class IHXAudioDeviceResponse;
 #ifdef USE_HELIX_ALSA
 struct _snd_mixer;
 struct _snd_mixer_elem;
 #endif
 
 // scope delay queue
-struct DelayQueue
+class DelayQueue
 {
-   DelayQueue(int bufsize) : fwd(0), len(bufsize), time(0), etime(0), nchan(0), bps(0),buf(0) { buf = new unsigned char [ bufsize ]; }
-   ~DelayQueue() { delete [] buf; }
+public:
+   DelayQueue() : fwd(0), len(0), time(0), etime(0), nchan(0), bps(0),  allocd(false), buf(0) {}
+   DelayQueue(int bufsize) : fwd(0), len(bufsize), time(0), etime(0), nchan(0), bps(0), allocd(true), buf(0) 
+      { buf = new unsigned char [ bufsize ]; }
+   DelayQueue(DelayQueue &src) : fwd(0), len(src.len), time(src.time), etime(src.etime), nchan(src.nchan), bps(src.bps),  allocd(true), buf(0)
+      { buf = new unsigned char [ len ]; memcpy( (void *) buf, (void *) src.buf, src.len ); }
+   ~DelayQueue() { if (allocd) delete [] buf; }
    struct DelayQueue *fwd;
    int           len;      // len of the buffer
    unsigned long time;     // start time of the buffer
@@ -71,6 +81,7 @@ struct DelayQueue
    int           bps;      // bytes per sample
    double        tps;      // time per sample
    int           spb;      // samples per buffer
+   bool          allocd;   // did we allocate the memory?
    unsigned char *buf;
 };
 
@@ -104,13 +115,15 @@ public:
    void play(const char *url, int playerIndex = ALL_PLAYERS,
              bool fadein = false, bool fadeout = false,
              unsigned long fadetime = 0);                            // play the file, setting it as the current url; wait for it to finish
-   int  setURL(const char *url, int playerIndex = ALL_PLAYERS);      // set the current url
+   int  setURL(const char *url, 
+               int playerIndex = ALL_PLAYERS,
+               bool islocal = true);                                 // set the current url
    bool done(int playerIndex = ALL_PLAYERS);                         // test to see if the player(s) is(are) done
    void start(int playerIndex = ALL_PLAYERS, 
-              bool fadein = false, bool fadeout = false,
+              bool fadein = false, 
               unsigned long fadetime = 0);                           // start the player
    void start(const char *file, int playerIndex = ALL_PLAYERS,
-              bool fadein = false, bool fadeout = false,
+              bool fadein = false,
               unsigned long fadetime = 0);                           // start the player, setting the current url first
    void stop(int playerIndex = ALL_PLAYERS);                         // stop the player(s)
    void pause(int playerIndex = ALL_PLAYERS);                        // pause the player(s)
@@ -125,6 +138,10 @@ public:
    void dispatch();                                                  // dispatch the player(s)
 
    void cleanUpStream(int playerIndex);                              // cleanup after stream complete
+   bool isPlaying(int playerIndex) const;                            // is the player currently playing?
+   bool isLocal(int playerIndex) const;                              // playing a local file
+   int numPlayers() const { return nNumPlayers; }                    // return the number of players
+
 
    virtual void onVolumeChange(int) {}                   // called when the volume is changed
    virtual void onMuteChange(int) {}                     // called when mute is changed
@@ -160,6 +177,10 @@ private:
    IHXCommonClassFactory*   pCommonClassFactory;
    IHXPluginEnumerator*     pPluginE;
    IHXPlugin2Handler*       pPlugin2Handler;
+   IHXAudioDeviceManager*   pAudioDeviceManager;
+   IHXAudioHookManager*     pAudioHookManager;
+   IHXAudioHook*            pFinalAudioHook;
+   HSPAudioDevice*          pAudioDevice;
 
    struct playerCtrl
    {
@@ -167,7 +188,7 @@ private:
       bool                        bStarting;
       bool                        bFadeIn;
       bool                        bFadeOut;
-      unsigned long               ulFadeTime;
+      unsigned long               ulFadeLength;
       IHXAudioStream*             pStream;
       HSPClientContext*           pHSPContext;
       IHXPlayer*                  pPlayer;
@@ -176,15 +197,22 @@ private:
       IHXAudioCrossFade*          pCrossFader;
       IHXVolume*                  pVolume;
       IHXVolumeAdviseSink*        pVolumeAdvise;
-      IHXAudioHook*               pPostMixHook;
       IHXAudioStreamInfoResponse* pStreamInfoResponse;
       IHXAudioHook*               pPreMixHook;
+      IHXAudioHook*               pPostMixHook;
       metaData                    md;
       char*                       pszURL;
+      bool                        isLocal;
       unsigned short              volume;
       bool                        ismute;
+      // scope
+      int                         scopecount;
+      struct DelayQueue          *scopebufhead;
+      struct DelayQueue          *scopebuftail;
+      pthread_mutex_t             m_scope_m;
    } **ppctrl;
 
+   IHXAudioDeviceResponse *pAudioDeviceResponse;
    bool                    bURLFound;
    int                     nNumPlayers;
    int                     nNumPlayRepeats;
@@ -195,33 +223,19 @@ private:
    bool 		   bStopping;
    int 		           nPlay;
 
-   // crossfading
-   struct xfade
-   {
-      bool                 crossfading;
-      unsigned long        duration;
-      int                  fromIndex;
-      int                  toIndex;
-      IHXAudioStream       *fromStream;
-      IHXAudioStream       *toStream;
-   } m_xf;
-   
 public:
-   // cross fade the next track (given by url). startPos is in the old stream
-   void enableCrossFader(int playerFrom, int playerTo);
-   void disableCrossFader();
-   void crossFade(const char *url, unsigned long startPos, unsigned long xfduration);
-   inline struct xfade &xf() { return m_xf; }
    const IHXAudioPlayer *getAudioPlayer(int playerIndex) const { return ppctrl[playerIndex]->pAudioPlayer; }
    const IHXAudioCrossFade *getCrossFader(int playerIndex) const { return ppctrl[playerIndex]->pCrossFader; }
-   void startCrossFade();
+
+   // crossfade
+   void setFadeout(bool fadeout, unsigned long fadelength, int playerIndex = ALL_PLAYERS);
 
    // scope
-   void addScopeBuf(struct DelayQueue *item);
-   struct DelayQueue *getScopeBuf();
-   int getScopeCount() { return scopecount; }
-   int peekScopeTime(unsigned long &t);
-   void clearScopeQ();
+   void addScopeBuf(struct DelayQueue *item, int playerIndex);
+   DelayQueue *getScopeBuf(int playerIndex);
+   int getScopeCount(int playerIndex) { return playerIndex >= 0 && playerIndex < nNumPlayers ? ppctrl[playerIndex]->scopecount : 0; }
+   int peekScopeTime(unsigned long &t, int playerIndex);
+   void clearScopeQ(int playerIndex = ALL_PLAYERS);
 
    // equalizer
    void enableEQ(bool enabled) { m_eq_enabled = enabled; }
@@ -232,8 +246,7 @@ public:
    int getPluginInfo(int index, 
                      const char *&description, 
                      const char *&copyright, 
-                     const char *&moreinfourl, 
-                     unsigned long &ver) const;
+                     const char *&moreinfourl) const;
 
 
    metaData *getMetaData(int playerIndex);
@@ -260,12 +273,6 @@ private:
    MimeList            *mimehead;
    int                  mimelistlen;
 
-   // scope
-   int                  scopecount;
-   struct DelayQueue   *scopebufhead;
-   struct DelayQueue   *scopebuftail;
-   pthread_mutex_t      m_scope_m;
-
    // equalizer
    bool                 m_eq_enabled;
 
@@ -291,6 +298,9 @@ public:
    void setAlsaCapableCore() { m_AlsaCapableCore = true; }
    virtual int fallbackToOSS() { return 0; }
 
+   int                  m_preamp;
+   vector<int>          m_equalizerGains;
+
 private:
    AUDIOAPI m_outputsink;
    char *m_device;
@@ -299,36 +309,57 @@ private:
 protected:
    void openAudioDevice();
    void closeAudioDevice();
-   int getDirectHWVolume();
-   void setDirectHWVolume(int vol);
+#ifdef USE_HELIX_ALSA
+   int getDirectMasterVolume();
+   void setDirectMasterVolume(int vol);
+#endif
+   int getDirectPCMVolume();
+   void setDirectPCMVolume(int vol);
 
 private:
    AUDIOAPI m_direct;
    bool m_AlsaCapableCore;
    int m_nDevID;
 #ifdef USE_HELIX_ALSA
-   struct _snd_mixer* m_pAlsaMixerHandle;
-   struct _snd_mixer_elem* m_pAlsaMixerElem;
+   struct _snd_mixer*      m_pAlsaMixerHandle;
+   struct _snd_mixer_elem* m_pAlsaMasterMixerElem;
+   struct _snd_mixer_elem* m_pAlsaPCMMixerElem;
 #endif
    char *m_alsaDevice;
    bool m_urlchanged;
    int m_volBefore;
    int m_volAtStart;
+#ifdef USE_HELIX_ALSA
+   int m_MvolBefore;
+   int m_MvolAtStart;
+#endif
+
+   int m_numPlugins;
+   struct pluginInfo
+   {
+      char *description; 
+      char *copyright; 
+      char *moreinfourl; 
+   } **m_pluginInfo;
+
 
 protected:
-   int                  m_preamp;
-   vector<int>          m_equalizerGains;
    virtual int print2stdout(const char *fmt, ...);
    virtual int print2stderr(const char *fmt, ...);
+   virtual void notifyUser(unsigned long/*code*/, const char */*moreinfo*/, const char */*moreinfourl*/) {}
+   virtual void interruptUser(unsigned long/*code*/, const char */*moreinfo*/, const char */*moreinfourl*/) {}
 
    friend class HSPClientAdviceSink;
    friend class HSPErrorSink;
    friend class HSPAuthenticationManager;
    friend class HelixSimplePlayerAudioStreamInfoResponse;
    friend class HSPPreMixAudioHook;
+   friend class HSPPostProcessor;
    friend class HSPPostMixAudioHook;
+   friend class HSPFinalAudioHook;
    friend class HelixSimplePlayerVolumeAdvice;
    friend class HSPEngineContext;
+   friend class HSPAudioDevice;
 };
 
 #endif
