@@ -1,5 +1,8 @@
 /***************************************************************************
- *   Copyright (C) 2004,5 Max Howell <max.howell@methylblue.com>           *
+ *   Copyright (C) 2005   Christophe Thommeret <hftom@free.fr>             *
+ *             (C) 2005   Ian Monroe <ian@monroe.nu>                       *
+ *             (C) 2005,6 Mark Kretschmann <markey@web.de>                 *
+ *             (C) 2004,5 Max Howell <max.howell@methylblue.com>           *
  *             (C) 2003,4 J. Kofler <kaffeine@gmx.net>                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -67,10 +70,12 @@ XineEngine::XineEngine()
         , m_preamp( 1.0 )
         , m_stopFader( false )
 {
-    addPluginProperty( "StreamingMode", "NoStreaming" );
     addPluginProperty( "HasConfigure", "true" );
     addPluginProperty( "HasEqualizer", "true" );
+    #ifndef __NetBSD__  // NetBSD does not offer audio mixing
     addPluginProperty( "HasCrossfade", "true" );
+    #endif
+    addPluginProperty("HasCDDA", "true"); // new property
 }
 
 XineEngine::~XineEngine()
@@ -83,7 +88,7 @@ XineEngine::~XineEngine()
     }
 
     // NOTE The fadeout gets stuck when the EQ is active, so we skip it then
-    if( !m_equalizerEnabled && m_stream && xine_get_status( m_stream ) == XINE_STATUS_PLAY )
+    if( !m_equalizerEnabled && m_stream && state() == Engine::Playing )
     {
         const int volume = xine_get_param( m_stream, XINE_PARAM_AUDIO_AMP_LEVEL );
         const double D = 300000 * std::pow( (double)volume, -0.4951 );
@@ -191,13 +196,21 @@ XineEngine::makeNewStream()
    xine_set_param( m_stream, XINE_PARAM_METRONOM_PREBUFFER, 6000 );
    xine_set_param( m_stream, XINE_PARAM_IGNORE_VIDEO, 1 );
    #endif
-
+#ifdef XINE_PARAM_EARLY_FINISHED_EVENT
+    if ( xine_check_version(1,1,1) ) {
+        // enable gapless playback
+        debug() << "gapless playback enabled." << endl;
+        xine_set_param(m_stream, XINE_PARAM_EARLY_FINISHED_EVENT, 1 );
+    }
+#endif
    return true;
 }
 
 bool
 XineEngine::load( const KURL &url, bool isStream )
 {
+    DEBUG_BLOCK
+
     Engine::Base::load( url, isStream );
 
     if( m_xfadeLength > 0 && xine_get_status( m_stream ) == XINE_STATUS_PLAY )
@@ -223,7 +236,13 @@ XineEngine::load( const KURL &url, bool isStream )
 
       return true;
    }
-
+   else
+   {
+      #ifdef XINE_PARAM_GAPLESS_SWITCH
+        if ( xine_check_version(1,1,1) )
+            xine_set_param( m_stream, XINE_PARAM_GAPLESS_SWITCH, 0);
+      #endif
+   }
    //s_fader will delete itself
 
    return false;
@@ -239,16 +258,6 @@ XineEngine::play( uint offset )
 
         emit stateChanged( Engine::Playing );
 
-        //HACK Emit two dummy bundles, because the context-browser expects them for
-        //     the stream metadata display.
-        if( isStream() ) {
-            Engine::SimpleMetaBundle bundle;
-            bundle.title = "dummy";
-            emit metaData( bundle );
-            bundle.title = i18n( "Radio Stream" );
-            emit metaData( bundle );
-        }
-
         return true;
     }
 
@@ -286,6 +295,7 @@ XineEngine::stop()
     std::fill( m_scope.begin(), m_scope.end(), 0 );
 
     xine_stop( m_stream );
+    xine_close( m_stream );
     xine_set_param( m_stream, XINE_PARAM_AUDIO_CLOSE_DEVICE, 1);
 
     emit stateChanged( Engine::Empty );
@@ -328,22 +338,40 @@ XineEngine::position() const
 
     xine_get_pos_length( m_stream, &pos, &time, &length );
 
+
+    // Here we check for new metadata periodically, because xine does not emit an event
+    // in all cases (e.g. with ogg streams). See BUG 122505
+    const Engine::SimpleMetaBundle bundle = fetchMetaData();
+    if( bundle.title != m_currentBundle.title || bundle.artist != m_currentBundle.artist ) {
+        debug() << "Metadata received." << endl;
+        m_currentBundle = bundle;
+
+        XineEngine* p = const_cast<XineEngine*>( this );
+        p->emit metaData( bundle );
+    }
+
+
     return time;
 }
 
 uint
 XineEngine::length() const
 {
-    if( m_url.fileName().endsWith( ".ogg", false ) )
+    // xine often delivers nonsense values for VBR files and such, so we only
+    // use the length for remote files
+
+    if( m_url.isLocalFile() )
         return 0;
 
-    int pos;
-    int time;
-    int length = 0;
+    else {
+        int pos;
+        int time;
+        int length = 0;
 
-    xine_get_pos_length( m_stream, &pos, &time, &length );
+        xine_get_pos_length( m_stream, &pos, &time, &length );
 
-    return length;
+        return length;
+    }
 }
 
 void
@@ -373,30 +401,43 @@ XineEngine::setEqualizerEnabled( bool enable )
     if( !enable ) {
         QValueList<int> gains;
         for( uint x = 0; x < 10; x++ )
-            gains << 0;
+            gains << -102; // sets eq gains to zero.
 
-        setEqualizerParameters( -100, gains );
+        setEqualizerParameters( 0, gains );
    }
 }
 
+/*
+ sets the eq params for xine engine - have to rescale eq params to fitting range (adapted from kaffeine)
+ sideeffect: volume decreases with enabled eq, don't know why, occurs also in kaffeine
+
+ preamp
+   pre: (-100..100)
+   post: (0.2..1.8) - this is not really a preamp but xine offers no preamp so we make a postamp
+
+ gains
+   pre: (-100..100)
+   post: (1..100) - range adapted from kaffeine (1 = down, 50 = middle, 100 = up, 0 = off[!!!])
+ */
 void
 XineEngine::setEqualizerParameters( int preamp, const QValueList<int> &gains )
 {
     m_equalizerGains = gains;
     m_intPreamp = preamp;
     QValueList<int>::ConstIterator it = gains.begin();
-    xine_set_param( m_stream, XINE_PARAM_EQ_30HZ, *it );
-    xine_set_param( m_stream, XINE_PARAM_EQ_60HZ, *++it );
-    xine_set_param( m_stream, XINE_PARAM_EQ_125HZ, *++it );
-    xine_set_param( m_stream, XINE_PARAM_EQ_250HZ, *++it );
-    xine_set_param( m_stream, XINE_PARAM_EQ_500HZ, *++it );
-    xine_set_param( m_stream, XINE_PARAM_EQ_1000HZ, *++it );
-    xine_set_param( m_stream, XINE_PARAM_EQ_2000HZ, *++it );
-    xine_set_param( m_stream, XINE_PARAM_EQ_4000HZ, *++it );
-    xine_set_param( m_stream, XINE_PARAM_EQ_8000HZ, *++it );
-    xine_set_param( m_stream, XINE_PARAM_EQ_16000HZ, *++it );
 
-    m_preamp = ( preamp / 2 + 150 ) / 100.0;
+    xine_set_param( m_stream, XINE_PARAM_EQ_30HZ,    int( (*it  )*0.49 + 50 ) );
+    xine_set_param( m_stream, XINE_PARAM_EQ_60HZ,    int( (*++it)*0.49 + 50 ) );
+    xine_set_param( m_stream, XINE_PARAM_EQ_125HZ,   int( (*++it)*0.49 + 50 ) );
+    xine_set_param( m_stream, XINE_PARAM_EQ_250HZ,   int( (*++it)*0.49 + 50 ) );
+    xine_set_param( m_stream, XINE_PARAM_EQ_500HZ,   int( (*++it)*0.49 + 50 ) );
+    xine_set_param( m_stream, XINE_PARAM_EQ_1000HZ,  int( (*++it)*0.49 + 50 ) );
+    xine_set_param( m_stream, XINE_PARAM_EQ_2000HZ,  int( (*++it)*0.49 + 50 ) );
+    xine_set_param( m_stream, XINE_PARAM_EQ_4000HZ,  int( (*++it)*0.49 + 50 ) );
+    xine_set_param( m_stream, XINE_PARAM_EQ_8000HZ,  int( (*++it)*0.49 + 50 ) );
+    xine_set_param( m_stream, XINE_PARAM_EQ_16000HZ, int( (*++it)*0.49 + 50 ) );
+
+    m_preamp = ( preamp - 0.2 * preamp + 100 ) / 100.0;
     setVolume( m_volume );
 }
 
@@ -408,7 +449,7 @@ XineEngine::canDecode( const KURL &url ) const
     {
         char* exts = xine_get_file_extensions( m_xine );
         list = QStringList::split( ' ', exts );
-        delete [] exts; exts = NULL;
+        free( exts ); exts = NULL;
         //images
         list.remove("png");
         list.remove("jpg");
@@ -428,10 +469,15 @@ XineEngine::canDecode( const KURL &url ) const
         if(!list.contains("m4a"))
             list << "m4a";
     }
-    const QString path = url.path();
-    const QString ext  = path.mid( path.findRev( '.' ) + 1 ).lower();
 
-    return list.contains( ext );
+    QString path = url.path();
+
+    if (path.endsWith( ".part" ))
+        path = path.left( path.length() - 5 );
+
+    const QString ext = path.mid( path.findRev( '.' ) + 1 ).lower();
+
+    return list.contains( ext ) || url.protocol() == "cdda";
 }
 
 const Engine::Scope&
@@ -443,6 +489,10 @@ XineEngine::scope()
     MyNode* const myList         = scope_plugin_list( m_post );
     metronom_t* const myMetronom = scope_plugin_metronom( m_post );
     const int myChannels         = scope_plugin_channels( m_post );
+    int scopeidx = 0;
+
+    if (myChannels > 2)
+       return m_scope;
 
     //prune the buffer list and update m_currentVpts
     timerEvent( 0 );
@@ -481,10 +531,12 @@ XineEngine::scope()
 
         for( int a, c; frame < n; ++frame, data16 += myChannels ) {
             for( a = c = 0; c < myChannels; ++c )
-                a += data16[c];
-
-            a /= myChannels;
-            m_scope[frame] = a;
+            {
+               // we now give interleaved pcm to the scope
+               m_scope[scopeidx++] = data16[c];
+               if (myChannels == 1) // duplicate mono samples
+                  m_scope[scopeidx++] = data16[c];
+            }
         }
 
         m_currentVpts = best_node->vpts_end;
@@ -545,7 +597,7 @@ XineEngine::customEvent( QCustomEvent *e )
 
     switch( e->type() )
     {
-    case 3000:
+    case 3000: //XINE_EVENT_UI_PLAYBACK_FINISHED
         emit trackEnded();
         break;
 
@@ -560,17 +612,9 @@ XineEngine::customEvent( QCustomEvent *e )
         break;
 
     case 3003: { //meta info has changed
-
         debug() << "Metadata received." << endl;
-
-        Engine::SimpleMetaBundle bundle;
-        bundle.title   = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_TITLE ) );
-        bundle.artist  = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_ARTIST ) );
-        bundle.album   = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_ALBUM ) );
-        bundle.comment = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_COMMENT ) );
-        bundle.genre   = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_GENRE ) );
-        bundle.bitrate = QString::number( xine_get_stream_info( m_stream, XINE_STREAM_INFO_BITRATE ) / 1000 );
-
+        const Engine::SimpleMetaBundle bundle = fetchMetaData();
+        m_currentBundle = bundle;
         emit metaData( bundle );
     }   break;
 
@@ -637,7 +681,10 @@ XineEngine::XineEventListener( void *p, const xine_event_t* xineEvent )
         break;
 
     case XINE_EVENT_UI_PLAYBACK_FINISHED:
-
+        #ifdef XINE_PARAM_GAPLESS_SWITCH
+            if ( xine_check_version(1,1,1) && xe->m_url.isLocalFile() ) //Remote media break with gapless
+                xine_set_param( xe->m_stream, XINE_PARAM_GAPLESS_SWITCH, 1);
+        #endif
         //emit signal from GUI thread
         QApplication::postEvent( xe, new QCustomEvent(3000) );
         break;
@@ -776,6 +823,24 @@ XineEngine::XineEventListener( void *p, const xine_event_t* xineEvent )
     #undef xe
 }
 
+Engine::SimpleMetaBundle
+XineEngine::fetchMetaData() const
+{
+    Engine::SimpleMetaBundle bundle;
+    bundle.title      = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_TITLE ) );
+    bundle.artist     = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_ARTIST ) );
+    bundle.album      = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_ALBUM ) );
+    bundle.comment    = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_COMMENT ) );
+    bundle.genre      = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_GENRE ) );
+    bundle.bitrate    = QString::number( xine_get_stream_info( m_stream, XINE_STREAM_INFO_AUDIO_BITRATE ) / 1000 );
+    bundle.samplerate = QString::number( xine_get_stream_info( m_stream, XINE_STREAM_INFO_AUDIO_SAMPLERATE ) );
+    bundle.year       = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_YEAR ) );
+    bundle.tracknr    = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_TRACK_NUMBER ) );
+
+    return bundle;
+}
+
+
 //////////////////
 /// class Fader
 //////////////////
@@ -907,4 +972,89 @@ Fader::run()
     deleteLater();
 }
 
+
+bool XineEngine::metaDataForUrl(const KURL &url, Engine::SimpleMetaBundle &b)
+{
+    bool result = false;
+    xine_stream_t* tmpstream = xine_stream_new(m_xine, NULL, NULL);
+    if (xine_open(tmpstream, QFile::encodeName(url.url()))) {
+        QString audioCodec = QString::fromUtf8(xine_get_meta_info(tmpstream, XINE_META_INFO_SYSTEMLAYER));
+
+        if (audioCodec == "CDDA") {
+            QString title = QString::fromUtf8(
+                xine_get_meta_info(tmpstream, XINE_META_INFO_TITLE));
+            if ((!title.isNull()) && (!title.isEmpty())) { //no meta info
+                b.title = title;
+                b.artist =
+                    QString::fromUtf8(
+                        xine_get_meta_info(tmpstream, XINE_META_INFO_ARTIST));
+                b.album =
+                    QString::fromUtf8(
+                        xine_get_meta_info(tmpstream, XINE_META_INFO_ALBUM));
+                b.genre =
+                    QString::fromUtf8(
+                        xine_get_meta_info(tmpstream, XINE_META_INFO_GENRE));
+                b.year =
+                    QString::fromUtf8(
+                        xine_get_meta_info(tmpstream, XINE_META_INFO_YEAR));
+                b.tracknr =
+                    QString::fromUtf8(
+                        xine_get_meta_info(tmpstream, XINE_META_INFO_TRACK_NUMBER));
+            } else {
+                b.title = QString(i18n("Track %1")).arg(url.filename());
+                b.album = i18n("AudioCD");
+            }
+        }
+
+        if (audioCodec == "CDDA" || audioCodec == "WAV") {
+            result = TRUE;
+            int samplerate = xine_get_stream_info( tmpstream, XINE_STREAM_INFO_AUDIO_SAMPLERATE );
+
+            // xine would provide a XINE_STREAM_INFO_AUDIO_BITRATE, but unfortunately not for CDDA or WAV
+            // so we calculate the bitrate by our own
+            int bitsPerSample = xine_get_stream_info( tmpstream, XINE_STREAM_INFO_AUDIO_BITS );
+            int nbrChannels = xine_get_stream_info( tmpstream, XINE_STREAM_INFO_AUDIO_CHANNELS );
+            int bitrate = (samplerate * bitsPerSample * nbrChannels) / 1000;
+
+            b.bitrate = QString::number(bitrate);
+            b.samplerate = QString::number(samplerate);
+            int pos, time, length = 0;
+            xine_get_pos_length(tmpstream, &pos, &time, &length);
+            b.length = QString::number(length / 1000);
+        }
+        xine_close(tmpstream);
+    }
+    xine_dispose(tmpstream);
+    return result;
+}
+
+bool XineEngine::getAudioCDContents(const QString &device, KURL::List &urls)
+{
+    char **xine_urls = NULL;
+    int num;
+    int i = 0;
+
+    if (!device.isNull()) {
+        debug() << "xine-engine setting CD Device to: " << device << endl;
+        xine_cfg_entry_t config;
+        xine_config_lookup_entry(m_xine, "input.cdda_device", &config);
+        config.str_value = (char *)device.latin1();
+        xine_config_update_entry(m_xine, &config);
+    }
+
+    emit statusText(i18n("Getting AudioCD contents..."));
+
+    xine_urls = xine_get_autoplay_mrls(m_xine, "CD", &num);
+
+    if (xine_urls) {
+        while (xine_urls[i]) {
+            urls << KURL(xine_urls[i]);
+            ++i;
+        }
+    }
+    else emit statusText(i18n("Could not read AudioCD"));
+
+    return true;
+}
+
 #include "xine-engine.moc"
