@@ -18,9 +18,9 @@
 #include "enginebase.h"
 #include "enginecontroller.h"
 #include "playlist.h"
+#include "playlistloader.h"
 #include "pluginmanager.h"
 #include "statusbar.h"
-#include "streamprovider.h"
 
 #include <qfile.h>
 #include <qtimer.h>
@@ -56,9 +56,9 @@ EngineController::EngineController()
         , m_muteVolume( 0 )
         , m_xFadeThisTrack( false )
         , m_timer( new QTimer( this ) )
-        , m_stream( 0 )
+        , m_playFailureCount( 0 )
 {
-    m_voidEngine = m_engine = (EngineBase*)loadEngine( "void-engine" );
+    m_voidEngine = m_engine = static_cast<EngineBase*>( loadEngine( "void-engine" ) );
 
     connect( m_timer, SIGNAL( timeout() ), SLOT( slotMainTimer() ) );
 }
@@ -94,8 +94,8 @@ EngineController::loadEngine() //static
         PluginManager::unload( oldEngine );
 
         // the engine is not required to do this when we unload it but
-        // we need to do it to ensure amaroK looks correct and to delete
-        // m_stream. We don't do this for the void-engine because that
+        // we need to do it to ensure amaroK looks correct.
+        // We don't do this for the void-engine because that
         // means amaroK sets all components to empty on startup, which is
         // their responsibility.
         slotStateChanged( Engine::Empty );
@@ -155,7 +155,7 @@ EngineController::loadEngine( const QString &engineName )
 
         if( plugin ) {
             QObject *bar = amaroK::StatusBar::instance();
-            EngineBase *engine = (EngineBase*)plugin;
+            EngineBase *engine = static_cast<EngineBase*>( plugin );
 
             connect( engine, SIGNAL(stateChanged( Engine::State )),
                        this,   SLOT(slotStateChanged( Engine::State )) );
@@ -210,8 +210,7 @@ bool EngineController::canDecode( const KURL &url ) //static
     const QString fileName = url.fileName();
     const QString ext = amaroK::extension( fileName );
 
-    //FIXME why do we do this? Please add comments to odd looking code!
-    if ( ext == "m3u" || ext == "pls" ) return false;
+    if ( PlaylistFile::isPlaylistFile( fileName ) ) return false;
 
     // Ignore protocols "fetchcover" and "musicbrainz", they're not local but we dont really want them in the playlist :)
     if ( url.protocol() == "fetchcover" || url.protocol() == "musicbrainz" ) return false;
@@ -261,8 +260,7 @@ void EngineController::restoreSession()
     {
         const KURL url = AmarokConfig::resumeTrack();
 
-        if ( m_engine->load( url ) && m_engine->play( AmarokConfig::resumeTime() ) )
-            newMetaDataNotify( m_bundle = MetaBundle( url ), true );
+        play( MetaBundle( url ), AmarokConfig::resumeTime() );
     }
 }
 
@@ -270,10 +268,11 @@ void EngineController::restoreSession()
 void EngineController::endSession()
 {
     //only update song stats, when we're not going to resume it
-    if ( m_bundle.length() > 0 && !AmarokConfig::resumePlayback() )
+    if ( !AmarokConfig::resumePlayback() )
         trackEnded( m_engine->position(), m_bundle.length() * 1000 );
 
     PluginManager::unload( m_voidEngine );
+    m_voidEngine = 0;
 }
 
 //////////////////////////////////////////////////////////////////////////////////////////
@@ -303,29 +302,25 @@ void EngineController::play() //SLOT
 }
 
 
-void EngineController::play( const MetaBundle &bundle )
+void EngineController::play( const MetaBundle &bundle, uint offset )
 {
-    const KURL &url = bundle.url();
+    DEBUG_BLOCK
+
+    //Holds the time since we started trying to play non-existent files
+    //so we know when to abort
+    static QTime failure_time;
+    if ( !m_playFailureCount )
+        failure_time.start();
+
+    const KURL url = bundle.url();
     debug() << "Loading URL: " << url.url() << endl;
-    // Destroy stale StreamProvider
-    delete m_stream;
     m_lastMetadata.clear();
 
     //TODO bummer why'd I do it this way? it should _not_ be in play!
     //let amaroK know that the previous track is no longer playing
-    if ( m_timer->isActive() && m_bundle.length() > 0 )
+    if ( m_timer->isActive() )
         trackEnded( m_engine->position(), m_bundle.length() * 1000 );
 
-    if ( url.protocol() == "http" ||
-        ( url.protocol() == "zeroconf" && url.path().section( '/',1,1 ) == "_shoutcast._tcp" ) ) {
-        m_bundle = bundle;
-        // Detect mimetype of remote file
-        KIO::MimetypeJob* job = KIO::mimetype( url, false );
-        connect( job, SIGNAL(result( KIO::Job* )), SLOT(playRemote( KIO::Job* )) );
-        amaroK::StatusBar::instance()->shortMessage( i18n("Connecting to stream source...") );
-        return; //don't do notify
-    }
-
     if ( url.isLocalFile() ) {
         // does the file really exist? the playlist entry might be old
         if ( ! QFile::exists( url.path()) ) {
@@ -334,39 +329,82 @@ void EngineController::play( const MetaBundle &bundle )
             goto some_kind_of_failure;
         }
     }
+    else
+        amaroK::StatusBar::instance()->shortMessage( i18n("Connecting to stream source...") );
 
-    if( m_engine->load( url ) )
+
+    if( m_engine->load( url, url.protocol() == "http" || url.protocol() == "rtsp" ) )
     {
         //assign bundle now so that it is available when the engine
         //emits stateChanged( Playing )
         m_bundle = bundle;
 
-        if( m_engine->play() )
+        if( m_engine->play( offset ) )
         {
+            //Reset failure count as we are now successfully playing a song
+            m_playFailureCount = 0;
+
             // Ask engine for track length, if available. It's more reliable than TagLib.
-            const uint trackLength = m_engine->length();
-            if ( trackLength ) m_bundle.setLength( trackLength / 1000 );
+            const uint trackLength = m_engine->length() / 1000;
+            if ( trackLength ) m_bundle.setLength( trackLength );
 
             m_xFadeThisTrack = AmarokConfig::crossfade() &&
                                m_engine->hasPluginProperty( "HasCrossfade" ) &&
                               !m_engine->isStream() &&
-                               m_bundle.length()*1000 - AmarokConfig::crossfadeLength()*2 > 0;
+                              !(url.protocol() == "cdda") &&
+                               m_bundle.length()*1000 - offset - AmarokConfig::crossfadeLength()*2 > 0;
 
             newMetaDataNotify( m_bundle, true /* track change */ );
+            return;
         }
-        else goto some_kind_of_failure;
     }
-    else
+
     some_kind_of_failure:
-        //NOTE now we don't do next() at all
-        // say the user has a 4000 item playlist with all URLs in that playlist
-        // being bad, amaroK will appear to freeze. We need to either stat files in
-        // the background or not call next() more than 5 times in a row
-        //don't do for repeatPlaylist() as it can produce a freeze
-        //FIXME -> mxcl
-        //if ( !AmarokConfig::repeatPlaylist() )
-        //    next()
-                ;
+        debug() << "Failed to play this track." << endl;
+
+        ++m_playFailureCount;
+
+        //Code to skip to next track if playback fails:
+        //
+        //* The failure counter is reset if a track plays successfully or if playback is
+        //  stopped, for whatever reason.
+        //* For normal playback, the attempt to play is stopped at the end of the playlist
+        //* For repeat playlist , a whole playlist worth of songs is tried
+        //* For repeat album, the number of songs tried is the number of tracks from the
+        //  album that are in the playlist.
+        //* For repeat track, no attempts are made
+        //* For the nmm engine, no attempts are made (necessary? / FIXME)
+        //* To prevent GUI freezes we don't try to play again after 0.5s of failure
+        int totalTracks = Playlist::instance()->totalTrackCount();
+        int currentTrack = Playlist::instance()->currentTrackIndex();
+        if ( ( ( amaroK::repeatPlaylist() && static_cast<int>(m_playFailureCount) < totalTracks )
+            || ( amaroK::repeatNone() && currentTrack != totalTracks - 1 )
+            || ( amaroK::repeatAlbum() && m_playFailureCount < Playlist::instance()->repeatAlbumTrackCount() ) )
+            && AmarokConfig::soundSystem() != "nmm-engine"
+            && failure_time.elapsed() < 500 )
+        {
+
+           debug() << "Skipping to next track." << endl;
+
+           // The test for loaded must be done _before_ next is called
+           if ( !m_engine->loaded() )
+           {
+               //False gives behaviour as if track played successfully
+               next( false );
+               QTimer::singleShot(   0, this, SLOT(   play() ) );
+           }
+           else
+           {
+               //False gives behaviour as if track played successfully
+               next( false );
+           }
+        }
+        else
+        {
+            //Stop playback, including resetting failure count (as all new failures are
+            //treated as independent after playback is stopped)
+            stop();
+        }
 }
 
 
@@ -379,12 +417,16 @@ void EngineController::pause() //SLOT
 
 void EngineController::stop() //SLOT
 {
-    //let amaroK know that the previous track is no longer playing
-    if ( m_bundle.length() > 0 )
-        trackEnded( m_engine->position(), m_bundle.length() * 1000 );
+    //Reset failure counter as after stop, everything else is unrelated
+    m_playFailureCount = 0;
 
-    if ( m_engine->loaded() )
-        m_engine->stop();
+    //let amaroK know that the previous track is no longer playing
+    trackEnded( m_engine->position(), m_bundle.length() * 1000 );
+
+    //Remove requirement for track to be loaded for stop to be called (fixes gltiches
+    //where stop never properly happens if call to m_engine->load fails in play)
+    //if ( m_engine->loaded() )
+    m_engine->stop();
 }
 
 
@@ -409,11 +451,11 @@ void EngineController::seek( int ms ) //SLOT
 
 void EngineController::seekRelative( int ms ) //SLOT
 {
-  if( m_engine->state() == Engine::Playing )
-  {
-    int newPos = m_engine->position() + ms;
-    seek( newPos <= 0 ? 1 : newPos );
-  }
+    if( m_engine->state() != Engine::Empty )
+    {
+        int newPos = m_engine->position() + ms;
+        seek( newPos <= 0 ? 1 : newPos );
+    }
 }
 
 
@@ -487,65 +529,6 @@ EngineController::bundle() const
 }
 
 
-//////////////////////////////////////////////////////////////////////////////////////////
-// PRIVATE SLOTS
-//////////////////////////////////////////////////////////////////////////////////////////
-
-void EngineController::playRemote( KIO::Job* job ) //SLOT
-{
-    DEBUG_BLOCK
-
-    const QString mimetype = static_cast<KIO::MimetypeJob*>( job )->mimetype();
-    debug() << "Detected mimetype: " << mimetype << endl;
-
-    const KURL &url = static_cast<KIO::MimetypeJob*>( job )->url();
-
-    debug() << "MimetypeJob returned with url: " << url.prettyURL() << endl;
-
-    const bool isStream = mimetype.isEmpty() || mimetype == "text/html" ||
-                          url.host().endsWith( "last.fm" ); // HACK last.fm uses the mimetype audio/x-mp3
-
-    if( isStream && m_engine->pluginProperty( "StreamingMode" ) != "NoStreaming" )
-    {
-        m_xFadeThisTrack = false;
-
-        delete m_stream;
-        m_stream = new amaroK::StreamProvider( url, m_engine->pluginProperty( "StreamingMode" ) );
-
-        if ( !m_stream->initSuccess() || !m_engine->play( m_stream->proxyUrl(), isStream ) ) {
-            delete m_stream;
-            if ( !AmarokConfig::repeatPlaylist() )
-                next();
-            return; //don't notify
-        }
-
-        connect( m_stream, SIGNAL(metaData( const MetaBundle& )),
-                 this,       SLOT(slotStreamMetaData( const MetaBundle& )) );
-        connect( m_stream, SIGNAL(streamData( char*, int )),
-                 m_engine,   SLOT(newStreamData( char*, int )) );
-        connect( m_stream, SIGNAL(sigError()),
-                 this,       SLOT(slotSigError()));
-    }
-    else if( m_engine->play( url, isStream ) )
-    {
-        // Ask engine for track length, if available.
-        const uint trackLength = m_engine->length();
-        if ( trackLength ) m_bundle.setLength( trackLength / 1000 );
-    }
-    else if( !AmarokConfig::repeatPlaylist() )
-    {
-        next();
-        return; //don't notify
-    }
-
-    newMetaDataNotify( m_bundle, true /* track change */ );
-}
-
-void EngineController::slotSigError()
-{
-    emit orderNext( true );
-}
-
 void EngineController::slotStreamMetaData( const MetaBundle &bundle ) //SLOT
 {
     // Prevent spamming by ignoring repeated identical data (some servers repeat it every 10 seconds)
@@ -563,10 +546,13 @@ void EngineController::slotStreamMetaData( const MetaBundle &bundle ) //SLOT
     newMetaDataNotify( m_bundle, false /* not a new track */ );
 }
 
+
+//////////////////////////////////////////////////////////////////////////////////////////
+// PRIVATE SLOTS
+//////////////////////////////////////////////////////////////////////////////////////////
+
 void EngineController::slotEngineMetaData( const Engine::SimpleMetaBundle &simpleBundle ) //SLOT
 {
-    DEBUG_BLOCK
-
     if ( !m_bundle.url().isLocalFile() )
     {
         MetaBundle bundle = m_bundle;
@@ -579,11 +565,18 @@ void EngineController::slotEngineMetaData( const Engine::SimpleMetaBundle &simpl
             bundle.setGenre( simpleBundle.genre );
         if( !simpleBundle.bitrate.isEmpty() )
             bundle.setBitrate( simpleBundle.bitrate.toInt() );
+        if( !simpleBundle.samplerate.isEmpty() )
+            bundle.setSampleRate( simpleBundle.samplerate.toInt() );
+        if( !simpleBundle.year.isEmpty() )
+            bundle.setYear( simpleBundle.year.toInt() );
+        if( !simpleBundle.tracknr.isEmpty() )
+            bundle.setTrack( simpleBundle.tracknr.toInt() );
 
         slotStreamMetaData( bundle );
     }
 }
 
+
 void EngineController::slotMainTimer() //SLOT
 {
     const uint position = m_engine->position();
@@ -593,6 +586,7 @@ void EngineController::slotMainTimer() //SLOT
     // Crossfading
     if ( m_engine->state() == Engine::Playing &&
          m_xFadeThisTrack &&
+         !AmarokConfig::crossfadeManualOnly() &&
          Playlist::instance()->isTrackAfter() &&
          m_bundle.length()*1000 - position < (uint) AmarokConfig::crossfadeLength() )
     {
@@ -601,6 +595,7 @@ void EngineController::slotMainTimer() //SLOT
     }
 }
 
+
 void EngineController::slotTrackEnded() //SLOT
 {
     if ( AmarokConfig::trackDelayLength() > 0 )
@@ -616,14 +611,13 @@ void EngineController::slotTrackEnded() //SLOT
     else trackFinished();
 }
 
+
 void EngineController::slotStateChanged( Engine::State newState ) //SLOT
 {
     switch( newState )
     {
     case Engine::Empty:
 
-        delete m_stream;
-
         //FALL THROUGH...
 
     case Engine::Paused:
@@ -644,12 +638,4 @@ void EngineController::slotStateChanged( Engine::State newState ) //SLOT
 }
 
 
-void EngineController::streamError() //SLOT
-{
-    delete m_stream;
-    next();
-}
-
-
-
 #include "enginecontroller.moc"
