@@ -17,14 +17,12 @@
 #include "engineobserver.h"
 #include "metabundle.h"
 
-#include <qguardedptr.h>
 #include <qmap.h>
 #include <qobject.h>
 #include <qvaluelist.h>
 
 class QTimer;
 
-namespace amaroK { class StreamProvider; }
 namespace KIO { class Job; }
 
 
@@ -55,13 +53,11 @@ public:
 
     uint trackLength() const { return m_bundle.length() * 1000; }
     const MetaBundle &bundle() const;
-    const KURL &playingURL() const { return bundle().url(); }
+    KURL playingURL() const { return bundle().url(); }
 
     void restoreSession();
     void endSession();
 
-    void currentTrackMetaDataChanged( const MetaBundle& bundle ) { m_bundle = bundle; newMetaDataNotify( bundle, false /* no track change */ ); }
-
     //xx000, xx100, xx200, so at most will be 200ms delay before time displays are updated
     static const int MAIN_TIMER = 300;
 
@@ -72,7 +68,7 @@ public slots:
     void next( const bool forceNext = true );
     void trackFinished() { next(false); };
     void play();
-    void play( const MetaBundle& );
+    void play( const MetaBundle&, uint offset = 0 );
     void pause();
     void stop();
     void playPause(); //pauses if playing, plays if paused or stopped
@@ -88,6 +84,9 @@ public slots:
 
     void mute();
 
+    void slotStreamMetaData( const MetaBundle &bundle );
+    void currentTrackMetaDataChanged( const MetaBundle& bundle ) { m_bundle = bundle; newMetaDataNotify( bundle, false /* no track change */ ); }
+
 signals:
     void orderPrevious();
     void orderCurrent();
@@ -95,14 +94,11 @@ signals:
     void statusText( const QString& );
 
 private slots:
-    void playRemote( KIO::Job* );
-    void slotStreamMetaData( const MetaBundle& );
     void slotEngineMetaData( const Engine::SimpleMetaBundle& );
     void slotMainTimer();
     void slotTrackEnded();
     void slotStateChanged( Engine::State );
-    void streamError();
-    void slotSigError();
+
 protected:
     EngineController();
    ~EngineController();
@@ -125,8 +121,7 @@ private:
     bool            m_xFadeThisTrack;
     bool            m_isTiming;
     QTimer*         m_timer;
-
-    QGuardedPtr<amaroK::StreamProvider> m_stream;
+    uint            m_playFailureCount;
 };
 
 
