@@ -15,21 +15,38 @@ email                : fh@ez.no
  *                                                                         *
  ***************************************************************************/
 
+#include "debug.h"
 #include "engineobserver.h"
-
 #include <qptrlist.h>
-#include <kdebug.h>
+
+
+//////////////////////////////////////////////////////////////////////////////////////////
+/// CLASS EngineObserver
+//////////////////////////////////////////////////////////////////////////////////////////
 
 EngineObserver::EngineObserver()
+    : m_subject( 0 )
+{}
+
+EngineObserver::EngineObserver( EngineSubject *s )
+    : m_subject( s )
 {
-    // autoadd to subject in constructor?
+   m_subject->attach( this );
 }
 
 EngineObserver::~EngineObserver()
-{}
+{
+    if ( m_subject )
+        m_subject->detach( this );
+}
 
 
+//////////////////////////////////////////////////////////////////////////////////////////
+/// CLASS EngineSubject
+//////////////////////////////////////////////////////////////////////////////////////////
+
 EngineSubject::EngineSubject()
+    : m_oldEngineState( Engine::Empty )
 {}
 
 EngineSubject::~EngineSubject()
@@ -38,18 +55,24 @@ EngineSubject::~EngineSubject()
 
 void EngineSubject::stateChangedNotify( Engine::State state )
 {
+    DEBUG_BLOCK
+
     QPtrListIterator<EngineObserver> it( Observers );
     EngineObserver *observer;
     while( ( observer = it.current() ) != 0 )
     {
         ++it;
-        observer->engineStateChanged( state );
+        observer->engineStateChanged( state, m_oldEngineState );
     }
+
+    m_oldEngineState = state;
 }
 
 
 void EngineSubject::newMetaDataNotify( const MetaBundle &bundle, bool trackChanged )
 {
+    DEBUG_BLOCK
+
     QPtrListIterator<EngineObserver> it( Observers );
     EngineObserver *observer;
     while( ( observer = it.current() ) != 0 )
@@ -79,14 +102,26 @@ void EngineSubject::volumeChangedNotify( int percent )
 }
 
 
-void EngineSubject::trackPositionChangedNotify( long position )
+void EngineSubject::trackPositionChangedNotify( long position, bool userSeek )
 {
     QPtrListIterator<EngineObserver> it( Observers );
     EngineObserver *observer;
     while( ( observer = it.current() ) != 0 )
     {
         ++it;
-        observer->engineTrackPositionChanged( position );
+        observer->engineTrackPositionChanged( position, userSeek );
+    }
+}
+
+
+void EngineSubject::trackLengthChangedNotify( long length )
+{
+    QPtrListIterator<EngineObserver> it( Observers );
+    EngineObserver *observer;
+    while( ( observer = it.current() ) != 0 )
+    {
+        ++it;
+        observer->engineTrackLengthChanged( length );
     }
 }
 
