@@ -18,8 +18,9 @@ email                : fh@ez.no
 #ifndef AMAROK_ENGINEOBSERVER_H
 #define AMAROK_ENGINEOBSERVER_H
 
-#include "enginebase.h"
+#include "engine_fwd.h"
 
+class EngineSubject;
 class MetaBundle;
 
 /**
@@ -31,12 +32,17 @@ class EngineObserver
 {
 public:
     EngineObserver();
+    EngineObserver( EngineSubject* );
     virtual ~EngineObserver();
-    virtual void engineStateChanged( Engine::State /*state*/ ) {}
+    virtual void engineStateChanged( Engine::State /*state*/, Engine::State /*oldState*/ = Engine::Empty ) {}
     virtual void engineNewMetaData( const MetaBundle &/*bundle*/, bool /*trackChanged*/ ) {}
     virtual void engineTrackEnded( int /*finalPosition*/, int /*trackLength*/ ) {}
     virtual void engineVolumeChanged( int /*percent*/ ) {}
-    virtual void engineTrackPositionChanged( long /*position*/ ) {}
+    virtual void engineTrackPositionChanged( long /*position*/ , bool /*userSeek*/ ) {}
+    virtual void engineTrackLengthChanged( long /*length*/ ) {}
+
+private:
+    EngineSubject *m_subject;
 };
 
 #include <qptrlist.h>
@@ -57,10 +63,13 @@ protected:
     void newMetaDataNotify( const MetaBundle &/*bundle*/, bool /*trackChanged*/ );
     void trackEnded( int /*finalPosition*/, int /*trackLength*/ );
     void volumeChangedNotify( int /*percent*/ );
-    void trackPositionChangedNotify( long /*position*/ );
+    /* userSeek means the position didn't change due to normal playback */
+    void trackPositionChangedNotify( long /*position*/ , bool userSeek=false );
+    void trackLengthChangedNotify( long /*length*/ );
 
 private:
     QPtrList<EngineObserver> Observers;
+    Engine::State m_oldEngineState;
 };
 
 #endif // AMAROK_ENGINEOBSERVER_H
