@@ -27,6 +27,7 @@
 #include "enginecontroller.h"
 #include "filebrowser.h"
 #include "k3bexporter.h"
+
 #include <kaction.h>
 #include <kapplication.h>
 #include "kbookmarkhandler.h"
@@ -42,10 +43,15 @@
 #include <kurlcompletion.h>
 
 #include "mediabrowser.h"
+#include "medium.h"
+#include "mydirlister.h"
+#include "mydiroperator.h"
 #include "playlist.h"
 #include "playlistbrowser.h"
 #include "playlistloader.h"
 #include "playlistwindow.h"
+#include "collectionbrowser.h"
+#include "tagdialog.h"
 
 #include <qdir.h>
 #include <qhbox.h>
@@ -55,74 +61,48 @@
 #include <qtooltip.h>
 
 
-//TODO wait for lister to finish, if there are no files shown, but there are
-//     media files in that directory show a longMessage (preferably one that disappears when given a signal)
-
-
-class MyDirLister : public KDirLister {
-public:
-    MyDirLister( bool delayedMimeTypes ) : KDirLister( delayedMimeTypes ) {}
-
-protected:
-    virtual bool matchesMimeFilter( const KFileItem *item ) const {
-        return
-            item->isDir() ||
-            EngineController::canDecode( item->url() ) ||
-            item->url().protocol() == "audiocd" ||
-            PlaylistFile::isPlaylistFile( item->name() ) ||
-            item->name().endsWith( ".mp3", false ) || //for now this is less confusing for the user
-            item->name().endsWith( ".ogg", false );
-    }
-};
-
-class MyDirOperator : public KDirOperator {
-public:
-    MyDirOperator( const KURL &url, QWidget *parent ) : KDirOperator( url, parent ) {
-        setDirLister( new MyDirLister( true ) );
-//the delete key event was being eaten with the file browser open
-//so folks couldn't use the del key to remove items from the playlist
-//refer to KDirOperator::setupActions() to understand this code
-        KActionCollection* dirActionCollection = static_cast<KActionCollection*>(KDirOperator::child("KDirOperator::myActionCollection"));
-        if(dirActionCollection)
-        {
-            KAction* trash = dirActionCollection->action("trash");
-            if(trash)
-                trash->setEnabled(false);
-        }
-    }
-public slots:
-    //reimplemented due to a bug in KDirOperator::activatedMenu ( KDE 3.4.2 ) - See Bug #103305
-    virtual void activatedMenu (const KFileItem *, const QPoint &pos) {
-        updateSelectionDependentActions();
-        ((KActionMenu*)actionCollection()->action("popupMenu"))->popupMenu()->popup( pos );
-    }
-};
-
 
 //BEGIN Constructor/destructor
 
-FileBrowser::FileBrowser( const char * name )
+FileBrowser::FileBrowser( const char * name, Medium * medium )
         : QVBox( 0, name )
 {
     KActionCollection *actionCollection;
     KConfig* const config = amaroK::config( "Filebrowser" );
     SearchPane *searchPane;
 
-    KURL location( config->readPathEntry( "Location", QDir::homeDirPath() ) );
-    KFileItem currentFolder( KFileItem::Unknown, KFileItem::Unknown, location );
-    //KIO sucks, NetAccess::exists puts up a dialog and has annoying error message boxes
-    //if there is a problem so there is no point in using it anyways.
-    //so... setting the diroperator to ~ is the least sucky option
-    if ( !location.isLocalFile() || !currentFolder.isReadable() ) {
-        location = QDir::homeDirPath() ;
+    KURL *location;
+    KFileItem *currentFolder;
+
+    // Try to keep filebrowser working even if not in a medium context
+    // so if a medium object not passed in, keep earlier behavior
+    if (!medium) {
+        m_medium = 0;
+        location = new KURL( config->readPathEntry( "Location", QDir::homeDirPath() ) );
+        currentFolder = new KFileItem( KFileItem::Unknown, KFileItem::Unknown, *location );
+        //KIO sucks, NetAccess::exists puts up a dialog and has annoying error message boxes
+        //if there is a problem so there is no point in using it anyways.
+        //so... setting the diroperator to ~ is the least sucky option
+        if ( !location->isLocalFile() || !currentFolder->isReadable() ) {
+            delete location;
+            location = new KURL( QDir::homeDirPath() ) ;
+        }
     }
+    else{
+        m_medium = medium;
+        location = new KURL( m_medium->mountPoint() );
+        currentFolder = new KFileItem( KFileItem::Unknown, KFileItem::Unknown, *location );
+    }
+
+    KActionCollection* ac = new KActionCollection( this );
+    KStdAction::selectAll( this, SLOT( selectAll() ), ac, "filebrowser_select_all" );
 
     KToolBar *toolbar = new Browser::ToolBar( this );
 
     { //Filter LineEdit
         KToolBar* searchToolBar = new Browser::ToolBar( this );
         KToolBarButton *button = new KToolBarButton( "locationbar_erase", 0, searchToolBar );
-        m_filter = new ClickLineEdit( i18n( "Filter here..." ), searchToolBar );
+        m_filter = new ClickLineEdit( i18n( "Enter search terms here" ), searchToolBar );
 
         searchToolBar->setStretchableWidget( m_filter );
 
@@ -147,20 +127,34 @@ FileBrowser::FileBrowser( const char * name )
 
         //folder selection combo box
         m_combo = new KURLComboBox( KURLComboBox::Directories, true, box, "path combo" );
-        m_combo->setCompletionObject( new KURLCompletion( KURLCompletion::DirCompletion ) );
-        m_combo->setAutoDeleteCompletionObject( true );
+
+        if (!m_medium){
+            m_combo->setCompletionObject( new KURLCompletion( KURLCompletion::DirCompletion ) );
+            m_combo->setAutoDeleteCompletionObject( true );
+        }
         m_combo->setMaxItems( 9 );
         m_combo->setURLs( config->readPathListEntry( "Dir History" ) );
-        m_combo->lineEdit()->setText( location.path() );
+
+        if (!m_medium)
+            m_combo->lineEdit()->setText( location->path() );
+        else
+            m_combo->lineEdit()->setText( "/" );
 
         //The main widget with file listings and that
-        m_dir = new MyDirOperator( location, container );
+        m_dir = new MyDirOperator( *location, container, m_medium );
         m_dir->setEnableDirHighlighting( true );
         m_dir->setMode( KFile::Mode((int)KFile::Files | (int)KFile::Directory) ); //allow selection of multiple files + dirs
         m_dir->setOnlyDoubleClickSelectsFiles( true ); //amaroK type settings
         m_dir->readConfig( config );
         m_dir->setView( KFile::Default ); //will set userconfigured view, will load URL
         m_dir->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Expanding );
+        m_dir->setAcceptDrops( true );
+        //Automatically open folder after hovering above it...probably a good thing
+        //but easily disabled by commenting this line out
+        //Disabled for now because can't show . and .. folders.
+        //TODO: Find out a way to fix this?
+        //m_dir->setDropOptions( KFileView::AutoOpenDirs );
+
         static_cast<QFrame*>(m_dir->viewWidget())->setFrameStyle( QFrame::NoFrame );
         static_cast<QIconView*>(m_dir->viewWidget())->setSpacing( 1 );
 
@@ -172,7 +166,7 @@ FileBrowser::FileBrowser( const char * name )
     }
 
     {
-        QPopupMenu* const menu = ((KActionMenu*)actionCollection->action("popupMenu"))->popupMenu();
+        QPopupMenu* const menu = static_cast<KActionMenu*>(actionCollection->action("popupMenu"))->popupMenu();
 
         menu->clear();
         menu->insertItem( SmallIconSet( "fileopen" ), i18n( "&Load" ), MakePlaylist );
@@ -180,13 +174,19 @@ FileBrowser::FileBrowser( const char * name )
         menu->insertItem( SmallIconSet( "filesave" ), i18n( "&Save as Playlist..." ), SavePlaylist );
         menu->insertSeparator();
 
-        menu->insertItem( SmallIconSet( "usbpendrive_unmount" ), i18n( "Add to Media Device &Transfer Queue" ), MediaDevice );
-        menu->insertItem( SmallIconSet( "cdrom_unmount" ), i18n("Burn to CD"), BurnCd );
+        if (!m_medium)
+            menu->insertItem( SmallIconSet( amaroK::icon( "device" ) ), i18n( "&Transfer to Media Device" ), MediaDevice );
+
+        menu->insertItem( SmallIconSet( amaroK::icon( "collection" ) ), i18n( "&Organize Files..." ), OrganizeFiles );
+        menu->insertItem( SmallIconSet( amaroK::icon( "collection" ) ), i18n( "&Copy Files to Collection..." ), CopyToCollection );
+        menu->insertItem( SmallIconSet( amaroK::icon( "collection" ) ), i18n( "&Move Files to Collection..." ), MoveToCollection );
+        menu->insertItem( SmallIconSet( "cdrom_unmount" ), i18n("Burn to CD..."), BurnCd );
         menu->insertSeparator();
         menu->insertItem( i18n( "&Select All Files" ), SelectAllFiles );
         menu->insertSeparator();
         actionCollection->action( "delete" )->plug( menu );
         menu->insertSeparator();
+        menu->insertItem( SmallIconSet( amaroK::icon( "info" ) ), i18n( "Edit Track &Information..." ), EditTags );
         actionCollection->action( "properties" )->plug( menu );
 
         menu->setItemEnabled( BurnCd, K3bExporter::isAvailable() );
@@ -198,7 +198,7 @@ FileBrowser::FileBrowser( const char * name )
     {
         KActionMenu *a;
 
-        a = (KActionMenu*)actionCollection->action( "sorting menu" );
+        a = static_cast<KActionMenu*>( actionCollection->action( "sorting menu" ) );
         a->setIcon( "configure" );
         a->setDelayed( false ); //TODO should be done by KDirOperator
 
@@ -211,13 +211,40 @@ FileBrowser::FileBrowser( const char * name )
     }
 
     {
-        QStringList actions;
-        actions << "up" << "back" << "forward" << "home" << "reload" << "short view"
-                << "detailed view" << "sorting menu" << "bookmarks";
+        if ( KAction *a = actionCollection->action( "up" ) )
+            a->plug( toolbar );
+        if ( KAction *a = actionCollection->action( "back" ) )
+            a->plug( toolbar );
+        if ( KAction *a = actionCollection->action( "forward" ) )
+            a->plug( toolbar );
+        if ( KAction *a = actionCollection->action( "home" ) )
+            a->plug( toolbar );
+        if ( KAction *a = actionCollection->action( "reload" ) )
+            a->plug( toolbar );
 
-        foreach( actions )
-            if ( KAction *a = actionCollection->action( (*it).latin1() ) )
-                a->plug( toolbar );
+        toolbar->insertLineSeparator();
+
+        if ( KAction *a = actionCollection->action( "short view" ) )
+            a->plug( toolbar );
+        if ( KAction *a = actionCollection->action( "detailed view" ) )
+            a->plug( toolbar );
+
+        toolbar->insertLineSeparator();
+
+        if ( KAction *a = actionCollection->action( "sorting menu" ) )
+            a->plug( toolbar );
+        if ( KAction *a = actionCollection->action( "bookmarks" ) )
+            a->plug( toolbar );
+
+
+        KAction *gotoCurrent = new KAction( i18n("Goto Current Track Folder"), "today", 0,
+                                            this, SLOT( gotoCurrentFolder() ), actionCollection );
+        gotoCurrent->plug( toolbar );
+
+        disconnect( actionCollection->action( "up" ), SIGNAL( activated() ), m_dir, SLOT( cdUp() ) );
+        connect( actionCollection->action( "up" ), SIGNAL( activated() ), m_dir, SLOT( myCdUp() ) );
+        disconnect( actionCollection->action( "home" ), SIGNAL( activated() ), m_dir, SLOT( home() ) );
+        connect( actionCollection->action( "home" ), SIGNAL( activated() ), m_dir, SLOT( myHome() ) );
     }
 
     connect( m_filter, SIGNAL(textChanged( const QString& )), SLOT(setFilter( const QString& )) );
@@ -227,6 +254,8 @@ FileBrowser::FileBrowser( const char * name )
     connect( m_dir, SIGNAL(fileSelected( const KFileItem* )), SLOT(activate( const KFileItem* )) );
     connect( m_dir, SIGNAL(urlEntered( const KURL& )), SLOT(urlChanged( const KURL& )) );
     connect( m_dir, SIGNAL(urlEntered( const KURL& )), searchPane, SLOT(urlChanged( const KURL& )) );
+    connect( m_dir, SIGNAL(dropped( const KFileItem*, QDropEvent*, const KURL::List& )),
+                        SLOT(dropped( const KFileItem*, QDropEvent*, const KURL::List& )) );
 
     setSpacing( 4 );
     setFocusProxy( m_dir ); //so the dirOperator is focussed when we get focus events
@@ -246,6 +275,30 @@ FileBrowser::~FileBrowser()
 
 //END Constructor/Destructor
 
+void FileBrowser::setUrl( const KURL &url )
+{
+    m_dir->setFocus();
+    if (!m_medium)
+        m_dir->setURL( url, true );
+    else {
+        QString urlpath = url.isLocalFile() ? url.path() : url.prettyURL();
+        KURL newURL( urlpath.prepend( m_medium->mountPoint() ).remove("..") );
+        //debug() << "set-url-kurl: changing to: " << newURL.path() << endl;
+        m_dir->setURL( newURL, true );
+    }
+}
+
+void FileBrowser::setUrl( const QString &url )
+{
+    if (!m_medium)
+        m_dir->setURL( KURL(url), true );
+    else{
+        KURL newURL( QString(url).prepend( m_medium->mountPoint() ).remove("..") );
+        //debug() << "set-url-qstring: changing to: " << newURL.path() << endl;
+        m_dir->setURL( newURL, true );
+    }
+}
+
 //BEGIN Private Methods
 
 KURL::List FileBrowser::selectedItems()
@@ -300,6 +353,24 @@ FileBrowser::setFilter( const QString &text )
     m_dir->updateDir();
 }
 
+void
+FileBrowser::dropped( const KFileItem* /*item*/, QDropEvent* event, const KURL::List &urls){
+    //Do nothing right now
+    event->ignore();
+    //Run into const problems iterating over the list, so copy it to a malleable one
+    //(besides, need to filter for local giles)
+    KURL::List list(urls);
+
+    for ( KURL::List::iterator it = list.begin(); it != list.end(); ){
+        if ( m_medium && !(*it).isLocalFile() )
+            it = list.erase( it );
+        else{
+            debug() << "Dropped: " << (*it) << endl;
+            it++;
+        }
+    }
+}
+
 //END Public Slots
 
 
@@ -310,11 +381,17 @@ FileBrowser::urlChanged( const KURL &u )
 {
     //the DirOperator's URL has changed
 
-    const QString url = u.url();
+    QString url = u.isLocalFile() ? u.path() : u.prettyURL();
+
+    if( m_medium ){
+        //remove the leading mountPoint value
+        url.remove( 0, m_medium->mountPoint().length() );
+    }
 
     QStringList urls = m_combo->urls();
     urls.remove( url );
     urls.prepend( url );
+
     m_combo->setURLs( urls, KURLComboBox::RemoveBottom );
 }
 
@@ -339,10 +416,13 @@ inline void
 FileBrowser::prepareContextMenu()
 {
     const KFileItemList &items = *m_dir->selectedItems();
-    ((KActionMenu*)m_dir->actionCollection()->action("popupMenu"))->popupMenu()->setItemVisible( SavePlaylist,
+    static_cast<KActionMenu*>(m_dir->actionCollection()->action("popupMenu"))->popupMenu()->setItemVisible( SavePlaylist,
         items.count() > 1 || ( items.count() == 1 && items.getFirst()->isDir() ) );
-    ((KActionMenu*)m_dir->actionCollection()->action("popupMenu"))->popupMenu()->setItemVisible( MediaDevice,
-        MediaDevice::instance()->isConnected() );
+    static_cast<KActionMenu*>(m_dir->actionCollection()->action("popupMenu"))->popupMenu()->setItemVisible( MediaDevice,
+        MediaBrowser::isAvailable() );
+    static_cast<KActionMenu*>(m_dir->actionCollection()->action("popupMenu"))->popupMenu()->setItemVisible( MoveToCollection, !CollectionDB::instance()->isDirInCollection( url().path() ) );
+    static_cast<KActionMenu*>(m_dir->actionCollection()->action("popupMenu"))->popupMenu()->setItemVisible( CopyToCollection, !CollectionDB::instance()->isDirInCollection( url().path() ) );
+    static_cast<KActionMenu*>(m_dir->actionCollection()->action("popupMenu"))->popupMenu()->setItemVisible( OrganizeFiles, CollectionDB::instance()->isDirInCollection( url().path() ) );
 }
 
 inline void
@@ -362,20 +442,41 @@ FileBrowser::contextMenuActivated( int id )
         Playlist::instance()->insertMedia( selectedItems() );
         break;
 
+    case EditTags:
+        {
+            KURL::List list = amaroK::recursiveUrlExpand( selectedItems() );
+            TagDialog *dialog = NULL;
+            if( list.count() == 1 )
+            {
+                dialog = new TagDialog( list.first(), this );
+            }
+            else
+            {
+                dialog = new TagDialog( list, this );
+            }
+            dialog->show();
+        }
+        break;
+
+    case CopyToCollection:
+        CollectionView::instance()->organizeFiles( selectedItems(), i18n( "Copy Files To Collection" ), true );
+        break;
+
+    case MoveToCollection:
+        CollectionView::instance()->organizeFiles( selectedItems(), i18n( "Move Files To Collection" ), false );
+        break;
+
+    case OrganizeFiles:
+        CollectionView::instance()->organizeFiles( selectedItems(), i18n( "Organize Collection Files" ), false );
+        break;
+
     case MediaDevice:
-        MediaDevice::instance()->addURLs( selectedItems() );
+        MediaBrowser::queue()->addURLs( selectedItems() );
         break;
 
     case SelectAllFiles:
-    {
-        KFileItemList list( *m_dir->view()->items() );
-
-        // Select all items which represent files
-        for( KFileItem* item = list.first(); item; item = list.next() )
-            m_dir->view()->setSelected( item, item->isFile() );
-
+        selectAll();
         break;
-    }
 
     case BurnCd:
         K3bExporter::instance()->exportTracks( selectedItems() );
@@ -383,9 +484,27 @@ FileBrowser::contextMenuActivated( int id )
     }
 }
 
+inline void
+FileBrowser::gotoCurrentFolder()
+{
+    const KURL &url = EngineController::instance()->bundle().url();
+    KURL dirURL = KURL::fromPathOrURL( url.directory() );
+
+    m_combo->setURL( dirURL );
+    setUrl( dirURL );
+}
+
 //END Private Slots
 
+void
+FileBrowser::selectAll()
+{
+    KFileItemList list( *m_dir->view()->items() );
 
+    // Select all items which represent files
+    for( KFileItem* item = list.first(); item; item = list.next() )
+        m_dir->view()->setSelected( item, item->isFile() );
+}
 
 #include <kurldrag.h>
 #include <qpainter.h>
@@ -414,7 +533,7 @@ public:
         QPtrList<QListViewItem> items = selectedItems();
         KURL::List urls;
 
-        for( Item *item = (Item*)items.first(); item; item = (Item*)items.next() )
+        for( Item *item = static_cast<Item*>( items.first() ); item; item = static_cast<Item*>( items.next() ) )
             urls += item->m_url;
 
         return new KURLDrag( urls, this );
@@ -539,7 +658,7 @@ SearchPane::searchMatches( const KFileItemList &list )
         if( (*it)->isDir() )
             m_dirs += (*it)->url();
         else if( m_filter.exactMatch( (*it)->name() ) )
-            new KURLView::Item( (*it)->url(), (KURLView*)m_listView );
+            new KURLView::Item( (*it)->url(), static_cast<KURLView*>( m_listView ) );
     }
 }
 
