@@ -23,6 +23,7 @@
     Boston, MA 02111-1307, USA.
 ***************************************************************************/
 
+#include "debug.h"
 #include "multitabbar.h"
 #include "multitabbar.moc"
 #include "multitabbar_p.h"
@@ -39,12 +40,17 @@
 #include <qstyle.h>
 #include <qtimer.h>
 
+#include <kconfig.h>
 #include <kdebug.h>
 #include <kiconloader.h>
+#include <klocale.h>
+#include <kpopupmenu.h>
 #include <kstringhandler.h>
 
 #define NEARBYINT(i) ((int(float(i) + 0.5)))
 
+namespace amaroK { extern KConfig *config( const QString& ); }
+
 class MultiTabBarTabPrivate
 {
     public:
@@ -76,7 +82,7 @@ MultiTabBarInternal::MultiTabBarInternal( QWidget *parent, MultiTabBar::MultiTab
     addChild( box );
     setFrameStyle( NoFrame );
     viewport() ->setBackgroundMode( Qt::PaletteBackground );
-    /*	box->setPaletteBackgroundColor(Qt::red);
+    /*  box->setPaletteBackgroundColor(Qt::red);
         setPaletteBackgroundColor(Qt::green);*/
 }
 
@@ -146,10 +152,7 @@ void MultiTabBarInternal::drawContents ( QPainter * paint, int clipx, int clipy,
 
                 /*                paint->setPen(colorGroup().shadow());
                                 paint->drawLine(0,0,0,viewport()->height());*/
-
             }
-
-
 }
 
 void MultiTabBarInternal::contentsMousePressEvent( QMouseEvent *ev )
@@ -159,16 +162,33 @@ void MultiTabBarInternal::contentsMousePressEvent( QMouseEvent *ev )
 
 void MultiTabBarInternal::mousePressEvent( QMouseEvent *ev )
 {
-    ev->ignore();
+    if ( ev->button() != QMouseEvent::RightButton ){
+        ev->ignore();
+        return;
+    }
+
+    // right button pressed
+    KPopupMenu popup;
+    popup.insertTitle(  i18n("Browsers") , /*id*/ -1, /*index*/ 1 );
+    popup.setCheckable( true );
+    for( uint i = 0; i < m_tabs.count(); i++ ) {
+        MultiTabBarTab* tab = m_tabs.at( i );
+        popup.insertItem( tab->text(), i );
+        popup.setItemChecked(i, tab->visible() ? true : false);
+    }
+
+    int col = popup.exec( static_cast<QMouseEvent *>(ev)->globalPos() );
+    if ( col >= 0 )
+        setTabVisible( col, !popup.isItemChecked(col) );
 }
 
 
 #define CALCDIFF(m_tabs,diff,i) if (m_lines>(int)lines) {\
-                    /*kdDebug()<<"i="<<i<<" tabCount="<<tabCount<<" space="<<space<<endl;*/ \
+                    /*kdDebug()<<"i="<<i<<" visibleTabCount="<<visibleTabCount()<<" space="<<space<<endl;*/ \
                     uint ulen=0;\
                     diff=0; \
-                    for (uint i2=i;i2<tabCount;i2++) {\
-                        uint l1=m_tabs.at(i2)->neededSize();\
+                    for (uint i2=i;i2<visibleTabCount();i2++) {\
+                        uint l1=sizePerTab();\
                         if ((ulen+l1)>space){\
                             if (ulen==0) diff=0;\
                             else diff=((float)(space-ulen))/(i2-i);\
@@ -181,7 +201,7 @@ void MultiTabBarInternal::mousePressEvent( QMouseEvent *ev )
 
 void MultiTabBarInternal::resizeEvent( QResizeEvent *ev )
 {
-    /*	kdDebug()<<"MultiTabBarInternal::resizeEvent"<<endl;
+    /*  kdDebug()<<"MultiTabBarInternal::resizeEvent"<<endl;
         kdDebug()<<"MultiTabBarInternal::resizeEvent - box geometry"<<box->geometry()<<endl;
         kdDebug()<<"MultiTabBarInternal::resizeEvent - geometry"<<geometry()<<endl;*/
     if ( ev ) QScrollView::resizeEvent( ev );
@@ -201,9 +221,11 @@ void MultiTabBarInternal::resizeEvent( QResizeEvent *ev )
         int cnt = 0;
         //CALCULATE LINES
         const uint tabCount = m_tabs.count();
+
         for ( uint i = 0;i < tabCount;i++ ) {
+            if ( ! m_tabs.at( i ) ->visible() ) continue;
             cnt++;
-            tmp += m_tabs.at( i ) ->neededSize();
+            tmp += sizePerTab();
             if ( tmp > space ) {
                 if ( cnt > 1 ) i--;
                 else if ( i == ( tabCount - 1 ) ) break;
@@ -264,6 +286,7 @@ void MultiTabBarInternal::resizeEvent( QResizeEvent *ev )
                 }
             }
         } else {
+            // Left or Right
             setFixedWidth( lines * 24 );
             box->setFixedWidth( lines * 24 );
             m_lines = lines = width() / 24;
@@ -273,8 +296,9 @@ void MultiTabBarInternal::resizeEvent( QResizeEvent *ev )
 
             for ( uint i = 0;i < tabCount;i++ ) {
                 MultiTabBarTab *tab = m_tabs.at( i );
+                if ( ! tab->visible() ) continue;
                 cnt++;
-                tmp += tab->neededSize() + diff;
+                tmp += sizePerTab() + diff;
                 if ( tmp > space ) {
                     if ( cnt > 1 ) {
                         CALCDIFF( m_tabs, diff, i );
@@ -282,7 +306,7 @@ void MultiTabBarInternal::resizeEvent( QResizeEvent *ev )
                         i--;
                     } else {
                         tab->removeEventFilter( this );
-                        tab->move( lines * 24, NEARBYINT( tmp - tab->neededSize() ) );
+                        tab->move( lines * 24, NEARBYINT( tmp - sizePerTab() ) );
                         tab->setFixedHeight( NEARBYINT( tmp + diff ) - tab->y() );;
                         tab->installEventFilter( this );
                     }
@@ -291,8 +315,8 @@ void MultiTabBarInternal::resizeEvent( QResizeEvent *ev )
                     lines++;
                 } else {
                     tab->removeEventFilter( this );
-                    tab->move( lines * 24, NEARBYINT( tmp - tab->neededSize() ) );
-                    tab->setFixedHeight( NEARBYINT( tmp + diff ) - tab->y() );;
+                    tab->move( lines * 24, NEARBYINT( tmp - sizePerTab() ) );
+                    tab->setFixedHeight( NEARBYINT( tmp + diff ) - tab->y() );
                     tab->installEventFilter( this );
                 }
             }
@@ -317,7 +341,6 @@ void MultiTabBarInternal::showActiveTabTexts( bool show )
     m_showActiveTabTexts = show;
 }
 
-
 MultiTabBarTab* MultiTabBarInternal::tab( int id ) const
 {
     for ( QPtrListIterator<MultiTabBarTab> it( m_tabs );it.current();++it ) {
@@ -343,8 +366,20 @@ bool MultiTabBarInternal::eventFilter( QObject *, QEvent *e )
 
         // Calculate index of the new tab to activate
         int newTab = i - delta;
-        if ( newTab < 0 ) newTab = 0;
-        if ( newTab > (int)m_tabs.count() - 1 ) newTab = m_tabs.count() - 1;
+        while (true) {
+            if ( newTab < 0 ) {
+                newTab = i;
+                break;
+            }
+            if ( newTab > (int)m_tabs.count() - 1 ) {
+                newTab = i;
+                break;
+            }
+            if ( m_tabs.at( newTab )->visible() )
+                break;
+            // try one tab more
+            newTab -= delta;
+        }
 
         if ( i < m_tabs.count() && newTab != (int)i )
             m_tabs.at( newTab )->animateClick();
@@ -362,6 +397,7 @@ int MultiTabBarInternal::appendTab( const QPixmap &pic , int id, const QString&
     m_tabs.append( tab = new MultiTabBarTab( pic, text, id, box, m_position, m_style ) );
     tab->installEventFilter( this );
     tab->showActiveTabText( m_showActiveTabTexts );
+    tab->setVisible( amaroK::config( "BrowserBar" )->readBoolEntry( text, true ) );
 
     if ( m_style == MultiTabBar::KONQSBC ) {
         if ( m_expandedTabSize < tab->neededSize() ) {
@@ -371,8 +407,12 @@ int MultiTabBarInternal::appendTab( const QPixmap &pic , int id, const QString&
 
         } else tab->setSize( m_expandedTabSize );
     } else tab->updateState();
-    tab->show();
-    resizeEvent( 0 );
+
+    if ( tab->visible() ) {
+        tab->show();
+        resizeEvent( 0 );
+    } else tab->hide();
+
     return 0;
 }
 
@@ -387,6 +427,35 @@ void MultiTabBarInternal::removeTab( int id )
     }
 }
 
+void MultiTabBarInternal::setTabVisible( int id, bool visible )
+{
+    for ( uint pos = 0;pos < m_tabs.count();pos++ ) {
+        if ( m_tabs.at( pos ) ->id() == id ) {
+            MultiTabBarTab* tab = m_tabs.at( pos );
+
+            tab->setVisible( visible );
+            amaroK::config( "BrowserBar" )->writeEntry( tab->text(), tab->visible() );
+
+            if ( tab->visible() )
+                tab->show();
+            else {
+                tab->hide();
+                // if the user wants to hide the currently avtive tab
+                // turn on another tab
+                if ( tab->isOn() )
+                    for( uint i = 0; i < m_tabs.count(); i++ ) {
+                        if ( m_tabs.at( i )->visible() ) {
+                            m_tabs.at( i )->animateClick();
+                            break;
+                        }
+                    }
+            }
+            // redraw the bar
+            resizeEvent( 0 );
+        }
+    }
+}
+
 void MultiTabBarInternal::setPosition( enum MultiTabBar::MultiTabBarPosition pos )
 {
     m_position = pos;
@@ -396,23 +465,43 @@ void MultiTabBarInternal::setPosition( enum MultiTabBar::MultiTabBarPosition pos
 }
 
 
+uint MultiTabBarInternal::visibleTabCount()
+{
+    uint visibleTabCount = 0;
+    for ( uint i = 0; i < m_tabs.count(); i++ )
+        if ( m_tabs.at( i ) ->visible() ) visibleTabCount++;
+
+    return visibleTabCount;
+}
+
+uint MultiTabBarInternal::sizePerTab()
+{
+    return (height() - 3 ) / visibleTabCount();
+}
+
+
 MultiTabBarButton::MultiTabBarButton( const QPixmap& pic, const QString& text, QPopupMenu *popup,
                                       int id, QWidget *parent, MultiTabBar::MultiTabBarPosition pos, MultiTabBar::MultiTabBarStyle style )
-        : QPushButton( QIconSet(), text, parent ), m_style( style )
+        : QPushButton( QIconSet(), text, parent )
+        , m_position( pos )
+        , m_style( style )
+        , m_id( id )
         , m_animCount( 0 )
         , m_animTimer( new QTimer( this ) )
+        , m_dragSwitchTimer( new QTimer( this ) )
 {
+    setAcceptDrops( true );
     setIconSet( pic );
     setText( text );
-    m_position = pos;
     if ( popup ) setPopup( popup );
     setFlat( true );
     setFixedHeight( 24 );
     setFixedWidth( 24 );
-    m_id = id;
-//     QToolTip::add( this, text );
+
+//     QToolTip::add( this, text );  // Deactivated cause it's annoying
     connect( this, SIGNAL( clicked() ), this, SLOT( slotClicked() ) );
     connect( m_animTimer, SIGNAL( timeout() ), this, SLOT( slotAnimTimer() ) );
+    connect( m_dragSwitchTimer, SIGNAL( timeout() ), this, SLOT( slotDragSwitchTimer() ) );
 }
 
 MultiTabBarButton::MultiTabBarButton( const QString& text, QPopupMenu *popup,
@@ -420,7 +509,9 @@ MultiTabBarButton::MultiTabBarButton( const QString& text, QPopupMenu *popup,
         : QPushButton( QIconSet(), text, parent ), m_style( style )
         , m_animCount( 0 )
         , m_animTimer( new QTimer( this ) )
+        , m_dragSwitchTimer( new QTimer( this ) )
 {
+    setAcceptDrops( true );
     setText( text );
     m_position = pos;
     if ( popup ) setPopup( popup );
@@ -431,6 +522,7 @@ MultiTabBarButton::MultiTabBarButton( const QString& text, QPopupMenu *popup,
 //     QToolTip::add( this, text );
     connect( this, SIGNAL( clicked() ), this, SLOT( slotClicked() ) );
     connect( m_animTimer, SIGNAL( timeout() ), this, SLOT( slotAnimTimer() ) );
+    connect( m_dragSwitchTimer, SIGNAL( timeout() ), this, SLOT( slotDragSwitchTimer() ) );
 }
 
 MultiTabBarButton::~MultiTabBarButton()
@@ -497,6 +589,28 @@ void MultiTabBarButton::leaveEvent( QEvent* )
     m_animTimer->start( ANIM_INTERVAL );
 }
 
+void MultiTabBarButton::dragEnterEvent ( QDragEnterEvent *e )
+{
+    enterEvent ( e );
+}
+
+void MultiTabBarButton::dragMoveEvent ( QDragMoveEvent * )
+{
+    if ( !m_dragSwitchTimer->isActive() )
+        m_dragSwitchTimer->start( ANIM_INTERVAL * ANIM_MAX + 300, TRUE );
+}
+
+void MultiTabBarButton::dragLeaveEvent ( QDragLeaveEvent *e )
+{
+    m_dragSwitchTimer->stop();
+    leaveEvent( e );
+}
+
+void MultiTabBarButton::slotDragSwitchTimer()
+{
+    emit ( initiateDrag ( m_id ) );
+}
+
 void MultiTabBarButton::slotAnimTimer()
 {
     if ( m_animEnter ) {
@@ -531,7 +645,7 @@ QSize MultiTabBarButton::sizeHint() const
         w += style().pixelMetric( QStyle::PM_MenuButtonIndicator, this );
 
     if ( pixmap() ) {
-        QPixmap * pm = ( QPixmap * ) pixmap();
+        QPixmap * pm = const_cast< QPixmap * >( pixmap() );
         w += pm->width();
         h += pm->height();
     } else {
@@ -560,6 +674,7 @@ MultiTabBarTab::MultiTabBarTab( const QPixmap& pic, const QString& text,
                                 int id, QWidget *parent, MultiTabBar::MultiTabBarPosition pos,
                                 MultiTabBar::MultiTabBarStyle style )
         : MultiTabBarButton( text, 0, id, parent, pos, style ),
+        m_visible(true),
         m_showActiveTabText( false )
 {
     d = new MultiTabBarTabPrivate();
@@ -589,7 +704,7 @@ void MultiTabBarTab::setTabsPosition( MultiTabBar::MultiTabBarPosition pos )
     }
 
     setPosition( pos );
-    //	repaint();
+    // repaint();
 }
 
 void MultiTabBarTab::setIcon( const QString& icon )
@@ -672,11 +787,11 @@ void MultiTabBarTab::updateState()
 
 int MultiTabBarTab::neededSize()
 {
-    //PATCH by markey
+/*    //PATCH by markey
     if ( m_style == MultiTabBar::AMAROK )
         return ( parentWidget() ->height() - 3 ) / NUM_TABS;
     else
-        return ( ( ( m_style != MultiTabBar::KDEV3 ) ? 24 : 0 ) + QFontMetrics( QFont() ).width( m_text ) + 6 );
+ */       return ( ( ( m_style != MultiTabBar::KDEV3 ) ? 24 : 0 ) + QFontMetrics( QFont() ).width( m_text ) + 6 );
 }
 
 void MultiTabBarTab::setSize( int size )
@@ -745,8 +860,6 @@ void MultiTabBarTab::drawButtonStyled( QPainter *paint )
     }
     //	style().drawControl(QStyle::CE_PushButtonLabel,painter,this, QRect(0,0,pixmap.width(),pixmap.height()),
     //		colorGroup(),QStyle::Style_Enabled);
-
-
 }
 
 void MultiTabBarTab::drawButtonClassic( QPainter *paint )
@@ -1047,6 +1160,8 @@ void MultiTabBar::setTab( int id, bool state )
     MultiTabBarTab * ttab = tab( id );
     if ( ttab ) {
         ttab->setState( state );
+        if( state && !ttab->visible() )
+            m_internal->setTabVisible( id, true );
     }
 }
 
@@ -1066,6 +1181,16 @@ void MultiTabBar::showActiveTabTexts( bool show )
     m_internal->showActiveTabTexts( show );
 }
 
+uint MultiTabBar::visibleTabCount()
+{
+    return m_internal->visibleTabCount( );
+}
+
+uint MultiTabBar::sizePerTab()
+{
+    return m_internal->sizePerTab( );
+}
+
 void MultiTabBar::setStyle( MultiTabBarStyle style )
 {
     m_internal->setStyle( style );
