@@ -14,11 +14,13 @@
 
 #include "amarok.h"
 #include "amarokconfig.h"
-#include "debug.h"
 #include "collectiondb.h"    //for albumCover location
+#include "debug.h"
 #include "osd.h"
 #include "playlist.h"        //if osdUsePlaylistColumns()
 #include "playlistitem.h"    //ditto
+#include "podcastbundle.h"
+#include "qstringx.h"
 
 #include <kapplication.h>
 #include <kpixmap.h>
@@ -80,6 +82,7 @@ OSDWidget::show() //virtual
     const QRect oldGeometry = QRect( pos(), size() );
     const QRect newGeometry = determineMetrics( M );
 
+#ifdef Q_WS_X11
     if( m_translucency && !isShown() || !newGeometry.intersects( oldGeometry ) )
         m_screenshot = QPixmap::grabWindow( qt_xrootwin(),
                 newGeometry.x(), newGeometry.y(),
@@ -98,6 +101,7 @@ OSDWidget::show() //virtual
         p = newGeometry.topLeft() - unite.topLeft();
         bitBlt( &m_screenshot, 0, 0, &pix, p.x(), p.y() );
     }
+#endif
 
     if( newGeometry.width() > 0 && newGeometry.height() > 0 )
     {
@@ -126,6 +130,8 @@ OSDWidget::determineMetrics( const uint M )
 
     // If we don't do that, the boundingRect() might not be suitable for drawText() (Qt issue N67674)
     m_text.replace( QRegExp(" +\n"), "\n" );
+    // remove consecutive line breaks
+    m_text.replace( QRegExp("\n+"), "\n" );
 
     // The osd cannot be larger than the screen
     QRect rect = fontMetrics().boundingRect( 0, 0,
@@ -392,10 +398,12 @@ void OSDPreviewWidget::mouseMoveEvent( QMouseEvent *e )
 {
     if( m_dragging && this == mouseGrabber() )
     {
+        // Here we implement a "snap-to-grid" like positioning system for the preview widget
+
         const QRect screen      = QApplication::desktop()->screenGeometry( m_screen );
         const uint  hcenter     = screen.width() / 2;
         const uint  eGlobalPosX = e->globalPos().x() - screen.left();
-        const uint  snapZone    = screen.width() / 12;
+        const uint  snapZone    = screen.width() / 24;
 
         QPoint destination = e->globalPos() - m_dragOffset - screen.topLeft();
         int maxY = screen.height() - height() - MARGIN;
@@ -438,8 +446,16 @@ void OSDPreviewWidget::mouseMoveEvent( QMouseEvent *e )
 #include "metabundle.h"
 #include <qregexp.h>
 
+amaroK::OSD::OSD(): OSDWidget( 0 )
+{
+    connect( CollectionDB::instance(), SIGNAL( coverChanged( const QString&, const QString& ) ),
+             this,                   SLOT( slotCoverChanged( const QString&, const QString& ) ) );
+    connect( CollectionDB::instance(), SIGNAL( imageFetched( const QString& ) ),
+             this,                   SLOT( slotImageChanged( const QString& ) ) );
+}
+
 template<class T>
-class MyVector: public QValueVector<T> //fucking QValueVector doesn't have operator<<, wtf?
+class MyVector: public QValueVector<T> //QValueVector doesn't have operator<<, wtf?
 {
     public:
     inline MyVector<T> &operator<< (const T &x)
@@ -453,7 +469,7 @@ void
 amaroK::OSD::show( const MetaBundle &bundle ) //slot
 {
     QString text = "";
-    if( MetaBundle() == bundle )
+    if( bundle.url().isEmpty() )
         text = i18n( "No track playing" );
 
     else
@@ -461,13 +477,10 @@ amaroK::OSD::show( const MetaBundle &bundle ) //slot
         MyVector<QString> tags;
         tags << bundle.prettyTitle();
         for( int i = 0; i < PlaylistItem::NUM_COLUMNS; ++i )
-            tags << bundle.infoByColumn( i, true ).stripWhiteSpace();
+            tags << bundle.prettyText( i );
 
-        const int score = CollectionDB::instance()->getSongPercentage( bundle.url().path() );
-        if( score > 0 )
-            tags[PlaylistItem::Score+1] = QString::number( score );
-        tags[PlaylistItem::Playcount+1] = QString::number( CollectionDB::instance()
-                                                           ->getPlayCount( bundle.url().path() ) );
+        if( bundle.rating() )
+            tags[PlaylistItem::Rating+1] = QString().fill( '*', bundle.rating() / 2 );
         if( bundle.length() <= 0 )
             tags[PlaylistItem::Length+1] = QString::null;
 
@@ -476,11 +489,12 @@ amaroK::OSD::show( const MetaBundle &bundle ) //slot
             QString tag;
             MyVector<int> availableTags; //eg, ones that aren't empty
             static const QValueList<int> parens = //display these in parentheses
-                QValueList<int>() << PlaylistItem::Playcount << PlaylistItem::Year   << PlaylistItem::Comment
-                                  << PlaylistItem::Genre     << PlaylistItem::Length << PlaylistItem::Bitrate
-                                  << PlaylistItem::Score;
+                QValueList<int>() << PlaylistItem::PlayCount  << PlaylistItem::Year   << PlaylistItem::Comment
+                                  << PlaylistItem::Genre      << PlaylistItem::Length << PlaylistItem::Bitrate
+                                  << PlaylistItem::LastPlayed << PlaylistItem::Rating << PlaylistItem::Score
+                                  << PlaylistItem::Filesize;
 
-            for( int n = Playlist::instance()->visibleColumns(), i = 0, column; i < n; ++i )
+            for( int n = Playlist::instance()->numVisibleColumns(), i = 0, column; i < n; ++i )
             {
                 column = Playlist::instance()->mapToLogicalColumn( i );
                 if( !tags.at( column + 1 ).isEmpty() )
@@ -493,43 +507,23 @@ amaroK::OSD::show( const MetaBundle &bundle ) //slot
                 QString append = ( i == 0 ) ? ""
                                : ( n > 1 && i == n / 2 ) ? "\n"
                                : ( parens.contains( column ) || parens.contains( availableTags.at( i - 1 ) ) ) ? " "
-                               : " - ";
+                               : i18n(" - ");
                 append += ( parens.contains( column ) ? "(%1)" : "%1" );
                 text += append.arg( tags.at( column + 1 ) );
             }
         }
         else
         {
-            // we special case prettyTitle and put it first
-            // so that we handle things like streams better
-            // keep these 1:1 with the playlist's columns otherwise
-            static const MyVector<QString> tokens =
-                MyVector<QString>() << "%artist - %title" << "%file"      << "%title"      << "%artist"
-                                    << "%album"           << "%year"      << "%comment"    << "%genre"
-                                    << "%track"           << "%directory" << "%length"     << "%bitrate"
-                                    << "%score"           << "%type"      << "%playcount";
+            QMap<QString, QString> args;
+            args["prettytitle"] = bundle.prettyTitle();
+            for( int i = 0; i < PlaylistItem::NUM_COLUMNS; ++i )
+                args[bundle.exactColumnName( i ).lower()] = bundle.prettyText( i );
+            args["rating"] = QString().fill( '*', bundle.rating() / 2 );
+            if( bundle.length() <= 0 )
+                args["length"] = QString::null;
 
-            text = AmarokConfig::osdText();
-
-            for( MyVector<QString>::ConstIterator tok = tokens.begin(), end = tokens.end(), tag = tags.begin(); tok != end; ++tok, ++tag )
-            {
-                if( (*tok).isNull() )
-                    continue;
-                if( (*tag).isEmpty() ) {
-                    text.remove( QRegExp(QString("\\{[^}]*%1[^{]*\\}").arg( *tok )) );
-                    text.remove( *tok ); //above only works if {} surround the token
-                }
-                else
-                    //NOTE leaves the {} braces
-                    text.replace( *tok, *tag );
-            }
-
-            text.remove( '{' );
-            text.remove( '}' );
-            text.replace( QRegExp("\n{2,}"), "\n" ); //multiple \n characters may remain
-            text.replace( "&lt;",  "<" );
-            text.replace( "&gt;",  ">" );
-            text.replace( "&amp;", "&" ); //replace some common HTML type stuff
+            QStringx osd = AmarokConfig::osdText();
+            text = osd.namedOptArgs( args );
 
             // KDE 3.3 rejects \n in the .kcfg file, and KConfig turns \n into \\n, so...
             text.replace( "\\n", "\n" );
@@ -538,7 +532,12 @@ amaroK::OSD::show( const MetaBundle &bundle ) //slot
         if ( AmarokConfig::osdCover() ) {
             //avoid showing the generic cover.  we can overwrite this by passing an arg.
             //get large cover for scaling if big cover needed
-            QString location = CollectionDB::instance()->albumImage( bundle, 0 );
+
+            QString location = QString::null;
+            if( bundle.podcastBundle() ) 
+                location = CollectionDB::instance()->podcastImage( bundle, 0 );
+            else
+                location = CollectionDB::instance()->albumImage( bundle, 0 );
 
             if ( location.find( "nocover" ) != -1 )
                 setImage( amaroK::icon() );
@@ -552,6 +551,9 @@ amaroK::OSD::show( const MetaBundle &bundle ) //slot
     if( text.isEmpty() )
         text = MetaBundle::prettyTitle( bundle.url().fileName() ).stripWhiteSpace();
 
+    if( text.startsWith( "- " ) ) //When we only have a title tag, _something_ prepends a fucking hyphen. Remove that.
+        text = text.mid( 2 );
+
     if( text.isEmpty() ) //still
         text = i18n("No information available for this track");
 
@@ -561,7 +563,7 @@ amaroK::OSD::show( const MetaBundle &bundle ) //slot
 void
 amaroK::OSD::applySettings()
 {
-    setAlignment( (OSDWidget::Alignment)AmarokConfig::osdAlignment() );
+    setAlignment( static_cast<OSDWidget::Alignment>( AmarokConfig::osdAlignment() ) );
     setDuration( AmarokConfig::osdDuration() );
     setEnabled( AmarokConfig::osdEnabled() );
     setOffset( AmarokConfig::osdYOffset() );
@@ -591,6 +593,43 @@ amaroK::OSD::forceToggleOSD()
         hide();
 }
 
+void
+amaroK::OSD::slotCoverChanged( const QString &artist, const QString &album )
+{
+    if( AmarokConfig::osdCover() && artist == EngineController::instance()->bundle().artist()
+                                 && album  == EngineController::instance()->bundle().album()  )
+    {
+        QString location = CollectionDB::instance()->albumImage( artist, album, 0 );
+
+        if( location.find( "nocover" ) != -1 )
+            setImage( amaroK::icon() );
+        else
+            setImage( location );
+    }
+}
+
+void
+amaroK::OSD::slotImageChanged( const QString &remoteURL )
+{
+    QString url = EngineController::instance()->bundle().url().url();
+    PodcastEpisodeBundle peb;
+    if( CollectionDB::instance()->getPodcastEpisodeBundle( url, &peb ) )
+    {
+        PodcastChannelBundle pcb;
+        if( CollectionDB::instance()->getPodcastChannelBundle( peb.parent().url(), &pcb ) )
+        {
+            if( pcb.imageURL().url() == remoteURL )
+            {
+                QString location = CollectionDB::instance()->podcastImage( remoteURL, 0 );
+                if( location == CollectionDB::instance()->notAvailCover( 0 ) )
+                    setImage( amaroK::icon() );
+                else
+                    setImage( location );
+            }
+        }
+    }
+}
+
 
 /* Code copied from kshadowengine.cpp
  *
