@@ -45,8 +45,11 @@ email                : markey@web.de
 #include <kurldrag.h>
 #include <kwin.h>            //eventFilter()
 
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
+#ifdef HAVE_EXSCALIBAR
+#define WANT_MOODBAR AmarokConfig::showMoodbar()
+#else
+#define WANT_MOODBAR false
+#endif
 
 
 //simple function for fetching amarok images
@@ -106,7 +109,7 @@ PlayerWidget::PlayerWidget( QWidget *parent, const char *name, bool enablePlayli
         move( AmarokConfig::playerPos() );
 
     setModifiedPalette();
-    setFixedSize( 311, 140 );
+    setFixedSize( 311, WANT_MOODBAR ? 147 : 140 );
     setCaption( "amaroK" );
     setAcceptDrops( true );
 
@@ -122,7 +125,7 @@ PlayerWidget::PlayerWidget( QWidget *parent, const char *name, bool enablePlayli
 
     { //<NavButtons>
         //NOTE we use a layout for the buttons so resizing will be possible
-        m_pFrameButtons = createWidget<QHBox>( QRect(0, 118, 311, 22), this );
+        m_pFrameButtons = createWidget<QHBox>( QRect(0, WANT_MOODBAR ? 125 : 118, 311, 22), this );
 
         KActionCollection *ac =amaroK::actionCollection();
 
@@ -144,10 +147,10 @@ PlayerWidget::PlayerWidget( QWidget *parent, const char *name, bool enablePlayli
     } //</NavButtons>
 
     { //<Sliders>
-        m_pSlider    = new amaroK::PrettySlider( Qt::Horizontal, this );
+        m_pSlider    = new amaroK::TrackSlider( this );
         m_pVolSlider = new amaroK::PrettySlider( Qt::Vertical, this, amaroK::VOLUME_MAX );
 
-        m_pSlider->setGeometry( 4,103, 303,12 );
+        m_pSlider->setGeometry( 4,103, 303, WANT_MOODBAR ? 19 : 12 );
         m_pVolSlider->setGeometry( 294,18, 12,79 );
         m_pVolSlider->setValue( AmarokConfig::masterVolume() );
 
@@ -211,6 +214,8 @@ PlayerWidget::PlayerWidget( QWidget *parent, const char *name, bool enablePlayli
     //KWin::setOnAllDesktops( winId(), true );
 
     connect( m_pAnimTimer, SIGNAL( timeout() ), SLOT( drawScroll() ) );
+
+    TrackToolTip::instance()->addToWidget( m_pScrollFrame );
 }
 
 
@@ -218,6 +223,7 @@ PlayerWidget::~PlayerWidget()
 {
     AmarokConfig::setPlayerPos( pos() );
     AmarokConfig::setPlaylistWindowEnabled( m_pPlaylistButton->isOn() );
+    TrackToolTip::instance()->removeFromWidget( m_pScrollFrame );
 }
 
 
@@ -361,11 +367,13 @@ void PlayerWidget::engineNewMetaData( const MetaBundle &bundle, bool )
     m_pSlider->setEnabled( bundle.length() > 0 );
 
     m_rateString     = bundle.prettyBitrate();
-    const QString Hz = bundle.prettySampleRate( true );
+    QString Hz = bundle.prettySampleRate( true );
     if( !Hz.isEmpty() )
     {
-        if( !m_rateString.isEmpty() ) m_rateString += " - ";
-        m_rateString += Hz;
+        if( m_rateString.isEmpty() )
+            m_rateString = Hz;
+        else
+            m_rateString = i18n("%1 kBit - %2").arg( m_rateString, Hz );
     }
 
     QStringList list( bundle.prettyTitle() );
@@ -373,9 +381,6 @@ void PlayerWidget::engineNewMetaData( const MetaBundle &bundle, bool )
     if( bundle.length() ) list << bundle.prettyLength();
     setScroll( list );
 
-    //update image tooltip
-    TrackToolTip::add( m_pScrollFrame, bundle, EngineController::instance()->engine()->position() );
-
     update(); //we need to update rateString
 }
 
@@ -385,8 +390,12 @@ void PlayerWidget::engineTrackPositionChanged( long position, bool /*userSeek*/
     m_pSlider->setValue( position );
 
     if( !m_pSlider->isEnabled() ) timeDisplay( position );
+}
 
-    TrackToolTip::add( m_pScrollFrame, EngineController::instance()->bundle(), position );
+
+void PlayerWidget::engineTrackLengthChanged( long length )
+{
+    m_pSlider->setMaxValue( length * 1000 );
 }
 
 
@@ -475,6 +484,12 @@ void PlayerWidget::applySettings()
     default:
         engineNewMetaData( EngineController::instance()->bundle(), false );
     }
+
+    if(m_pAnalyzer)
+    {
+        setMinimalView(m_minimalView);
+        dynamic_cast<amaroK::TrackSlider *>(m_pSlider)->newMoodData();
+    }
 }
 
 void PlayerWidget::setMinimalView( bool enable )
@@ -491,17 +506,19 @@ void PlayerWidget::setMinimalView( bool enable )
     {
         uint space = 2;
         m_pScrollFrame->setGeometry ( 6,space,  m_pScrollFrame->width(),  m_pScrollFrame->height() );
-        m_pSlider->setGeometry      ( 4,space + m_pScrollFrame->height(), 303,12 );
+        m_pSlider->setGeometry      ( 4,space + m_pScrollFrame->height(), 303,WANT_MOODBAR ? 19 : 12 );
         m_pFrameButtons->setGeometry( 0,space + m_pScrollFrame->height() + m_pSlider->height(), 311,22 );
         uint height = m_pFrameButtons->height() + m_pScrollFrame->height() + m_pSlider->height() + space;
         setFixedSize( 311, height );
+        AmarokConfig::setPlayerWindowMinimalView( true );
     }
     else
     {
         m_pScrollFrame->setGeometry( 6,18, m_pScrollFrame->width(),m_pScrollFrame->height() );
-        m_pSlider->setGeometry( 4,103, 303,12 );
-        m_pFrameButtons->setGeometry(0, 118, 311,22);
-        setFixedSize( 311, 140 );
+        m_pSlider->setGeometry( 4,103, 303,WANT_MOODBAR ? 19 : 12 );
+        m_pFrameButtons->setGeometry(0, WANT_MOODBAR ? 125 : 118, 311,22);
+        setFixedSize( 311, WANT_MOODBAR ? 147 : 140 );
+        AmarokConfig::setPlayerWindowMinimalView( false );
     }
 
     m_minimalView = enable;
@@ -760,7 +777,7 @@ void PlayerWidget::mousePressEvent( QMouseEvent *e )
         if ( rect.contains( e->pos() ) )
         {
             AmarokConfig::setTimeDisplayRemaining( !AmarokConfig::timeDisplayRemaining() );
-            timeDisplay( EngineController::engine()->position() / 1000 );
+            timeDisplay( EngineController::engine()->position() );
         }
         else m_startDragPos = e->pos();
     }
