@@ -19,30 +19,40 @@
 #ifndef AMAROK_PLAYLIST_H
 #define AMAROK_PLAYLIST_H
 
+#include "amarok_export.h"
 #include "amarokconfig.h"
 #include "amarokdcophandler.h"
 #include "engineobserver.h"  //baseclass
+#include "dynamicmode.h"
 #include "playlistwindow.h"  //friend
 #include "playlistitem.h"
 #include "metabundle.h"
+#include "tooltip.h"         //baseclass
 #include "tracktooltip.h"
 
 #include <klistview.h>       //baseclass
 #include <kurl.h>            //KURL::List
 #include <qdir.h>            //stack allocated
-#include <qstringlist.h>     //stack allocated
+#include <qpoint.h>          //stack allocated
 #include <qptrlist.h>        //stack allocated
+#include <qstringlist.h>     //stack allocated
 #include <vector>            //stack allocated
 
 class KAction;
 class KActionCollection;
+class MyAtomicString;
 class PlaylistItem;
 class PlaylistEntry;
 class PlaylistLoader;
+class PlaylistAlbum;
 class QBoxLayout;
 class QLabel;
 class QTimer;
 
+class Medium;
+class ReadMood;
+namespace amaroK { class TrackSlider; }
+
 /**
  * @authors Mark Kretschmann && Max Howell
  *
@@ -60,14 +70,14 @@ class QTimer;
  */
 
 
-class Playlist : private KListView, public EngineObserver
+class Playlist : private KListView, public EngineObserver, public amaroK::ToolTipClient
 {
         Q_OBJECT
 
     public:
         ~Playlist();
 
-        static Playlist *instance() { return s_instance; }
+        LIBAMAROK_EXPORT static Playlist *instance() { return s_instance; }
         static QString defaultPlaylistPath();
         static const int NO_SORT = 200;
 
@@ -78,31 +88,50 @@ class Playlist : private KListView, public EngineObserver
         static const int DirectPlay = 8;     /// start playback of the first item in the list
         static const int Unique     = 16;    /// don't insert anything already in the playlist
 
+        // it's really just the *ListView parts we want to hide...
+        QScrollView *qscrollview() const
+        {
+            return reinterpret_cast<QScrollView*>( const_cast<Playlist*>( this ) );
+        }
+
         /** Add media to the playlist
          *  @param options you can OR these together, see the enum
          *  @param sql     Sql program to execute */
-        void insertMedia( KURL::List, int options = Append );
+        LIBAMAROK_EXPORT void insertMedia( KURL::List, int options = Append );
         void insertMediaSql( const QString& sql, int options = Append );
 
         /// Dynamic mode functions
-        void addSpecialTracks( uint songCount, const QString type = "Random" );
+        void addSpecialTracks( uint songCount, const int type = DynamicMode::RANDOM );
         void addSpecialCustomTracks( uint songCount );
-        void adjustPartyUpcoming( uint songCount, const QString type = "Random" );
-        void adjustPartyPrevious( uint songCount );
-        void advancePartyTrack( PlaylistItem *item = 0 );
+        void adjustDynamicUpcoming( bool saveUndo = false, const int type = DynamicMode::RANDOM );
+        void adjustDynamicPrevious( uint songCount, bool saveUndo = false );
+        void advanceDynamicTrack( PlaylistItem *item = 0 );
         void alterHistoryItems( bool enable = false, bool entire = false );
 
         void burnPlaylist      ( int projectType = -1 );
         void burnSelectedTracks( int projectType = -1 );
-        int  currentTrackIndex();
+        int  currentTrackIndex( bool onlyCountVisible = true );
         bool isEmpty()       const  { return childCount() == 0; }
         bool isTrackBefore() const;
         bool isTrackAfter()  const;
         void restoreSession();          // called during initialisation
         bool saveM3U( const QString&, bool relative = AmarokConfig::relativePlaylist() ) const;
         void saveXML( const QString& );
+        bool saveXSPF( const QString& );
         int  totalTrackCount() const;
         BundleList nextTracks() const;
+        uint repeatAlbumTrackCount() const;    //returns number of tracks from same album
+        //as current track that are in playlist (may require Play Albums in Order on). 
+        //If the information is not available, returns 0.
+
+        //const so you don't change it behind Playlist's back, use modifyDynamicMode() for that
+        const DynamicMode *dynamicMode() const;
+
+        //modify the returned DynamicMode, then finishedModifying() it when done
+        DynamicMode *modifyDynamicMode();
+
+        //call this every time you modifyDynamicMode(), otherwise you'll get memory leaks and/or crashes
+        void finishedModifying( DynamicMode *mode );
 
         int  stopAfterMode() const;
 
@@ -113,36 +142,55 @@ class Playlist : private KListView, public EngineObserver
         void setFont( const QFont &f ) { KListView::setFont( f ); } //made public for convenience
         void unsetFont()               { KListView::unsetFont(); }
 
-        PlaylistItem *firstChild() const { return (PlaylistItem*)KListView::firstChild(); }
-        PlaylistItem *lastItem()   const { return (PlaylistItem*)KListView::lastItem(); }
+        PlaylistItem *firstChild() const { return static_cast<PlaylistItem*>( KListView::firstChild() ); }
+        PlaylistItem *lastItem()   const { return static_cast<PlaylistItem*>( KListView::lastItem() ); }
+        PlaylistItem *currentItem() const { return static_cast<PlaylistItem*>( KListView::currentItem() ); }
 
-        int  visibleColumns() const;
-        int  mapToLogicalColumn( int physical ); // Converts physical PlaylistItem column position to logical
+        int  numVisibleColumns() const;
+        QValueList<int> visibleColumns() const;
+        int  mapToLogicalColumn( int physical ) const; // Converts physical PlaylistItem column position to logical
         QString columnText( int c ) const { return KListView::columnText( c ); };
+        void setColumns( QValueList<int> order, QValueList<int> visible );
 
         /** Call this to prevent items being removed from the playlist, it is mostly for internal use only
          *  Dont forget to unlock() !! */
         void lock();
         void unlock();
 
+        //reimplemented to save columns by name instead of index, to be more resilient to reorderings and such
+        void saveLayout(KConfig *config, const QString &group) const;
+        void restoreLayout(KConfig *config, const QString &group);
+
+        //ATF-related functions
+        bool checkFileStatus( PlaylistItem * item );
+
         enum RequestType { Prev = -1, Current = 0, Next = 1 };
         enum StopAfterMode { DoNotStop, StopAfterCurrent, StopAfterQueue, StopAfterOther };
 
         class QDragObject *dragObject();
+        friend class amaroK::TrackSlider;
+        friend class PlaylistItem::ReadMood;
         friend class PlaylistItem;
         friend class UrlLoader;
         friend class QueueManager;
         friend class QueueLabel;
+        friend class PlaylistWindow;
+        friend class ColumnList;
         friend void amaroK::DcopPlaylistHandler::removeCurrentTrack(); //calls removeItem() and currentTrack()
+        friend void amaroK::DcopPlaylistHandler::removeByIndex( int ); //calls removeItem()
         friend void PlaylistWindow::init(); //setting up connections etc.
         friend TrackToolTip::TrackToolTip();
         friend bool PlaylistWindow::eventFilter( QObject*, QEvent* ); //for convenience we handle some playlist events here
 
+    public:
+        QPair<QString, QRect> toolTipText( QWidget*, const QPoint &pos ) const;
+
     signals:
         void aboutToClear();
         void itemCountChanged( int newCount, int newLength, int visCount, int visLength, int selCount, int selLength );
         void queueChanged( const PLItemList &queued, const PLItemList &dequeued );
         void columnsChanged();
+        void dynamicModeChanged( const DynamicMode *newMode );
 
     public slots:
         void activateByIndex(int);
@@ -151,37 +199,62 @@ class Playlist : private KListView, public EngineObserver
         void appendMedia( const QString &path );
         void clear();
         void copyToClipboard( const QListViewItem* = 0 ) const;
-        void countChanged( const QString &path );
         void deleteSelectedFiles();
+        void trashSelectedFiles();
+        void ensureItemCentered( QListViewItem* item );
         void playCurrentTrack();
         void playNextTrack( const bool forceNext = true );
         void playPrevTrack();
         void queueSelected();
+        void setSelectedRatings( int rating );
         void redo();
         void removeDuplicates();
         void removeSelectedItems();
+        void setDynamicMode( DynamicMode *mode );
+        void loadDynamicMode( DynamicMode *mode ); //saveUndoState() + setDynamicMode()
+        void disableDynamicMode() { setDynamicMode( 0 ); }
+        void editActiveDynamicMode();
         void repopulate();
+        void safeClear();
         void scoreChanged( const QString &path, int score );
+        void ratingChanged( const QString &path, int rating );
+        void fileMoved( const QString &srcPath, const QString &dstPath );
         void selectAll() { QListView::selectAll( true ); }
-        void setFilter( const QString &filter );                           //for the entire playlist
-        void setFilterForItem( const QString &query, PlaylistItem *item ); //for a single item
+        void setFilter( const QString &filter );
         void setFilterSlot( const QString &filter );                       //uses a delay where applicable
         void setStopAfterCurrent( bool on );
+        void toggleStopAfterCurrentItem();
+        void toggleStopAfterCurrentTrack();
         void setStopAfterMode( int mode );
-        void ensureItemCentered( QListViewItem* item );
         void showCurrentTrack() { ensureItemCentered( m_currentTrack ); }
         void showQueueManager();
         void shuffle();
         void undo();
         void updateMetaData( const MetaBundle& );
+        void fileHasMood( const QString path );
+        void refreshMoods();
+        void applySettings();
+        void adjustColumn( int n );
+
+    protected:
+        virtual void fontChange( const QFont &old );
+
+    protected slots:
+        void contentsMouseMoveEvent( QMouseEvent *e = 0 );
+        void leaveEvent( QEvent *e );
+        void contentsMousePressEvent( QMouseEvent *e );
+        void contentsWheelEvent( QWheelEvent *e );
 
     private slots:
+        void mediumChange( const Medium *medium, QString name );
         void slotCountChanged();
         void activate( QListViewItem* );
         void columnOrderChanged();
         void columnResizeEvent( int, int, int );
         void doubleClicked( QListViewItem* );
 
+        void generateInfo(); //generates info for Random Albums
+
         void queue( QListViewItem*, bool multi = false );
            /* the only difference multi makes is whether it emits queueChanged(). (if multi, then no)
               if you're queue()ing many items, consider passing true and emitting queueChanged() yourself. */
@@ -193,19 +266,20 @@ class Playlist : private KListView, public EngineObserver
         void slotGlowTimer();
         void reallyEnsureItemCentered();
         void slotMouseButtonPressed( int, QListViewItem*, const QPoint&, int );
-        void slotRepeatTrackToggled( bool enabled );
+        void slotSingleClick();
+        void slotContentsMoving();
+        void slotRepeatTrackToggled( int mode );
         void slotQueueChanged( const PLItemList &in, const PLItemList &out);
+        void slotUseScores( bool use );
+        void slotUseRatings( bool use );
         void updateNextPrev();
         void writeTag( QListViewItem*, const QString&, int );
 
-
     private:
         Playlist( QWidget* );
         Playlist( const Playlist& ); //not defined
 
-        static Playlist *s_instance;
-
-        typedef QMap<QString, QString> QStringMap;
+        LIBAMAROK_EXPORT static Playlist *s_instance;
 
         void countChanged();
 
@@ -214,7 +288,6 @@ class Playlist : private KListView, public EngineObserver
 
         void insertMediaInternal( const KURL::List&, PlaylistItem*, bool directPlay = false );
         bool isAdvancedQuery( const QString &query );
-        bool googleMatch( QString query, const QStringMap &all, const QStringMap &defaults );
         void refreshNextTracks( int = -1 );
         void removeItem( PlaylistItem*, bool = false );
         bool saveState( QStringList& );
@@ -224,6 +297,7 @@ class Playlist : private KListView, public EngineObserver
         void sortQueuedItems();
         void switchState( QStringList&, QStringList& );
         void saveSelectedAsPlaylist();
+        void initStarPixmaps();
 
         //engine observer functions
         void engineNewMetaData( const MetaBundle&, bool );
@@ -249,13 +323,26 @@ class Playlist : private KListView, public EngineObserver
         void viewportPaintEvent( QPaintEvent* );
         void viewportResizeEvent( QResizeEvent* );
 
+        void appendToPreviousTracks( PlaylistItem *item );
+        void appendToPreviousAlbums( PlaylistAlbum *album );
+        void removeFromPreviousTracks( PlaylistItem *item = 0 );
+        void removeFromPreviousAlbums( PlaylistAlbum *album = 0 );
+
+        typedef QMap<MyAtomicString, PlaylistAlbum*> AlbumMap;
+        typedef QMap<MyAtomicString, AlbumMap> ArtistAlbumMap;
+        ArtistAlbumMap m_albums;
+        uint m_startupTime_t; //QDateTime::currentDateTime().toTime_t as of startup
+        uint m_oldestTime_t; //the createdate of the oldest song in the collection
+
 
         /// ATTRIBUTES
 
         PlaylistItem  *m_currentTrack;          //the track that is playing
         QListViewItem *m_marker;                //track that has the drag/drop marker under it
+        PlaylistItem  *m_hoveredRating;         //if the mouse is hovering over the rating of an item
 
         //NOTE these container types were carefully chosen
+        QPtrList<PlaylistAlbum> m_prevAlbums; //the previously played albums in Entire Albums mode
         PLItemList m_prevTracks;    //the previous history
         PLItemList m_nextTracks;    //the tracks to be played after the current track
 
@@ -265,6 +352,8 @@ class Playlist : private KListView, public EngineObserver
 
         PLItemList m_itemsToChangeTagsFor;
 
+        bool          m_smartResizing;
+
         int           m_firstColumn;
         int           m_totalCount;
         int           m_totalLength;
@@ -272,6 +361,7 @@ class Playlist : private KListView, public EngineObserver
         int           m_selLength;
         int           m_visCount;
         int           m_visLength;
+        Q_INT64       m_total; //for Favor Tracks
         bool          m_itemCountDirty;
 
         KAction      *m_undoButton;
@@ -283,17 +373,22 @@ class Playlist : private KListView, public EngineObserver
         QStringList   m_redoList;
         uint          m_undoCounter;
 
+        DynamicMode  *m_dynamicMode;
         KURL::List    m_queueList;
         PlaylistItem *m_stopAfterTrack;
         bool          m_showHelp;
         bool          m_stateSwitched;
-        bool          m_partyDirt;          //So we dont call advancePartyTrack() on activate()
+        bool          m_dynamicDirt;          //So we dont call advanceDynamicTrack() on activate()
         bool          m_queueDirt;          //When queuing disabled items, we need to place the marker on the newly inserted item
         bool          m_undoDirt;           //Make sure we dont repopulate the playlist when dynamic mode and undo()
 
         QListViewItem *m_itemToReallyCenter;
         QListViewItem *m_renameItem;
         int            m_renameColumn;
+        QTimer        *m_clicktimer;
+        QListViewItem *m_itemToRename;
+        QPoint         m_clickPos;
+        int            m_columnToRename;
 
         QMap<QString, QStringList> m_customSubmenuItem;
         QMap<int, QString>         m_customIdItem;
@@ -306,6 +401,26 @@ class Playlist : private KListView, public EngineObserver
         QString m_editOldTag; //text before inline editing ( the new tag is written only if it's changed )
 
         std::vector<double> m_columnFraction;
+
+        bool m_atfEnabled;
+};
+
+class MyAtomicString: public AtomicString
+{
+public:
+    MyAtomicString() { }
+    MyAtomicString(const QString &string): AtomicString( string ) { }
+    MyAtomicString(const AtomicString &other): AtomicString( other ) { }
+    bool operator<(const AtomicString &other) const { return ptr() < other.ptr(); }
+};
+
+class PlaylistAlbum
+{
+public:
+    PLItemList tracks;
+    int refcount;
+    Q_INT64 total; //for Favor Tracks
+    PlaylistAlbum(): refcount( 0 ), total( 0 ) { }
 };
 
 #endif //AMAROK_PLAYLIST_H
