@@ -1,7 +1,11 @@
 // Author: Max Howell (C) Copyright 2003-4
 // Author: Mark Kretschmann (C) Copyright 2004
 // .ram file support from Kaffeine 0.5, Copyright (C) 2004 by Jürgen Kofler (GPL 2 or later)
+// .asx file support added by Michael Seiwert Copyright (C) 2006
+// .asx file support from Kaffeine, Copyright (C) 2004-2005 by Jürgen Kofler (GPL 2 or later)
+// .smil file support from Kaffeine 0.7
 // .pls parser (C) Copyright 2005 by Michael Buesch <mbuesch@freenet.de>
+// .xspf file support added by Mattias Fliesberg <mattias.fliesberg@gmail.com> Copyright (C) 2006
 // Copyright: See COPYING file that comes with this distribution
 //
 
@@ -14,10 +18,14 @@
 #include "collectiondb.h"
 #include "debug.h"
 #include "enginecontroller.h"
+#include "mydirlister.h"
 #include "playlist.h"
+#include "playlistbrowser.h"
 #include "playlistitem.h"
 #include "playlistloader.h"
 #include "statusbar.h"
+#include "contextbrowser.h"
+#include "xspfplaylist.h"
 
 #include <qdatetime.h>   //::recurse()
 #include <qeventloop.h>  //::recurse()
@@ -29,23 +37,44 @@
 
 #include <dcopref.h>
 #include <kapplication.h>
-#include <kdirlister.h>
 #include <kurl.h>
 
 
 
-typedef QValueList<QDomNode> NodeList;
-
-
 //TODO playlists within playlists, local or remote are legal entries in m3u and pls
 //TODO directories from inside playlists
 
+struct XMLData
+{
+    MetaBundle bundle;
+    int queue;
+    bool stopafter;
+    bool disabled;
+    XMLData(): queue(-1), stopafter(false), disabled(false) { }
+};
+
+class TagsEvent : public QCustomEvent {
+public:
+    TagsEvent( const QValueList<XMLData> &x ) : QCustomEvent( 1001 ), xml( x ) {}
+    TagsEvent( const BundleList &bees ) : QCustomEvent( 1000 ), bundles( bees ) {
+        for( BundleList::Iterator it = bundles.begin(), end = bundles.end(); it != end; ++it )
+            /// @see MetaBundle for explanation of audioproperties < 0
+            if( (*it).length() <= 0 || (*it).bitrate() <= 0 )
+                (*it).readTags( TagLib::AudioProperties::Fast );
+    }
+
+    QValueList<XMLData> xml;
+    BundleList bundles;
+};
+
 
 UrlLoader::UrlLoader( const KURL::List &urls, QListViewItem *after, bool playFirstUrl )
         : ThreadWeaver::DependentJob( Playlist::instance(), "UrlLoader" )
         , m_markerListViewItem( new PlaylistItem( Playlist::instance(), after ) )
         , m_playFirstUrl( playFirstUrl )
         , m_block( "UrlLoader" )
+        , m_oldQueue( Playlist::instance()->m_nextTracks )
+        , m_xmlSource( 0 )
 {
     DEBUG_BLOCK
 
@@ -64,8 +93,6 @@ UrlLoader::UrlLoader( const KURL::List &urls, QListViewItem *after, bool playFir
             .setAbortSlot( this, SLOT(abort()) )
             .setTotalSteps( 100 );
 
-    m_markerListViewItem->setText( 0, "MARKERITEM" );
-
     foreachType( KURL::List, urls ) {
         const KURL &url = *it;
         const QString protocol = url.protocol();
@@ -77,10 +104,16 @@ UrlLoader::UrlLoader( const KURL::List &urls, QListViewItem *after, bool playFir
                 m_URLs += url;
         }
 
-       else if( protocol == "media" ) {
+        // Note: remove for kde 4 - we don't need to be hacking around KFileDialog,
+        // it has been fixed for kde 3.5.3
+        else if( protocol == "media" || url.url().startsWith( "system:/media/" ) )
+        {
+            QString path = url.path( -1 );
+            if( url.url().startsWith( "system:/media/" ) )
+                path = path.mid( 6 );
             // url looks like media:/device/path
             DCOPRef mediamanager( "kded", "mediamanager" );
-            QString device = url.path( -1 ).mid( 1 ); // remove first slash
+            QString device = path.mid( 1 ); // remove first slash
             const int slash = device.find( '/' );
             const QString filePath = device.mid( slash ); // extract relative path
             device = device.left( slash ); // extract device
@@ -104,59 +137,17 @@ UrlLoader::UrlLoader( const KURL::List &urls, QListViewItem *after, bool playFir
             m_playFirstUrl = false;
         }
 
-        else if( protocol == "fetchcover" ) {
-            // url looks like:   fetchcover:<artist_name> @@@ <album_name>
-           QString myUrl = url.path();
-           if ( myUrl.endsWith( " @@@" ) )
-               myUrl += ' ';
-           const QStringList list = QStringList::split( " @@@ ", myUrl, true );
-           Q_ASSERT( !list.isEmpty() );
-           QString artist_name = list.front();
-           QString album_name  = list.back();
-
-           QStringList trackUrls = CollectionDB::instance()->albumTracks( artist_name, album_name, true );
-           KURL url;
-           foreach( trackUrls ) {
-                url.setPath( *it );
-                m_URLs += url;
-           }
-        }
         else if( protocol == "seek" )
             continue;
 
-        else if( protocol == "album" ) {
-           // url looks like:   album:<artist_id> @@@ <album_id>
-           QString myUrl = url.path();
-           if ( myUrl.endsWith( " @@@" ) )
-               myUrl += ' ';
-           const QStringList list = QStringList::split( " @@@ ", myUrl, true );
-           Q_ASSERT( !list.isEmpty() );
-           QString artist_id = list.front();
-           QString album_id = list.back();
+        else if( ContextBrowser::hasContextProtocol( url ) )
+        {
+            DEBUG_BLOCK
+            debug() << "context expandurl" << endl;
 
-           QStringList trackUrls = CollectionDB::instance()->albumTracks( artist_id, album_id );
-           KURL url;
-           foreach( trackUrls ) {
-                url.setPath( *it );
-                m_URLs += url;
-           }
+            m_URLs += ContextBrowser::expandURL( url );
         }
-        else if( protocol == "compilation" ) {
-            QueryBuilder qb;
-            qb.addReturnValue( QueryBuilder::tabSong, QueryBuilder::valURL );
-            qb.addMatch( QueryBuilder::tabSong, QueryBuilder::valAlbumID, url.path() );
-            qb.sortBy( QueryBuilder::tabSong, QueryBuilder::valTrack );
-            qb.setOptions( QueryBuilder::optOnlyCompilations );
-            QStringList values = qb.run();
 
-            KURL::List urls;
-            KURL url;
-
-            for( QStringList::ConstIterator it = values.begin(), end = values.end(); it != end; ++it ) {
-                url.setPath( *it );
-                m_URLs += url;
-            }
-        }
         else {
             // this is the best way I found for recursing if required
             // and not recusring if not required
@@ -176,23 +167,9 @@ UrlLoader::~UrlLoader()
     Playlist::instance()->unlock();
 
     delete m_markerListViewItem;
+    delete m_xmlSource;
 }
 
-
-class TagsEvent : public QCustomEvent {
-public:
-    TagsEvent() : QCustomEvent( 1001 ) {}
-    TagsEvent( const BundleList &bees ) : QCustomEvent( 1000 ), bundles( bees ) {
-        for( BundleList::Iterator it = bundles.begin(), end = bundles.end(); it != end; ++it )
-            /// @see MetaBundle for explanation of audioproperties < 0
-            if( (*it).length() <= 0 || (*it).bitrate() <= 0 )
-                (*it).readTags( TagLib::AudioProperties::Fast );
-    }
-
-    NodeList nodes;
-    BundleList bundles;
-};
-
 bool
 UrlLoader::doJob()
 {
@@ -215,7 +192,7 @@ UrlLoader::doJob()
             PlaylistFile playlist( url.path() );
 
             if( !playlist.isError() )
-                QApplication::postEvent( this, new TagsEvent( playlist.bundles() ) );
+                QApplication::postEvent( this, new TagsEvent( playlist.bundles()) );
             else
                 m_badURLs += url;
 
@@ -235,8 +212,9 @@ UrlLoader::doJob()
 }
 
 void
-UrlLoader::customEvent( QCustomEvent *e )
+UrlLoader::customEvent( QCustomEvent *e)
 {
+    bool atfEnabled = AmarokConfig::advancedTagFeatures();
     #define e static_cast<TagsEvent*>(e)
     switch( e->type() ) {
     case 1000:
@@ -244,55 +222,57 @@ UrlLoader::customEvent( QCustomEvent *e )
         {
             //Only add files that exist to the playlist
             if( !(*it).exists() )
-                continue;
+            {
+                if( !atfEnabled )
+                    continue;
 
-            PlaylistItem *item = new PlaylistItem( *it, m_markerListViewItem );
+                //add anyways, but disable
+                new PlaylistItem( *it, m_markerListViewItem, false );
+            }
+            else
+            {
+                PlaylistItem *item = new PlaylistItem( *it, m_markerListViewItem );
 
-            if( m_playFirstUrl ) {
-                Playlist::instance()->activate( item );
-                m_playFirstUrl = false;
+                if( m_playFirstUrl ) {
+                    Playlist::instance()->activate( item );
+                    m_playFirstUrl = false;
+                }
             }
         }
         break;
 
     case 1001:
     {
-        const QString QUEUE_INDEX = "queue_index";
-        const QString STOP_AFTER = "stop_after";
-        const QString DISABLED = "disabled";
-        foreachType( NodeList, e->nodes )
+        foreachType( QValueList<XMLData>, e->xml )
         {
-            if( (*it).isNull() ) //safety
+            if( (*it).bundle.isEmpty() ) //safety
                 continue;
 
-            const QDomElement element = QDomNode( *it ).toElement(); //we check it is an element in loadPlaylist
-            PlaylistItem* const item = new PlaylistItem( *it, m_markerListViewItem );
+            PlaylistItem* const item = new PlaylistItem( (*it).bundle, m_markerListViewItem );
 
             //TODO scrollbar position
             //TODO previous tracks queue
             //TODO current track position, even if user doesn't have resume playback turned on
 
-            if( element.hasAttribute( QUEUE_INDEX ) ) {
-                const int index = element.attribute( QUEUE_INDEX ).toInt();
-
-                if( index == 0 )
+            if( (*it).queue >= 0 ) {
+                if( (*it).queue == 0 )
                     Playlist::instance()->setCurrentTrack( item );
 
-                else if( index > 0 ) {
+                else if( (*it).queue > 0 ) {
                     PLItemList &m_nextTracks = Playlist::instance()->m_nextTracks;
                     int count = m_nextTracks.count();
 
-                    for( int c = count; c < index; c++ )
+                    for( int c = count; c < (*it).queue; c++ )
                         // Append foo values and replace with correct values later.
                         m_nextTracks.append( item );
 
-                    m_nextTracks.replace( index - 1, item );
+                    m_nextTracks.replace( (*it).queue - 1, item );
                 }
             }
-            if( element.hasAttribute( STOP_AFTER ) )
+            if( (*it).stopafter )
                 Playlist::instance()->m_stopAfterTrack = item;
 
-            if( element.hasAttribute( DISABLED ) )
+            if( (*it).disabled )
                 item->setEnabled( false );
         }
         break;
@@ -308,16 +288,35 @@ UrlLoader::customEvent( QCustomEvent *e )
 void
 UrlLoader::completeJob()
 {
-    if ( !m_badURLs.isEmpty() ) {
-        amaroK::StatusBar::instance()->shortLongMessage(
-                i18n("Some media could not be loaded (not playable)."),
-                i18n("These media could not be loaded into the playlist: " ) );
+    const PLItemList &newQueue = Playlist::instance()->m_nextTracks;
+    QPtrListIterator<PlaylistItem> it( newQueue );
+    PLItemList added;
+    for( it.toFirst(); *it; ++it )
+        if( !m_oldQueue.containsRef( *it ) )
+            added << (*it);
 
+    if( !added.isEmpty() )
+        emit queueChanged( added, PLItemList() );
+
+    if ( !m_badURLs.isEmpty() ) {
+        QString text = i18n("These media could not be loaded into the playlist: " );
         debug() << "The following urls were not suitable for the playlist:" << endl;
         for ( uint it = 0; it < m_badURLs.count(); it++  )
+        {
+            if( it < 5 )
+                text += QString("<br>%1").arg( m_badURLs[it].prettyURL() );
+            else if( it == 5 )
+                text += QString("<br>Plus %1 more").arg( m_badURLs.count() - it );
             debug() << "\t" << m_badURLs[it] << endl;
+        }
+
+        amaroK::StatusBar::instance()->shortLongMessage(
+                i18n("Some media could not be loaded (not playable)."), text );
     }
 
+    if( !m_dynamicMode.isEmpty() )
+        Playlist::instance()->setDynamicMode( PlaylistBrowser::instance()->findDynamicModeByTitle( m_dynamicMode ) );
+
     //syncronous, ie not using eventLoop
     QApplication::sendEvent( dependent(), this );
 }
@@ -359,6 +358,74 @@ UrlLoader::recurse( const KURL &url )
     return urls;
 }
 
+namespace amaroK
+{
+
+// almost the same as UrlLoader::recurse, but global
+KURL::List
+recursiveUrlExpand( const KURL &url, int maxURLs )
+{
+    typedef QMap<QString, KURL> FileMap;
+
+    if( url.protocol() != "file" || !QFileInfo( url.path() ).isDir() )
+        return KURL::List( url );
+
+    MyDirLister lister( false );
+    lister.setAutoUpdate( false );
+    lister.setAutoErrorHandlingEnabled( false, 0 );
+    if ( !lister.openURL( url ) )
+        return KURL::List();
+
+    // Fucking KDirLister sometimes hangs on remote media, so we add a timeout
+    const int timeout = 3000; // ms
+    QTime watchdog;
+    watchdog.start();
+
+    while( !lister.isFinished() && watchdog.elapsed() < timeout )
+        kapp->eventLoop()->processEvents( QEventLoop::ExcludeUserInput );
+
+    KFileItemList items = lister.items(); //returns QPtrList, so we MUST only do it once!
+    KURL::List urls;
+    FileMap files;
+    for( KFileItem *item = items.first(); item; item = items.next() ) {
+        if( maxURLs >= 0 && (int)(urls.count() + files.count()) >= maxURLs )
+            break;
+        if( item->isFile()
+                && !PlaylistFile::isPlaylistFile( item->url().fileName() )
+                )
+        {
+            files[item->name()] = item->url();
+            continue;
+        }
+        if( item->isDir() )
+            urls += recursiveUrlExpand( item->url(), maxURLs - urls.count() - files.count() );
+    }
+
+    foreachType( FileMap, files )
+        // users often have playlist files that reflect directories
+        // higher up, or stuff in this directory. Don't add them as
+        // it produces double entries
+        urls += *it;
+
+    return urls;
+}
+
+KURL::List
+recursiveUrlExpand( const KURL::List &list, int maxURLs )
+{
+    KURL::List urls;
+    foreachType( KURL::List, list )
+    {
+        if( maxURLs >= 0 && (int)urls.count() >= maxURLs )
+            break;
+        urls += recursiveUrlExpand( *it, maxURLs - urls.count() );
+    }
+
+    return urls;
+}
+
+} // amaroK
+
 void
 UrlLoader::loadXml( const KURL &url )
 {
@@ -367,53 +434,73 @@ UrlLoader::loadXml( const KURL &url )
         m_badURLs += url;
         return;
     }
+    m_currentURL = url;
 
-    QTextStream stream( &file );
-    stream.setEncoding( QTextStream::UnicodeUTF8 );
-
-    QDomDocument d;
-    QString er;
-    int l, c;
-    if( !d.setContent( stream.read(), &er, &l, &c ) ) { // return error values
+    delete m_xmlSource;
+    m_xmlSource = new QXmlInputSource( file );
+    MyXmlLoader loader;
+    connect( &loader, SIGNAL( newBundle( const MetaBundle&, const XmlAttributeList& ) ),
+             this, SLOT( slotNewBundle( const MetaBundle&, const XmlAttributeList& ) ) );
+    connect( &loader, SIGNAL( playlistInfo( const QString&, const QString&, const QString& ) ),
+             this, SLOT( slotPlaylistInfo( const QString&, const QString&, const QString& ) ) );
+    loader.load( m_xmlSource );
+    if( !m_xml.isEmpty() )
+    {
+        QApplication::postEvent( this, new TagsEvent( m_xml ) );
+        m_xml.clear();
+    }
+    if( !loader.lastError().isEmpty() )
+    {
         amaroK::StatusBar::instance()->longMessageThreadSafe( i18n(
                 //TODO add a link to the path to the playlist
                 "The XML in the playlist was invalid. Please report this as a bug to the amaroK "
                 "developers. Thank you." ), KDE::StatusBar::Error );
-        error() << "[PLAYLISTLOADER]: Error loading xml file: " << url.prettyURL() << "(" << er << ")"
-                << " at line " << l << ", column " << c << endl;
-        return;
+        ::error() << "[PLAYLISTLOADER]: Error in " << m_currentURL.prettyURL() << ": " << loader.lastError() << endl;
     }
-
-    const PLItemList oldQueue = Playlist::instance()->m_nextTracks;
-    NodeList nodes;
-    TagsEvent e;
-    const QString ITEM( "item" ); //so we don't construct this QString all the time
-    for( QDomNode n = d.namedItem( "playlist" ).firstChild(); !n.isNull(); n = n.nextSibling() )
-    {
-        if( n.nodeName() != ITEM ) continue;
-
-        if( !n.toElement().isNull() )
-            e.nodes += n;
-
-        if( e.nodes.count() == OPTIMUM_BUNDLE_COUNT ) {
-            QApplication::sendEvent( this, &e );
-            e = TagsEvent();
-        }
-    }
-    QApplication::sendEvent( this, &e );
-
-    const PLItemList &newQueue = Playlist::instance()->m_nextTracks;
-    QPtrListIterator<PlaylistItem> it( newQueue );
-    PLItemList added;
-    for( it.toFirst(); *it; ++it )
-        if( !oldQueue.containsRef( *it ) )
-            added << (*it);
-
-    if( !added.isEmpty() )
-        emit queueChanged( added, PLItemList() );
 }
 
+void UrlLoader::slotNewBundle( const MetaBundle &bundle, const XmlAttributeList &atts )
+{
+    XMLData data;
+    data.bundle = bundle;
+    for( int i = 0, n = atts.count(); i < n; ++i )
+    {
+        if( atts[i].first == "queue_index" )
+        {
+            bool ok = true;
+            data.queue = atts[i].second.toInt( &ok );
+            if( !ok )
+                data.queue = -1;
+        }
+        else if( atts[i].first == "stop_after" )
+            data.stopafter = true;
+        else if( atts[i].first == "disabled" )
+            data.disabled = true;
+    }
+    data.bundle.checkExists();
+    m_xml.append( data );
+    if( m_xml.count() == OPTIMUM_BUNDLE_COUNT )
+    {
+        QApplication::postEvent( this, new TagsEvent( m_xml ) );
+        m_xml.clear();
+    }
+}
 
+void UrlLoader::slotPlaylistInfo( const QString &, const QString &version, const QString &dynamicMode )
+{
+    if( version != amaroK::xmlVersion() )
+    {
+        amaroK::StatusBar::instance()->longMessageThreadSafe( i18n(
+            "Your last playlist was saved with a different version of amaroK than this one, "
+            "and this version can no longer read it.\n"
+            "You will have to create a new one.\n"
+            "Sorry :(" ) );
+        static_cast<MyXmlLoader*>( const_cast<QObject*>( sender() ) )->abort(); //HACK?
+        return;
+    }
+    else
+        m_dynamicMode = dynamicMode;
+}
 
 /// @class PlaylistFile
 
@@ -435,7 +522,9 @@ PlaylistFile::PlaylistFile( const QString &path )
         m_error = i18n( "This component of amaroK cannot translate XML playlists." );
         return;
     case RAM: loadRealAudioRam( stream ); break;
-    case ASX:
+    case ASX: loadASX( stream ); break;
+    case SMIL: loadSMIL( stream ); break;
+	case XSPF: loadXSPF( stream ); break;
     default:
         m_error = i18n( "amaroK does not support this playlist format." );
         return;
@@ -469,10 +558,14 @@ PlaylistFile::loadM3u( QTextStream &stream )
             if( url.startsWith( "/" ) )
                 url.prepend( "file://" );
 
-            if( KURL::isRelativeURL( url ) )
-                b.setPath( directory + line );
-            else
+            if( KURL::isRelativeURL( url ) ) {
+                KURL kurl( KURL::fromPathOrURL( directory + line ) );
+                kurl.cleanPath();
+                b.setPath( kurl.path() );
+            }
+            else {
                 b.setUrl( KURL::fromPathOrURL( line ) );
+            }
 
             m_bundles += b;
             b = MetaBundle();
@@ -603,6 +696,38 @@ PlaylistFile::loadPls( QTextStream &stream )
     return true;
 }
 
+bool
+PlaylistFile::loadXSPF( QTextStream &stream )
+{
+    MetaBundle b;
+
+    XSPFPlaylist* doc = new XSPFPlaylist( stream );
+
+    XSPFtrackList trackList = doc->trackList();
+
+    foreachType( XSPFtrackList, trackList )
+    {
+        if ( !(*it).location.url().isNull() )
+        {
+            if ( (*it).title.isNull() )
+                b.setTitle( (*it).location.fileName() );
+            else
+                b.setTitle( (*it).title );
+
+            b.setUrl( (*it).location );
+            b.setComment( (*it).annotation );
+            b.setAlbum( (*it).album );
+            if ( (*it).trackNum > 0 )
+                b.setTrack( (*it).trackNum );
+
+            m_bundles += b;
+            b = MetaBundle();
+        }
+    }
+
+    return true;
+}
+
 unsigned int
 PlaylistFile::loadPls_extractIndex( const QString &str ) const
 {
@@ -640,6 +765,151 @@ PlaylistFile::loadRealAudioRam( QTextStream &stream )
 
     return true;
 }
+
+bool
+PlaylistFile::loadASX( QTextStream &stream )
+{
+    //adapted from Kaffeine 0.7
+    MetaBundle b;
+    QDomDocument doc;
+    QString errorMsg;
+    int errorLine, errorColumn;
+    stream.setEncoding( QTextStream::UnicodeUTF8 );
+    if (!doc.setContent(stream.read(), &errorMsg, &errorLine, &errorColumn))
+    {
+        debug() << "[PLAYLISTLOADER]: Error loading xml file: " "(" << errorMsg << ")"
+                << " at line " << errorLine << ", column " << errorColumn << endl;
+        return false;
+    }
+
+    QDomElement root = doc.documentElement();
+
+    QString url;
+    QString title;
+    QString author;
+    QTime length;
+    QString duration;
+
+    if (root.nodeName().lower() != "asx") return false;
+
+    QDomNode node = root.firstChild();
+    QDomNode subNode;
+    QDomElement element;
+
+    while (!node.isNull())
+    {
+       url = QString::null;
+       title = QString::null;
+       author = QString::null;
+       length = QTime();
+       if (node.nodeName().lower() == "entry")
+       {
+          subNode = node.firstChild();
+          while (!subNode.isNull())
+          {
+             if ((subNode.nodeName().lower() == "ref") && (subNode.isElement()) && (url.isNull()))
+             {
+                element = subNode.toElement();
+                if (element.hasAttribute("href"))
+                   url = element.attribute("href");
+                if (element.hasAttribute("HREF"))
+                   url = element.attribute("HREF");
+                if (element.hasAttribute("Href"))
+                   url = element.attribute("Href");
+                if (element.hasAttribute("HRef"))
+                   url = element.attribute("HRef");
+              }
+		if ((subNode.nodeName().lower() == "duration") && (subNode.isElement()))
+                {
+                   duration = QString::null;
+                   element = subNode.toElement();
+                   if (element.hasAttribute("value"))
+                      duration = element.attribute("value");
+                   if (element.hasAttribute("Value"))
+                      duration = element.attribute("Value");
+                   if (element.hasAttribute("VALUE"))
+                      duration = element.attribute("VALUE");
+
+                   if (!duration.isNull())
+                      length = PlaylistFile::stringToTime(duration);
+                  }
+
+                  if ((subNode.nodeName().lower() == "title") && (subNode.isElement()))
+                  {
+                     title = subNode.toElement().text();
+                  }
+                  if ((subNode.nodeName().lower() == "author") && (subNode.isElement()))
+                  {
+                     author = subNode.toElement().text();
+                  }
+		  subNode = subNode.nextSibling();
+	  }
+	  if (!url.isNull())
+          {
+	     if (title.isNull())
+                title = url;
+                b.setUrl(KURL(url));
+                m_bundles += b;
+                b = MetaBundle();
+	   }
+        }
+	node = node.nextSibling();
+     }
+     return true;
+}
+
+bool
+PlaylistFile::loadSMIL( QTextStream &stream )
+{
+	// adapted from Kaffeine 0.7
+	QDomDocument doc;
+    if( !doc.setContent( stream.read() ) )
+    {
+        debug() << "Could now read smil playlist" << endl;
+        return false;
+    }   
+	QDomElement root = doc.documentElement();
+	stream.setEncoding ( QTextStream::UnicodeUTF8 );
+
+	if( root.nodeName().lower() != "smil" )
+       return false;
+
+	KURL kurl;
+	QString url;
+	QDomNodeList nodeList;
+	QDomNode node;
+	QDomElement element;
+
+	//audio sources...
+	nodeList = doc.elementsByTagName( "audio" );
+	for( uint i = 0; i < nodeList.count(); i++ )
+	{
+        MetaBundle b;   
+		node = nodeList.item(i);
+		url = QString::null;
+		if( (node.nodeName().lower() == "audio") && (node.isElement()) )
+		{
+			element = node.toElement();
+			if( element.hasAttribute("src") )
+				url = element.attribute("src");
+                
+			else if( element.hasAttribute("Src") )
+				url = element.attribute("Src");
+                
+			else if( element.hasAttribute("SRC") )
+				url = element.attribute("SRC");
+		}
+		if( !url.isNull() )
+		{
+			b.setUrl( url );
+			m_bundles += b;
+		}
+	}
+
+	return true;
+}
+
+
 /// @class RemotePlaylistFetcher
 
 #include <ktempfile.h>
@@ -647,7 +917,7 @@ PlaylistFile::loadRealAudioRam( QTextStream &stream )
 #include <klocale.h>
 
 RemotePlaylistFetcher::RemotePlaylistFetcher( const KURL &source, QListViewItem *after, bool playFirstUrl )
-        : QObject( (QObject*)Playlist::instance() )
+        : QObject( Playlist::instance()->qscrollview() )
         , m_source( source )
         , m_after( after )
         , m_playFirstUrl( playFirstUrl )
@@ -721,28 +991,32 @@ SqlLoader::doJob()
 
     BundleList bundles;
     uint x = 0;
-    for( for_iterators( QStringList, values ); it != end || isAborted(); ++it ) {
-        setProgress( x += 11 );
+    for( for_iterators( QStringList, values ); it != end && !isAborted(); ++it ) {
+        setProgress( x += 13 );
 
         MetaBundle b;
         b.setAlbum     (    *it );
         b.setArtist    (  *++it );
         b.setGenre     (  *++it );
         b.setTitle     (  *++it );
-        b.setYear      (  *++it );
+        b.setYear      ( (*++it).toInt() );
         b.setComment   (  *++it );
-        b.setTrack     (  *++it );
+        b.setTrack     ( (*++it).toInt() );
         b.setBitrate   ( (*++it).toInt() );
         b.setLength    ( (*++it).toInt() );
         b.setSampleRate( (*++it).toInt() );
+        b.setFilesize  ( (*++it).toInt() );
         b.setPath      (  *++it );
+        bool ok;
+        const int val = (*++it).toInt( &ok );
+        b.setCompilation( ok ? val : MetaBundle::CompilationUnknown );
 
         bundles += b;
 
         if( false && b.length() <= 0 ) {
             // we try to read the tags, despite the slow-down
             debug() << "Audioproperties not known for: " << b.url().fileName() << endl;
-            b.readTags( TagLib::AudioProperties::Fast );
+            b.readTags( TagLib::AudioProperties::Fast);
         }
 
         if( bundles.count() == OPTIMUM_BUNDLE_COUNT || it == last ) {
@@ -756,4 +1030,41 @@ SqlLoader::doJob()
     return true;
 }
 
+QTime PlaylistFile::stringToTime(const QString& timeString)
+{
+   int sec = 0;
+   bool ok = false;
+   QStringList tokens = QStringList::split(':',timeString);
+
+   sec += tokens[0].toInt(&ok)*3600; //hours
+   sec += tokens[1].toInt(&ok)*60; //minutes
+   sec += tokens[2].toInt(&ok); //secs
+
+   if (ok)
+      return QTime().addSecs(sec);
+         else
+            return QTime();
+}
+
+bool MyXmlLoader::startElement( const QString &a, const QString &name, const QString &b, const QXmlAttributes &atts )
+{
+    if( name == "playlist" )
+    {
+        QString product, version, dynamic;
+        for( int i = 0, n = atts.count(); i < n; ++i )
+        {
+            if( atts.localName( i ) == "product" )
+                product = atts.value( i );
+            else if( atts.localName( i ) == "version" )
+                version = atts.value( i );
+            else if( atts.localName( i ) == "dynamicMode" )
+                dynamic = atts.value( i );
+        }
+        emit playlistInfo( product, version, dynamic );
+        return !m_aborted;
+    }
+    else
+        return XmlLoader::startElement( a, name, b, atts );
+}
+
 #include "playlistloader.moc"
