@@ -2,7 +2,7 @@
   begin                : Fre Nov 15 2002
   copyright            : (C) Mark Kretschmann <markey@web.de>
                        : (C) Max Howell <max.howell@methylblue.com>
-                       : (C) Gábor Lehel <illissius@gmail.com>
+                       : (C) G??bor Lehel <illissius@gmail.com>
 ***************************************************************************/
 
 /***************************************************************************
@@ -24,20 +24,30 @@
 #include "collectionbrowser.h"
 #include "contextbrowser.h"
 #include "debug.h"
+#include "devicemanager.h"
 #include "enginecontroller.h" //for actions in ctor
 #include "filebrowser.h"
 #include "k3bexporter.h"
 #include "mediabrowser.h"
-#include "party.h"
+#include "dynamicmode.h"
 #include "playlist.h"
 #include "playlistbrowser.h"
 #include "playlistwindow.h"
 #include "scriptmanager.h"
+#include "statistics.h"
 #include "statusbar.h"
+#include "threadweaver.h"
 
 #include <qevent.h>           //eventFilter()
+#include <qfont.h>
+#include <qheader.h>
 #include <qlayout.h>
 #include <qlabel.h>           //search filter label
+
+#include <qpainter.h>         //dynamic title
+#include <qpen.h>
+
+#include <qsizepolicy.h>      //qspaceritem in dynamic bar
 #include <qtimer.h>           //search filter timer
 #include <qtooltip.h>         //QToolTip::add()
 #include <qvbox.h>            //contains the playlist
@@ -52,6 +62,7 @@
 #include <kmenubar.h>
 #include <kmessagebox.h>       //savePlaylist()
 #include <kpopupmenu.h>
+#include <kpushbutton.h>
 #include <kstandarddirs.h>    //Welcome Tab, locate welcome.html
 #include <ktoolbar.h>
 #include <ktoolbarbutton.h>   //createGUI()
@@ -88,10 +99,6 @@ namespace amaroK
     };
 }
 
-//////////////////////////////////////////////////////////////////////////////////////////
-/// CLASS PlaylistWindow
-//////////////////////////////////////////////////////////////////////////////////////////
-
 PlaylistWindow *PlaylistWindow::s_instance = 0;
 
 PlaylistWindow::PlaylistWindow()
@@ -125,22 +132,30 @@ PlaylistWindow::PlaylistWindow()
     new KAction( i18n("Play Audio CD"), "cdaudio_unmount", 0, this, SLOT(playAudioCD()), ac, "play_audiocd" );
     new KAction( i18n("Script Manager"), "pencil", 0, this, SLOT(showScriptSelector()), ac, "script_manager" );
     new KAction( i18n("Queue Manager"), "goto", 0, this, SLOT(showQueueManager()), ac, "queue_manager" );
+    new KAction( i18n("Statistics"), amaroK::icon( "info" ), 0, this, SLOT(showStatistics()), ac, "statistics" );
 
     ac->action( "options_configure_globals" )->setText( i18n( "Configure &Global Shortcuts..." ) );
 
-    new KAction( i18n( "Previous Track" ), "player_start", 0, ec, SLOT( previous() ), ac, "prev" );
-    new KAction( i18n( "Play" ), "player_play", 0, ec, SLOT( play() ), ac, "play" );
-    new KAction( i18n( "Pause" ), "player_pause", 0, ec, SLOT( pause() ), ac, "pause" );
-    new KAction( i18n( "Next Track" ), "player_end", 0, ec, SLOT( next() ), ac, "next" );
+    new KAction( i18n( "Previous Track" ), amaroK::icon( "back" ), 0, ec, SLOT( previous() ), ac, "prev" );
+    new KAction( i18n( "Play" ), amaroK::icon( "play" ), 0, ec, SLOT( play() ), ac, "play" );
+    new KAction( i18n( "Pause" ), amaroK::icon( "pause" ), 0, ec, SLOT( pause() ), ac, "pause" );
+    new KAction( i18n( "Next Track" ), amaroK::icon( "next" ), 0, ec, SLOT( next() ), ac, "next" );
+
+    {
+        KAction *gah = new KAction( i18n( "Toggle Focus" ), "reload", CTRL+Key_Tab,
+            this, SLOT( slotToggleFocus() ), ac, "toggle_focus" );
+        KPopupMenu asdf;
+        gah->plug( &asdf );
+        gah->unplug( &asdf );
+    }
 
     new amaroK::MenuAction( ac );
     new amaroK::StopAction( ac );
     new amaroK::PlayPauseAction( ac );
     new amaroK::AnalyzerAction( ac );
-    new amaroK::RepeatTrackAction( ac );
-    new amaroK::RepeatPlaylistAction( ac );
+    new amaroK::RepeatAction( ac );
     new amaroK::RandomAction( ac );
-    new amaroK::DynamicAction( ac );
+    new amaroK::FavorAction( ac );
     new amaroK::VolumeAction( ac );
 
     if( K3bExporter::isAvailable() )
@@ -165,8 +180,10 @@ PlaylistWindow::~PlaylistWindow()
 
 ///////// public interface
 
-void
-PlaylistWindow::init()
+/**
+ * This function will intialize the playlist window.
+ */
+void PlaylistWindow::init()
 {
     DEBUG_BLOCK
 
@@ -176,6 +193,9 @@ PlaylistWindow::init()
 
     m_browsers = new BrowserBar( this );
 
+    //<Dynamic Mode Status Bar />
+    DynamicBar *dynamicBar = new DynamicBar( m_browsers->container());
+
     { //<Search LineEdit>
         KToolBar *bar = new KToolBar( m_browsers->container(), "NotMainToolBar" );
         bar->setIconSize( 22, false ); //looks more sensible
@@ -184,7 +204,7 @@ PlaylistWindow::init()
 
         QWidget *button = new KToolBarButton( "locationbar_erase", 1, bar );
         QLabel *filter_label = new QLabel( i18n("S&earch:") + " ", bar );
-        m_lineEdit = new ClickLineEdit( i18n( "Filter here..." ), bar );
+        m_lineEdit = new ClickLineEdit( i18n( "Enter search terms here" ), bar );
 
         filter_label->setBuddy( m_lineEdit );
 
@@ -206,12 +226,13 @@ PlaylistWindow::init()
 
 
     QFrame *playlist = new Playlist( m_browsers->container() );
+    dynamicBar->init();
     m_toolbar = new amaroK::ToolBar( m_browsers->container(), "mainToolBar" );
     m_toolbar->setShown( AmarokConfig::showToolbar() );
     QWidget *statusbar = new amaroK::StatusBar( this );
 
-    KAction* repeatTrackAction = amaroK::actionCollection()->action( "repeat_track" );
-    connect( repeatTrackAction, SIGNAL( toggled( bool ) ), playlist, SLOT( slotRepeatTrackToggled( bool ) ) );
+    KAction* repeatAction = amaroK::actionCollection()->action( "repeat" );
+    connect( repeatAction, SIGNAL( activated( int ) ), playlist, SLOT( slotRepeatTrackToggled( int ) ) );
 
     connect( m_lineEdit, SIGNAL(textChanged( const QString& )), playlist, SLOT(setFilterSlot( const QString& )) );
 
@@ -252,6 +273,15 @@ PlaylistWindow::init()
     actionCollection()->action("playlist_select_all")->plug( playlistMenu );
     //END Playlist menu
 
+    //BEGIN Mode menu
+    KPopupMenu *modeMenu = new KPopupMenu( m_menubar );
+    actionCollection()->action("repeat")->plug( modeMenu );
+    KSelectAction *random = static_cast<KSelectAction*>( actionCollection()->action("random_mode") );
+    random->plug( modeMenu );
+    random->popupMenu()->insertSeparator();
+    actionCollection()->action("favor_tracks")->plug( random->popupMenu() );
+    //END Mode menu
+
     //BEGIN Tools menu
     m_toolsMenu = new KPopupMenu( m_menubar );
     m_toolsMenu->insertItem( SmallIconSet( "covermanager" ), i18n("&Cover Manager"), amaroK::Menu::ID_SHOW_COVER_MANAGER );
@@ -259,9 +289,10 @@ PlaylistWindow::init()
     m_toolsMenu->insertItem( SmallIconSet( "visualizations"), i18n("&Visualizations"), amaroK::Menu::ID_SHOW_VIS_SELECTOR );
     m_toolsMenu->insertItem( SmallIconSet( "equalizer"), i18n("&Equalizer"), kapp, SLOT( slotConfigEqualizer() ), 0, amaroK::Menu::ID_CONFIGURE_EQUALIZER );
     actionCollection()->action("script_manager")->plug( m_toolsMenu );
+    actionCollection()->action("statistics")->plug( m_toolsMenu );
     m_toolsMenu->insertSeparator();
     m_toolsMenu->insertItem( SmallIconSet( "wizard"), i18n("&First-Run Wizard"), amaroK::Menu::ID_SHOW_WIZARD );
-    m_toolsMenu->insertItem( SmallIconSet( "reload" ), i18n("&Rescan Collection"), amaroK::Menu::ID_RESCAN_COLLECTION );
+    m_toolsMenu->insertItem( SmallIconSet( amaroK::icon( "rescan" ) ), i18n("&Rescan Collection"), amaroK::Menu::ID_RESCAN_COLLECTION );
 
     #if defined HAVE_XMMS || defined HAVE_LIBVISUAL
     m_toolsMenu->setItemEnabled( amaroK::Menu::ID_SHOW_VIS_SELECTOR, true );
@@ -281,12 +312,6 @@ PlaylistWindow::init()
     m_settingsMenu->insertItem( AmarokConfig::showToolbar() ? i18n( "Hide Toolbar" ) : i18n("Show Toolbar"), ID_SHOW_TOOLBAR );
     m_settingsMenu->insertItem( AmarokConfig::showPlayerWindow() ? i18n("Hide Player &Window") : i18n("Show Player &Window"), ID_SHOW_PLAYERWINDOW );
     m_settingsMenu->insertSeparator();
-    //this should be only a context menu option and use next-queue graphics with an infinity symbol or something
-    actionCollection()->action("repeat_track")->plug( m_settingsMenu );
-    actionCollection()->action("repeat_playlist")->plug( m_settingsMenu );
-    actionCollection()->action("random_mode")->plug( m_settingsMenu );
-    actionCollection()->action("dynamic_mode")->plug( m_settingsMenu );
-    m_settingsMenu->insertSeparator();
     actionCollection()->action("options_configure_globals")->plug( m_settingsMenu );
     actionCollection()->action(KStdAction::name(KStdAction::KeyBindings))->plug( m_settingsMenu );
     actionCollection()->action(KStdAction::name(KStdAction::ConfigureToolbars))->plug( m_settingsMenu );
@@ -297,6 +322,7 @@ PlaylistWindow::init()
 
     m_menubar->insertItem( i18n( "&Actions" ), actionsMenu );
     m_menubar->insertItem( i18n( "&Playlist" ), playlistMenu );
+    m_menubar->insertItem( i18n( "&Mode" ), modeMenu );
     m_menubar->insertItem( i18n( "&Tools" ), m_toolsMenu );
     m_menubar->insertItem( i18n( "&Settings" ), m_settingsMenu );
     m_menubar->insertItem( i18n( "&Help" ), amaroK::Menu::helpMenu() );
@@ -344,20 +370,28 @@ PlaylistWindow::init()
             Debug::Block block( name ); \
             m_browsers->addBrowser( Type::instance(), text, icon ); }
 
-        addBrowserMacro( ContextBrowser, "ContextBrowser", i18n( "Context" ), "info" )
-        addBrowserMacro( CollectionBrowser, "CollectionBrowser", i18n( "Collection" ), "collection" )
-        addInstBrowserMacro( PlaylistBrowser, "PlaylistBrowser", i18n( "Playlists" ), "player_playlist_2" )
+        addBrowserMacro( ContextBrowser, "ContextBrowser", i18n( "Context" ), amaroK::icon( "info" ) )
+        addBrowserMacro( CollectionBrowser, "CollectionBrowser", i18n( "Collection" ), amaroK::icon( "collection" ) )
+        addInstBrowserMacro( PlaylistBrowser, "PlaylistBrowser", i18n( "Playlists" ), amaroK::icon( "playlist" ) )
 
-        if( MediaBrowser::isAvailable() )
-            addBrowserMacro( MediaBrowser, "MediaBrowser", i18n( "Media Device" ), "usbpendrive_unmount" )
+        //DEBUG: Comment out the addBrowserMacro line and uncomment the m_browsers line (passing in a vfat device name) to see the "virtual root" functionality
 
         addBrowserMacro( FileBrowser, "FileBrowser", i18n( "Files" ), "folder" )
+        //m_browsers->addBrowser( new FileBrowser( "FileBrowser", DeviceManager::instance()->getDevice( "hda5" ) ), i18n( "Files" ), "folder" );
 
+        new MediaBrowser( "MediaBrowser" );
+        if( MediaBrowser::isAvailable() )
+            addInstBrowserMacro( MediaBrowser, "MediaBrowser", i18n( "Media Device" ), amaroK::icon( "device" ) )
         #undef addBrowserMacro
         #undef addInstBrowserMacro
     }
     //</Browsers>
 
+    connect( Playlist::instance()->qscrollview(), SIGNAL( dynamicModeChanged( const DynamicMode* ) ),
+             PlaylistBrowser::instance(), SLOT( loadDynamicItems() ) );
+
+
+    qApp->installEventFilter( this ); // keyboards shortcuts for the browsers
 
     connect( playlist, SIGNAL( itemCountChanged(     int, int, int, int, int, int ) ),
              statusbar,  SLOT( slotItemCountChanged( int, int, int, int, int, int ) ) );
@@ -367,16 +401,28 @@ PlaylistWindow::init()
     amaroK::MessageQueue::instance()->sendMessages();
 }
 
+void PlaylistWindow::addBrowser( const QString &name, QWidget *browser, const QString &text, const QString &icon )
+{
+    if( !m_browsers->browser( name ) )
+        m_browsers->addBrowser( browser, text, icon );
+}
 
+
+/**
+ * Reload the amarokui.rc xml file.
+ * mainly just used by amarok::Menu
+ */
 void PlaylistWindow::recreateGUI()
 {
-    //mainly just used by amaroK::Menu
     setXMLFile( amaroK::config()->readEntry( "XMLFile", "amarokui.rc" ) );
     reloadXML();
     createGUI();
 }
 
 
+/**
+ * Create the amarok gui from the xml file.
+ */
 void PlaylistWindow::createGUI()
 {
     setUpdatesEnabled( false );
@@ -418,7 +464,7 @@ void PlaylistWindow::createGUI()
     const QStringList::ConstIterator last = list.fromLast();
     for( QStringList::ConstIterator it = list.constBegin(); it != end; ++it )
     {
-        KToolBarButton* const button = (KToolBarButton*)m_toolbar->child( (*it).latin1() );
+        KToolBarButton* const button = static_cast<KToolBarButton*>( m_toolbar->child( (*it).latin1() ) );
 
         if ( it == last ) {
             //if the user has no PlayerWindow, he MUST have the menu action plugged
@@ -444,6 +490,10 @@ void PlaylistWindow::createGUI()
 }
 
 
+/**
+ * Apply the loaded settings on the playlist window.
+ * this function loads the custom fonts (if chosen) and than calls PlayList::instance()->applySettings();
+ */
 void PlaylistWindow::applySettings()
 {
     switch( AmarokConfig::useCustomFonts() )
@@ -457,15 +507,22 @@ void PlaylistWindow::applySettings()
         ContextBrowser::instance()->unsetFont();
         break;
     }
+    Playlist::instance()->applySettings();
 }
 
 
+/**
+ * @param o The object
+ * @param e The event
+ *
+ * Here we filter some events for the Playlist Search LineEdit and the Playlist. @n
+ * this makes life easier since we have more useful functions available from this class
+ */
 bool PlaylistWindow::eventFilter( QObject *o, QEvent *e )
 {
-    //here we filter some events for the Playlist Search LineEdit and the Playlist
-    //this makes life easier since we have more useful functions available from this class
 
-    Playlist *pl = Playlist::instance();
+
+    Playlist* const pl = Playlist::instance();
     typedef QListViewItemIterator It;
 
     switch( e->type() )
@@ -493,38 +550,40 @@ bool PlaylistWindow::eventFilter( QObject *o, QEvent *e )
             return true;
         }
 
+        if( e->state() & ControlButton )
+        {
+            int n = -1;
+            switch( e->key() )
+            {
+                case Key_0: n = 0; break;
+                case Key_1: n = 1; break;
+                case Key_2: n = 2; break;
+                case Key_3: n = 3; break;
+                case Key_4: n = 4; break;
+                case Key_5: n = 5; break;
+            }
+            if( n == 0 )
+            {
+                m_browsers->closeCurrentBrowser();
+                return true;
+            }
+            else if( n > 0 && n <= m_browsers->visibleCount() )
+            {
+                m_browsers->showHideVisibleBrowser( n - 1 );
+                return true;
+            }
+        }
+
         if( o == m_lineEdit ) //the search lineedit
         {
             QListViewItem *item;
             switch( e->key() )
             {
             case Key_Up:
-                item = *It( pl, It::Visible );
-                if( item )
-                    while( item->itemBelow() )
-                        item = item->itemBelow();
-                if( item )
-                {
-                    pl->setFocus();
-                    pl->setCurrentItem( item );
-                    item->setSelected( true );
-                    pl->ensureItemVisible( item );
-                    return true;
-                }
-                return false;
             case Key_Down:
-                if( ( item = *It( pl, It::Visible ) ) )
-                {
-                    pl->setFocus();
-                    pl->setCurrentItem( item );
-                    item->setSelected( true );
-                    pl->ensureItemVisible( item );
-                    return true;
-                }
-                return false;
-
             case Key_PageDown:
             case Key_PageUp:
+                pl->setFocus();
                 QApplication::sendEvent( pl, e );
                 return true;
 
@@ -533,16 +592,44 @@ bool PlaylistWindow::eventFilter( QObject *o, QEvent *e )
                 item = *It( pl, It::Visible );
                 m_lineEdit->clear();
                 pl->m_filtertimer->stop(); //HACK HACK HACK
-                pl->setFilter( "" );       //SLASH
-                if( ( e->state() & ShiftButton ) && item )
+                if( e->state() & ControlButton )
                 {
-                    pl->queue( item );
-                    pl->ensureItemCentered( item );
+                    PLItemList in, out;
+                    if( e->state() & ShiftButton )
+                        for( It it( pl, It::Visible ); PlaylistItem *x = static_cast<PlaylistItem*>( *it ); ++it )
+                        {
+                            pl->queue( x, true );
+                            ( pl->m_nextTracks.contains( x ) ? in : out ).append( x );
+                        }
+                    else
+                    {
+                        It it( pl, It::Visible );
+                        pl->activate( *it );
+                        ++it;
+                        for( int i = 0; PlaylistItem *x = static_cast<PlaylistItem*>( *it ); ++i, ++it )
+                        {
+                            in.append( x );
+                            pl->m_nextTracks.insert( i, x );
+                        }
+                    }
+                    if( !in.isEmpty() || !out.isEmpty() )
+                        emit pl->queueChanged( in, out );
+                    pl->setFilter( "" );
+                    pl->ensureItemCentered( ( e->state() & ShiftButton ) ? item : pl->currentTrack() );
                 }
                 else
                 {
-                    pl->activate( item );
-                    pl->showCurrentTrack();
+                    pl->setFilter( "" );
+                    if( ( e->state() & ShiftButton ) && item )
+                    {
+                        pl->queue( item );
+                        pl->ensureItemCentered( item );
+                    }
+                    else
+                    {
+                        pl->activate( item );
+                        pl->showCurrentTrack();
+                    }
                 }
                 return true;
 
@@ -558,24 +645,38 @@ bool PlaylistWindow::eventFilter( QObject *o, QEvent *e )
         //following are for Playlist::instance() only
         //we don't handle these in the playlist because often we manipulate the lineEdit too
 
-        if( pl->currentItem() && ( ( e->key() == Key_Up   && pl->currentItem()->itemAbove() == 0 )
-                                || ( e->key() == Key_Down && pl->currentItem()->itemBelow() == 0 ) ) )
+        if( o == pl )
         {
-            pl->currentItem()->setSelected( false );
-            m_lineEdit->setFocus();
-            pl->ensureItemVisible( *It( pl, It::Visible ) );
-            return true;
-        }
-        if( e->key() == Key_Delete )
-        {
-            pl->removeSelectedItems();
-            return true;
-        }
-        if( ( e->key() >= Key_0 && e->key() <= Key_Z ) || e->key() == Key_Backspace )
-        {
-            m_lineEdit->setFocus();
-            QApplication::sendEvent( m_lineEdit, e );
-            return true;
+            if( pl->currentItem() && ( e->key() == Key_Up && pl->currentItem()->itemAbove() == 0 ) )
+            {
+                QListViewItem *lastitem = *It( pl, It::Visible );
+                while( lastitem && lastitem->itemBelow() )
+                    lastitem = lastitem->itemBelow();
+                pl->currentItem()->setSelected( false );
+                pl->setCurrentItem( lastitem );
+                lastitem->setSelected( true );
+                pl->ensureItemVisible( lastitem );
+                return true;
+            }
+            if( pl->currentItem() && ( e->key() == Key_Down && pl->currentItem()->itemBelow() == 0 ) )
+            {
+                pl->currentItem()->setSelected( false );
+                pl->setCurrentItem( *It( pl, It::Visible ) );
+                (*It( pl, It::Visible ))->setSelected( true );
+                pl->ensureItemVisible( *It( pl, It::Visible ) );
+                return true;
+            }
+            if( e->key() == Key_Delete )
+            {
+                pl->removeSelectedItems();
+                return true;
+            }
+            if( ( e->key() >= Key_0 && e->key() <= Key_Z ) || e->key() == Key_Backspace || e->key() == Key_Escape )
+            {
+                m_lineEdit->setFocus();
+                QApplication::sendEvent( m_lineEdit, e );
+                return true;
+            }
         }
         #undef e
         break;
@@ -594,6 +695,14 @@ void PlaylistWindow::closeEvent( QCloseEvent *e )
 }
 
 
+void PlaylistWindow::showEvent( QShowEvent* )
+{
+    static bool firstTime = true;
+    if( firstTime )
+        Playlist::instance()->setFocus();
+    firstTime = false;
+}
+
 #include <qdesktopwidget.h>
 QSize PlaylistWindow::sizeHint() const
 {
@@ -632,7 +741,7 @@ void PlaylistWindow::savePlaylist() const //SLOT
         }
 
         if( useArtist && useAlbum )
-            title = artist + " - " + album;
+            title = i18n("%1 - %2").arg( artist, album );
         else if( useArtist )
             title = artist;
         else if( useAlbum )
@@ -659,7 +768,7 @@ void PlaylistWindow::slotAddLocation( bool directPlay ) //SLOT
     KURL::List files;
     //files = KFileDialog::getOpenURLs( QString::null, "*.*|" + i18n("All Files"), this, i18n("Add Media") );
     KFileDialog dlg(QString::null, "*.*|", this, "openMediaDialog", true);
-    dlg.setCaption(directPlay ? i18n("Play Media") : i18n("Add Media"));
+    dlg.setCaption(directPlay ? i18n("Play Media (Files or URLs)") : i18n("Add Media (Files or URLs)"));
     dlg.setMode( KFile::Files | KFile::Directory );
     dlg.exec();
     files = dlg.selectedURLs();
@@ -678,9 +787,16 @@ void PlaylistWindow::slotAddLocation( bool directPlay ) //SLOT
 
 void PlaylistWindow::playAudioCD() //SLOT
 {
-    m_browsers->showBrowser( "FileBrowser" );
-    FileBrowser *fb = static_cast<FileBrowser *>( m_browsers->browser("FileBrowser") );
-    fb->setUrl( KURL("audiocd:/Wav/") );
+    KURL::List urls;
+    if (EngineController::engine()->getAudioCDContents(QString::null, urls)) {
+        if (!urls.isEmpty()) {
+            Playlist::instance()->insertMedia(urls, Playlist::Replace);
+        }
+    } else { // Default behaviour
+        m_browsers->showBrowser( "FileBrowser" );
+        FileBrowser *fb = static_cast<FileBrowser *>( m_browsers->browser("FileBrowser") );
+        fb->setUrl( KURL("audiocd:/Wav/") );
+    }
 }
 
 void PlaylistWindow::showScriptSelector() //SLOT
@@ -694,6 +810,24 @@ void PlaylistWindow::showQueueManager() //SLOT
     Playlist::instance()->showQueueManager();
 }
 
+void PlaylistWindow::showStatistics() //SLOT
+{
+    if( Statistics::instance() ) {
+        Statistics::instance()->raise();
+        return;
+    }
+    Statistics dialog;
+    dialog.exec();
+}
+
+void PlaylistWindow::slotToggleFocus() //SLOT
+{
+    if( m_browsers->currentBrowser() && ( Playlist::instance()->hasFocus() || m_lineEdit->hasFocus() ) )
+        m_browsers->currentBrowser()->setFocus();
+    else
+        Playlist::instance()->setFocus();
+}
+
 void PlaylistWindow::slotToggleMenu() //SLOT
 {
     if( static_cast<KToggleAction *>(actionCollection()->action(KStdAction::name(KStdAction::ShowMenubar)))->isChecked() ) {
@@ -738,30 +872,31 @@ void PlaylistWindow::slotMenuActivated( int index ) //SLOT
 
 void PlaylistWindow::actionsMenuAboutToShow() //SLOT
 {
-    actionCollection()->action("play_audiocd")->setEnabled( EngineController::hasEngineProperty( "HasKIO" ) );
 }
 
 void PlaylistWindow::toolsMenuAboutToShow() //SLOT
 {
     m_toolsMenu->setItemEnabled( amaroK::Menu::ID_CONFIGURE_EQUALIZER, EngineController::hasEngineProperty( "HasEqualizer" ) );
+    m_toolsMenu->setItemEnabled( amaroK::Menu::ID_RESCAN_COLLECTION, !ThreadWeaver::instance()->isJobPending( "CollectionScanner" ) );
 }
 
 
 #include <kwin.h>
+/**
+ * Show/hide playlist global shortcut and PlayerWindow PlaylistButton connect to this slot
+ * RULES:
+ * 1. hidden & iconified -> deiconify & show @n
+ * 2. hidden & deiconified -> show @n
+ * 3. shown & iconified -> deiconify @n
+ * 4. shown & deiconified -> hide @n
+ * 5. don't hide if there is no tray icon or playerWindow. todo (I can't be arsed) @n
+ *
+ * @note isMinimized() can only be true if the window isShown()
+ * this has taken me hours to get right, change at your peril!
+ * there are more contingencies than you can believe
+ */
 void PlaylistWindow::showHide() //SLOT
 {
-    //show/hide playlist global shortcut and PlayerWindow PlaylistButton connect to this slot
-
-    //RULES:
-    //1. hidden & iconified -> deiconify & show
-    //2. hidden & deiconified -> show
-    //3. shown & iconified -> deiconify
-    //4. shown & deiconified -> hide
-    //5. don't hide if there is no tray icon or playerWindow //TODO (I can't be arsed)
-
-    //NOTE isMinimized() can only be true if the window isShown()
-    //NOTE this has taken me hours to get right, change at your peril!
-    //     there are more contingencies than you can believe
 
     const KWin::WindowInfo info = KWin::windowInfo( winId() );
     const uint desktop = KWin::currentDesktop();
@@ -784,4 +919,98 @@ void PlaylistWindow::showHide() //SLOT
     if( isShown() ) KWin::deIconifyWindow( winId() );
 }
 
+bool PlaylistWindow::isReallyShown() const
+{
+    const KWin::WindowInfo info = KWin::windowInfo( winId() );
+    return isShown() && !info.isMinimized() && info.isOnDesktop( KWin::currentDesktop() );
+}
+
+//////////////////////////////////////////////////////////////////////////////////////////
+/// DynamicBar
+//////////////////////////////////////////////////////////////////////////////////////////
+DynamicBar::DynamicBar(QWidget* parent)
+    : QHBox( parent, "DynamicModeStatusBar" )
+{
+    m_titleWidget = new DynamicTitle(this);
+
+    setSpacing( KDialog::spacingHint() );
+    QWidget *spacer = new QWidget( this );
+    setStretchFactor( spacer, 10 );
+}
+
+// necessary because it has to be constructed before Playlist::instance(), but also connect to it
+void DynamicBar::init()
+{
+    connect(Playlist::instance()->qscrollview(), SIGNAL(dynamicModeChanged(const DynamicMode*)),
+                                                   SLOT(slotNewDynamicMode(const DynamicMode*)));
+
+    KPushButton* editDynamicButton = new KPushButton( i18n("Edit"), this, "DynamicModeEdit" );
+    connect( editDynamicButton, SIGNAL(clicked()), Playlist::instance()->qscrollview(), SLOT(editActiveDynamicMode()) );
+
+    KPushButton* repopButton = new KPushButton( i18n("Repopulate"), this, "DynamicModeRepopulate" );
+    connect( repopButton, SIGNAL(clicked()), Playlist::instance()->qscrollview(), SLOT(repopulate()) );
+
+    KPushButton* disableButton = new KPushButton( i18n("Turn Off"), this, "DynamicModeDisable" );
+    connect( disableButton, SIGNAL(clicked()), Playlist::instance()->qscrollview(), SLOT(disableDynamicMode()) );
+
+    slotNewDynamicMode( Playlist::instance()->dynamicMode() );
+}
+
+void DynamicBar::slotNewDynamicMode(const DynamicMode* mode)
+{
+    setShown(mode);
+    if (mode)
+        changeTitle(mode->title());
+}
+
+void DynamicBar::changeTitle(const QString& title)
+{
+   m_titleWidget->setTitle(title);
+}
+
+//////////////////////////////////////////////////////////////////////////////////////////
+/// DynamicTitle
+//////////////////////////////////////////////////////////////////////////////////////////
+DynamicTitle::DynamicTitle(QWidget* w)
+    : QWidget(w, "dynamic title")
+{
+    m_font.setBold( true );
+    setTitle("");
+}
+
+void DynamicTitle::setTitle(const QString& newTitle)
+{
+    m_title = newTitle;
+    QFontMetrics fm(m_font);
+    setMinimumWidth( s_curveWidth*3 + fm.width(m_title) + s_imageSize );
+    setMinimumHeight( fm.height() );
+}
+
+void DynamicTitle::paintEvent(QPaintEvent* /*e*/)
+{
+    QPainter p;
+    p.begin( this, false );
+    QPen pen( colorGroup().highlightedText(), 0, Qt::NoPen );
+    p.setPen( pen );
+    p.setBrush( colorGroup().highlight() );
+    p.setFont(m_font);
+
+    QFontMetrics fm(m_font);
+    int textHeight = fm.height();
+    if (textHeight < s_imageSize)
+        textHeight = s_imageSize;
+    int textWidth = fm.width(m_title);
+    int yStart = (height() - textHeight) / 2;
+    if(yStart < 0)
+        yStart = 0;
+
+    p.drawEllipse( 0, yStart, s_curveWidth * 2, textHeight);
+    p.drawEllipse( s_curveWidth + textWidth + s_imageSize, yStart, s_curveWidth*2, textHeight);
+    p.fillRect( s_curveWidth, yStart, textWidth + s_imageSize + s_curveWidth, textHeight
+                    , QBrush( colorGroup().highlight()) );
+    p.drawPixmap( s_curveWidth, yStart + ((textHeight - s_imageSize) /2), SmallIcon("dynamic") );
+    //not sure why first arg of Rect shouldn't add @curveWidth
+    p.drawText( QRect(s_imageSize, yStart, s_curveWidth + textWidth +s_imageSize, textHeight), Qt::AlignCenter, m_title);
+}
+
 #include "playlistwindow.moc"
