@@ -4,8 +4,8 @@
 #define DEBUG_PREFIX "RefreshImages"
 
 #include "amarok.h"
-#include "debug.h"
 #include "collectiondb.h"
+#include "debug.h"
 #include "refreshimages.h"
 #include "statusbar.h"
 
@@ -13,6 +13,7 @@
 #include <qimage.h>
 #include <qmap.h>
 #include <qobject.h>
+#include <qstringlist.h>
 #include <qvaluelist.h> 
 #include <qvariant.h>
 
@@ -25,27 +26,36 @@
 
 RefreshImages::RefreshImages()
 {
-    static const QString LICENSE = "0RQSQ0B8CRY7VX2VF3G2"; //Ian Monroe
     const QStringList staleImages = CollectionDB::instance()->staleImages();
     //"SELECT asin, locale, filename FROM amazon WHERE refetchdate > %1 ;"
     QStringList::ConstIterator it = staleImages.begin();
     QStringList::ConstIterator end = staleImages.end();
-    while(it != end)
+    while( it != end )
     {
         QString asin=*it;
         it++;
         QString locale = *it;
         it++;
         QString md5sum = *it;
+        if( asin.isEmpty() || locale.isEmpty() || md5sum.isEmpty())
+        {   
+            //somehow we have entries without ASIN    
+            if ( !md5sum.isEmpty() ) //I've never seen this, just to be sure
+                CollectionDB::instance()->removeInvalidAmazonInfo(md5sum); 
+            it++;
+            if(it==end)
+                deleteLater();
+            continue; 
+        }
         QString url =
-            QString("http://webservices.amazon.%1/onca/xml?Service=AWSECommerceService&SubscriptionId=%2&Operation=ItemLookup&ItemId=%3&ResponseGroup=Images")
+            QString("http://webservices.amazon.%1/onca/xml?Service=AWSECommerceService&SubscriptionId=%2&Operation=ItemLookup&ItemId=%3&ResponseGroup=Small,Images")
              .arg(localeToTLD(locale))
-             .arg(LICENSE)
+             .arg("0RQSQ0B8CRY7VX2VF3G2") //Ian Monroe
              .arg(asin);
-        debug() << "going to get " << url << endl;  
+        debug() << url << endl;
         KIO::TransferJob* job = KIO::storedGet( url, false, false );
         KIO::Scheduler::scheduleJob(job);
-        amaroK::StatusBar::instance()->newProgressOperation( job );
+        //amaroK::StatusBar::instance()->newProgressOperation( job );
         job->setName(md5sum.ascii());
         it++; //iterate to the next set
         m_jobInfo[md5sum] = JobInfo(asin, locale, it==end); 
@@ -58,6 +68,8 @@ void RefreshImages::finishedXmlFetch( KIO::Job* xmlJob ) //SLOT
 
     if( xmlJob->error() ) {
         amaroK::StatusBar::instance()->shortMessage(i18n("There was an error communicating with Amazon."));
+        if(m_jobInfo[xmlJob->name()].m_last)
+            deleteLater();
         return;
     }
     KIO::StoredTransferJob* const storedJob = static_cast<KIO::StoredTransferJob*>( xmlJob );
@@ -65,44 +77,64 @@ void RefreshImages::finishedXmlFetch( KIO::Job* xmlJob ) //SLOT
 
     QDomDocument doc;
     if( !doc.setContent( xml ) )
-      { debug() << "Amazon returned invalid XML" << endl;  return; }
+      return;
 
-    QString imageUrl = doc.documentElement()
-       .namedItem( "Items" )
-       .namedItem("Item")
-       .namedItem("LargeImage")
-       .namedItem("URL").firstChild().toText().data();
-    debug() << "setting up " << imageUrl << endl;
-    KURL testUrl(imageUrl);
-    if(!testUrl.isValid()) //KIO crashs on empty strings!!!
+    QStringList imageSizes;
+    imageSizes << "LargeImage" << "MediumImage" << "SmallImage";
+    QString imageUrl;
+    foreach( imageSizes )
     {
-        debug() << "Image url not returned, cover not being refreshed" << endl;
+        QDomNode imageNode = doc.documentElement()
+            .namedItem( "Items" )
+            .namedItem( "Item" )
+            .namedItem( *it );
+        if( !imageNode.isNull() )
+        {
+            imageUrl = imageNode.namedItem( "URL" ).firstChild().toText().data();
+            if( !imageUrl.isEmpty() ) 
+                break;
+        } 
+    }
+    debug() << imageUrl << endl;
+    KURL testUrl( imageUrl );
+    if( !testUrl.isValid() ) //KIO crashs on empty strings!!!
+    {    
+        //Amazon sometimes takes down covers
+        CollectionDB::instance()->removeInvalidAmazonInfo(xmlJob->name());
         return;
     }
+        
     KIO::TransferJob* imageJob = KIO::storedGet( imageUrl, false, false );
     KIO::Scheduler::scheduleJob(imageJob);
-    amaroK::StatusBar::instance()->newProgressOperation( imageJob );
+    //amaroK::StatusBar::instance()->newProgressOperation( imageJob );
     imageJob->setName(xmlJob->name());
-
-    connect( imageJob, SIGNAL(result( KIO::Job* )), SLOT(finishedImageFetch( KIO::Job* )) );
+    //get the URL of the detail page
+    m_jobInfo[xmlJob->name()].m_detailUrl = doc.documentElement()
+       .namedItem( "Items" )
+       .namedItem( "Item" )
+       .namedItem( "DetailPageURL" ).firstChild().toText().data();
+    connect( imageJob, SIGNAL( result(KIO::Job*) ), SLOT( finishedImageFetch(KIO::Job*) ) );
 }
 
 void RefreshImages::finishedImageFetch(KIO::Job* imageJob)
 {
+   if( imageJob->error() ) {
+        amaroK::StatusBar::instance()->shortMessage(i18n("There was an error communicating with Amazon."));
+        if(m_jobInfo[imageJob->name()].m_last)
+            deleteLater();
+        return;
+    }
     QImage img;
     img.loadFromData(static_cast<KIO::StoredTransferJob*>(imageJob)->data());
-    img.setText( "amazon-url", 0, m_jobInfo[imageJob->name()].m_asin);
+    img.setText( "amazon-url", 0, m_jobInfo[imageJob->name()].m_detailUrl);
     img.save( amaroK::saveLocation("albumcovers/large/") + imageJob->name(), "PNG");
 
     CollectionDB::instance()->newAmazonReloadDate( m_jobInfo[imageJob->name()].m_asin
         , m_jobInfo[imageJob->name()].m_locale
         , imageJob->name());
 
-    debug() << m_jobInfo[imageJob->name()].m_asin << " copied " << imageJob->name() << endl;
     if(m_jobInfo[imageJob->name()].m_last)
-    {
-      deleteLater();
-    }
+        deleteLater();
 }
 
 QString RefreshImages::localeToTLD(const QString& locale)
@@ -111,6 +143,8 @@ QString RefreshImages::localeToTLD(const QString& locale)
         return "com";
     else if(locale=="jp")
         return "co.jp";
+    else if(locale=="uk")
+        return "co.uk";
     else
         return locale;
 }
