@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2005 by Mark Kretschmann <markey@web.de>           *
+ *   Copyright (C) 2004-2006 by Mark Kretschmann <markey@web.de>           *
  *                      2005 by Seb Ruiz <me@sebruiz.net>                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -15,12 +15,13 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
  ***************************************************************************/
 
 #define DEBUG_PREFIX "ScriptManager"
 
 #include "amarok.h"
+#include "contextbrowser.h"
 #include "debug.h"
 #include "enginecontroller.h"
 #include "metabundle.h"
@@ -34,8 +35,10 @@
 #include <qdir.h>
 #include <qfileinfo.h>
 #include <qfont.h>
+#include <qlabel.h>
 #include <qtimer.h>
 
+#include <kaboutdialog.h>
 #include <kapplication.h>
 #include <kfiledialog.h>
 #include <kiconloader.h>
@@ -58,6 +61,32 @@
 #include <knewstuff/provider.h>       // "
 
 
+namespace amaroK {
+    void closeOpenFiles(int out, int in, int err) {
+        for(int i = sysconf(_SC_OPEN_MAX) - 1; i > 2; i--)
+            if(i!=out && i!=in && i!=err)
+                close(i);
+    }
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// class AmaroKProcIO
+////////////////////////////////////////////////////////////////////////////////
+/** Due to xine-lib, we have to make KProcess close all fds, otherwise we get "device is busy" messages
+  * Used by AmaroKProcIO and AmaroKProcess, exploiting commSetupDoneC(), a virtual method that
+  * happens to be called in the forked process
+  * See bug #103750 for more information.
+  */
+class AmaroKProcIO : public KProcIO {
+    public:
+    virtual int commSetupDoneC() {
+        const int i = KProcIO::commSetupDoneC();
+        amaroK::closeOpenFiles(KProcIO::out[0],KProcIO::in[0],KProcIO::err[0]);
+        return i;
+    };
+};
+
+
 ////////////////////////////////////////////////////////////////////////////////
 // class AmarokScriptNewStuff
 ////////////////////////////////////////////////////////////////////////////////
@@ -123,8 +152,8 @@ ScriptManager::ScriptManager( QWidget *parent, const char *name )
     m_gui->installButton  ->setIconSet( SmallIconSet( "fileopen" ) );
     m_gui->retrieveButton ->setIconSet( SmallIconSet( "khtml_kget" ) );
     m_gui->uninstallButton->setIconSet( SmallIconSet( "remove" ) );
-    m_gui->runButton      ->setIconSet( SmallIconSet( "player_play" ) );
-    m_gui->stopButton     ->setIconSet( SmallIconSet( "player_stop" ) );
+    m_gui->runButton      ->setIconSet( SmallIconSet( amaroK::icon( "play" ) ) );
+    m_gui->stopButton     ->setIconSet( SmallIconSet( amaroK::icon( "stop" ) ) );
     m_gui->configureButton->setIconSet( SmallIconSet( "configure" ) );
     m_gui->aboutButton    ->setIconSet( SmallIconSet( "help" ) );
 
@@ -132,6 +161,8 @@ ScriptManager::ScriptManager( QWidget *parent, const char *name )
     setMinimumSize( kMax( 350, sz.width() ), kMax( 250, sz.height() ) );
     resize( sizeHint() );
 
+    connect( this, SIGNAL(lyricsScriptChanged()), ContextBrowser::instance(), SLOT( lyricsScriptChanged() ) );
+
     // Delay this call via eventloop, because it's a bit slow and would block
     QTimer::singleShot( 0, this, SLOT( findScripts() ) );
 }
@@ -141,14 +172,12 @@ ScriptManager::~ScriptManager()
 {
     DEBUG_BLOCK
 
-    debug() << "Killing running scripts.\n";
-
     QStringList runningScripts;
     ScriptMap::Iterator it;
     ScriptMap::Iterator end( m_scripts.end() );
-    for ( it = m_scripts.begin(); it != end; ++it ) {
-        if ( it.data().process ) {
-            delete it.data().process;
+    for( it = m_scripts.begin(); it != end; ++it ) {
+        if( it.data().process ) {
+            terminateProcess( &it.data().process );
             runningScripts << it.key();
         }
     }
@@ -156,7 +185,6 @@ ScriptManager::~ScriptManager()
     // Save config
     KConfig* const config = amaroK::config( "ScriptManager" );
     config->writeEntry( "Running Scripts", runningScripts );
-    config->writeEntry( "Auto Run", m_gui->checkBox_autoRun->isChecked() );
 
     s_instance = 0;
 }
@@ -169,7 +197,7 @@ ScriptManager::~ScriptManager()
 bool
 ScriptManager::runScript( const QString& name )
 {
-    if ( !m_scripts.contains( name ) )
+    if( !m_scripts.contains( name ) )
         return false;
 
     m_gui->listView->setCurrentItem( m_scripts[name].li );
@@ -180,7 +208,7 @@ ScriptManager::runScript( const QString& name )
 bool
 ScriptManager::stopScript( const QString& name )
 {
-    if ( !m_scripts.contains( name ) )
+    if( !m_scripts.contains( name ) )
         return false;
 
     m_gui->listView->setCurrentItem( m_scripts[name].li );
@@ -194,10 +222,8 @@ QStringList
 ScriptManager::listRunningScripts()
 {
     QStringList runningScripts;
-    ScriptMap::ConstIterator it;
-    ScriptMap::ConstIterator end(m_scripts.end() );
-    for ( it = m_scripts.begin(); it != end; ++it )
-        if ( it.data().process )
+    foreachType( ScriptMap, m_scripts )
+        if( it.data().process )
             runningScripts << it.key();
 
     return runningScripts;
@@ -211,6 +237,75 @@ ScriptManager::customMenuClicked( const QString& message )
 }
 
 
+QString
+ScriptManager::specForScript( const QString& name )
+{
+    if( !m_scripts.contains( name ) )
+        return QString::null;
+    QFileInfo info( m_scripts[name].url.path() );
+    const QString specPath = info.dirPath() + "/" + info.baseName() + ".spec";
+
+    return specPath;
+}
+
+
+void
+ScriptManager::notifyFetchLyrics( const QString& artist, const QString& title )
+{
+    const QString args = KURL::encode_string( artist ) + " " + KURL::encode_string( title );
+    notifyScripts( "fetchLyrics " + args );
+}
+
+
+void
+ScriptManager::notifyFetchLyricsByUrl( const QString& url )
+{
+    notifyScripts( "fetchLyricsByUrl " + url );
+}
+
+
+QStringList
+ScriptManager::lyricsScripts() const
+{
+    QStringList scripts;
+    foreachType( ScriptMap, m_scripts )
+        if( it.data().type == "lyrics" )
+            scripts += it.key();
+
+    return scripts;
+}
+
+
+QString
+ScriptManager::lyricsScriptRunning() const
+{
+    foreachType( ScriptMap, m_scripts )
+        if( it.data().process )
+            if( it.data().type == "lyrics" )
+                return it.key();
+
+    return QString::null;
+}
+
+
+QString
+ScriptManager::transcodeScriptRunning() const
+{
+    foreachType( ScriptMap, m_scripts )
+        if( it.data().process )
+            if( it.data().type == "transcode" )
+                return it.key();
+
+    return QString::null;
+}
+
+
+void ScriptManager::notifyTranscode( const QString& srcUrl, const QString& filetype )
+{
+    notifyScripts( "transcode " + srcUrl + " " + filetype );
+}
+
+
 ////////////////////////////////////////////////////////////////////////////////
 // private slots
 ////////////////////////////////////////////////////////////////////////////////
@@ -218,36 +313,27 @@ ScriptManager::customMenuClicked( const QString& message )
 void
 ScriptManager::findScripts() //SLOT
 {
-    DEBUG_BLOCK
-
     const QStringList allFiles = kapp->dirs()->findAllResources( "data", "amarok/scripts/*", true );
 
     // Add found scripts to listview:
-
-    QStringList::ConstIterator it;
-    QStringList::ConstIterator end( allFiles.end() );
-    for ( it = allFiles.begin(); it != end; ++it )
-        if ( QFileInfo( *it ).isExecutable() )
-            loadScript( *it );
+    {
+        foreach( allFiles )
+            if( QFileInfo( *it ).isExecutable() )
+                loadScript( *it );
+    }
 
     // Handle auto-run:
 
     KConfig* const config = amaroK::config( "ScriptManager" );
-    const bool autoRun = config->readBoolEntry( "Auto Run", false );
-    m_gui->checkBox_autoRun->setChecked( autoRun );
+    const QStringList runningScripts = config->readListEntry( "Running Scripts" );
 
-    if ( autoRun ) {
-        const QStringList runningScripts = config->readListEntry( "Running Scripts" );
-
-        QStringList::ConstIterator it;
-        QStringList::ConstIterator end( runningScripts.end() );
-        for ( it = runningScripts.begin(); it != end; ++it ) {
-            if ( m_scripts.contains( *it ) ) {
+    {
+        foreach( runningScripts )
+            if( m_scripts.contains( *it ) ) {
                 debug() << "Auto-running script: " << *it << endl;
                 m_gui->listView->setCurrentItem( m_scripts[*it].li );
                 slotRunScript();
             }
-        }
     }
 
     m_gui->listView->setCurrentItem( m_gui->listView->firstChild() );
@@ -258,7 +344,7 @@ ScriptManager::findScripts() //SLOT
 void
 ScriptManager::slotCurrentChanged( QListViewItem* item )
 {
-    if ( item ) {
+    if( item ) {
         const QString name = item->text( 0 );
         m_gui->uninstallButton->setEnabled( true );
         m_gui->runButton->setEnabled( !m_scripts[name].process );
@@ -279,21 +365,19 @@ ScriptManager::slotCurrentChanged( QListViewItem* item )
 bool
 ScriptManager::slotInstallScript( const QString& path )
 {
-    DEBUG_BLOCK
-
     QString _path = path;
 
-    if ( path.isNull() ) {
-        KFileDialog dia( QString::null, "*.amarokscript.tar *.amarokscript.tar.bz2 *.amarokscript.tar.gz|" + i18n( "Script Packages (*.amarokscript.tar, *.amarokscript.tar.bz2, *.amarokscript.tar.gz)" ), 0, 0, true );
-        kapp->setTopWidget( &dia );
-        dia.setCaption( kapp->makeStdCaption( i18n( "Select Script Package" ) ) );
-        dia.setMode( KFile::File | KFile::ExistingOnly );
-        if ( !dia.exec() ) return false;
-        _path = dia.selectedURL().path();
+    if( path.isNull() ) {
+        _path = KFileDialog::getOpenFileName( QString::null,
+            "*.amarokscript.tar *.amarokscript.tar.bz2 *.amarokscript.tar.gz|"
+            + i18n( "Script Packages (*.amarokscript.tar, *.amarokscript.tar.bz2, *.amarokscript.tar.gz)" )
+            , this
+            , i18n( "Select Script Package" ) );
+        if( _path == QString::null ) return false;
     }
 
     KTar archive( _path );
-    if ( !archive.open( IO_ReadOnly ) ) {
+    if( !archive.open( IO_ReadOnly ) ) {
         KMessageBox::sorry( 0, i18n( "Could not read this package." ) );
         return false;
     }
@@ -303,7 +387,7 @@ ScriptManager::slotInstallScript( const QString& path )
 
     // Prevent installing a script that's already installed
     const QString scriptFolder = destination + archiveDir->entries().first();
-    if ( QFile::exists( scriptFolder ) ) {
+    if( QFile::exists( scriptFolder ) ) {
         KMessageBox::error( 0, i18n( "A script with the name '%1' is already installed. "
                                      "Please uninstall it first." ).arg( archiveDir->entries().first() ) );
         return false;
@@ -313,7 +397,7 @@ ScriptManager::slotInstallScript( const QString& path )
     m_installSuccess = false;
     recurseInstall( archiveDir, destination );
 
-    if ( m_installSuccess ) {
+    if( m_installSuccess ) {
         KMessageBox::information( 0, i18n( "Script successfully installed." ) );
         return true;
     }
@@ -335,20 +419,18 @@ ScriptManager::recurseInstall( const KArchiveDirectory* archiveDir, const QStrin
 {
     const QStringList entries = archiveDir->entries();
 
-    QStringList::ConstIterator it;
-    QStringList::ConstIterator end( entries.end() );
-    for ( it = entries.begin(); it != end; ++it ) {
+    foreach( entries ) {
         const QString entry = *it;
         const KArchiveEntry* const archEntry = archiveDir->entry( entry );
 
-        if ( archEntry->isDirectory() ) {
-            KArchiveDirectory* const dir = (KArchiveDirectory*) archEntry;
+        if( archEntry->isDirectory() ) {
+            const KArchiveDirectory* const dir = static_cast<const KArchiveDirectory*>( archEntry );
             recurseInstall( dir, destination + entry + "/" );
         }
         else {
             ::chmod( QFile::encodeName( destination + entry ), archEntry->permissions() );
 
-            if ( QFileInfo( destination + entry ).isExecutable() ) {
+            if( QFileInfo( destination + entry ).isExecutable() ) {
                 loadScript( destination + entry );
                 m_installSuccess = true;
             }
@@ -382,44 +464,39 @@ ScriptManager::slotRetrieveScript()
 void
 ScriptManager::slotUninstallScript()
 {
-    DEBUG_BLOCK
-
     const QString name = m_gui->listView->currentItem()->text( 0 );
 
-    if ( KMessageBox::warningContinueCancel( 0, i18n( "Are you sure you want to uninstall the script '%1'?" ).arg( name ), i18n("Uninstall Script"), i18n("Uninstall") ) == KMessageBox::Cancel )
+    if( KMessageBox::warningContinueCancel( 0, i18n( "Are you sure you want to uninstall the script '%1'?" ).arg( name ), i18n("Uninstall Script"), i18n("Uninstall") ) == KMessageBox::Cancel )
         return;
 
-    if ( m_scripts.find( name ) == m_scripts.end() )
+    if( m_scripts.find( name ) == m_scripts.end() )
         return;
 
     const QString directory = m_scripts[name].url.directory();
 
     // Delete directory recursively
     const KURL url = KURL::fromPathOrURL( directory );
-    if ( !KIO::NetAccess::del( url, 0 ) ) {
+    if( !KIO::NetAccess::del( url, 0 ) ) {
         KMessageBox::sorry( 0, i18n( "<p>Could not uninstall this script.</p><p>The ScriptManager can only uninstall scripts which have been installed as packages.</p>" ) );
         return;
     }
 
-    // Find all scripts that were in the uninstalled folder
     QStringList keys;
-    ScriptMap::Iterator it;
-    ScriptMap::Iterator end( m_scripts.end() );
-    for ( it = m_scripts.begin(); it != end; ++it )
-        if ( it.data().url.directory() == directory )
-            keys << it.key();
 
-    // Kill script processes, remove entries from script list
-    QStringList::Iterator itKeys;
-    for ( itKeys = keys.begin(); itKeys != keys.end(); ++itKeys ) {
-        delete m_scripts[*itKeys].li;
-        if ( m_scripts[*itKeys].process ) {
-            // Kill script process (with SIGTERM)
-            m_scripts[*itKeys].process->kill();
-            m_scripts[*itKeys].process->detach();
-            delete m_scripts[*itKeys].process;
+    // Find all scripts that were in the uninstalled folder
+    {
+        foreachType( ScriptMap, m_scripts )
+            if( it.data().url.directory() == directory )
+                keys << it.key();
+    }
+
+    // Terminate script processes, remove entries from script list
+    {
+        foreach( keys ) {
+            delete m_scripts[*it].li;
+            terminateProcess( &m_scripts[*it].process );
+            m_scripts.erase( *it );
         }
-        m_scripts.erase( *itKeys );
     }
 }
 
@@ -427,35 +504,49 @@ ScriptManager::slotUninstallScript()
 bool
 ScriptManager::slotRunScript()
 {
-    DEBUG_BLOCK
-
     QListViewItem* const li = m_gui->listView->currentItem();
     const QString name = li->text( 0 );
 
+    if( m_scripts[name].type == "lyrics" && lyricsScriptRunning() != QString::null ) {
+        KMessageBox::sorry( 0, i18n( "Another lyrics script is already running. "
+                                     "You may only run one lyrics script at a time." ) );
+        return false;
+    }
+
+    if( m_scripts[name].type == "transcode" && transcodeScriptRunning() != QString::null ) {
+        KMessageBox::sorry( 0, i18n( "Another transcode script is already running. "
+                                     "You may only run one transcode script at a time." ) );
+        return false;
+    }
+
     // Don't start a script twice
-    if ( m_scripts[name].process ) return false;
+    if( m_scripts[name].process ) return false;
 
+    AmaroKProcIO* script = new AmaroKProcIO();
+    script->setComm( static_cast<KProcess::Communication>( KProcess::All ) );
     const KURL url = m_scripts[name].url;
-    KProcIO* script = new KProcIO();
-//     script->setComm( (KProcess::Communication) ( KProcess::Stdin | KProcess::Stdout | KProcess::Stderr ) );
-
     *script << url.path();
     script->setWorkingDirectory( amaroK::saveLocation( "scripts-data/" ) );
 
-    if ( !script->start( KProcess::NotifyOnExit, true ) ) {
+    connect( script, SIGNAL( receivedStderr( KProcess*, char*, int ) ), SLOT( slotReceivedStderr( KProcess*, char*, int ) ) );
+    connect( script, SIGNAL( receivedStdout( KProcess*, char*, int ) ), SLOT( slotReceivedStdout( KProcess*, char*, int ) ) );
+    connect( script, SIGNAL( processExited( KProcess* ) ), SLOT( scriptFinished( KProcess* ) ) );
+
+    if( !script->start( KProcess::NotifyOnExit ) ) {
         KMessageBox::sorry( 0, i18n( "<p>Could not start the script <i>%1</i>.</p>"
                                      "<p>Please make sure that the file has execute (+x) permissions.</p>" ).arg( name ) );
         delete script;
         return false;
     }
 
-    li->setPixmap( 0, SmallIcon( "player_play" ) );
+    li->setPixmap( 0, SmallIcon( amaroK::icon( "play" ) ) );
     debug() << "Running script: " << url.path() << endl;
 
     m_scripts[name].process = script;
     slotCurrentChanged( m_gui->listView->currentItem() );
-//     connect( script, SIGNAL( receivedStdout( KProcess*, char*, int ) ), SLOT( slotReceivedStdout( KProcess*, char*, int ) ) );
-    connect( script, SIGNAL( processExited( KProcess* ) ), SLOT( scriptFinished( KProcess* ) ) );
+    if( m_scripts[name].type == "lyrics" )
+        emit lyricsScriptChanged();
+
     return true;
 }
 
@@ -463,93 +554,85 @@ ScriptManager::slotRunScript()
 void
 ScriptManager::slotStopScript()
 {
-    DEBUG_BLOCK
-
     QListViewItem* const li = m_gui->listView->currentItem();
     const QString name = li->text( 0 );
 
-    // Kill script process (with SIGTERM)
-    m_scripts[name].process->kill();
-    m_scripts[name].process->detach();
+    // Just a sanity check
+    if( m_scripts.find( name ) == m_scripts.end() )
+        return;
 
-    delete m_scripts[name].process;
-    m_scripts[name].process = 0;
+    terminateProcess( &m_scripts[name].process );
+    m_scripts[name].log = QString::null;
     slotCurrentChanged( m_gui->listView->currentItem() );
 
-    li->setPixmap( 0, SmallIcon( "stop" ) );
+    li->setPixmap( 0, QPixmap() );
 }
 
 
 void
 ScriptManager::slotConfigureScript()
 {
-    DEBUG_BLOCK
-
     const QString name = m_gui->listView->currentItem()->text( 0 );
-    if ( !m_scripts[name].process ) return;
+    if( !m_scripts[name].process ) return;
 
     const KURL url = m_scripts[name].url;
     QDir::setCurrent( url.directory() );
 
     m_scripts[name].process->writeStdin( "configure" );
-
-    debug() << "Starting script configuration." << endl;
 }
 
 
 void
 ScriptManager::slotAboutScript()
 {
-    DEBUG_BLOCK
-
     const QString name = m_gui->listView->currentItem()->text( 0 );
-    QFile file( m_scripts[name].url.directory( false ) + "README" );
-    debug() << "Path: " << file.name() << endl;
+    QFile readme( m_scripts[name].url.directory( false ) + "README" );
+    QFile license( m_scripts[name].url.directory( false ) + "COPYING" );
 
-    if ( !file.open( IO_ReadOnly ) ) {
+    if( !readme.open( IO_ReadOnly ) ) {
         KMessageBox::sorry( 0, i18n( "There is no information available for this script." ) );
         return;
     }
 
-    KTextEdit* editor = new KTextEdit();
-    kapp->setTopWidget( editor );
-    editor->setCaption( kapp->makeStdCaption( i18n( "About %1" ).arg( name ) ) );
-    editor->setReadOnly( true );
+    KAboutDialog* about = new KAboutDialog( KAboutDialog::AbtTabbed|KAboutDialog::AbtProduct,
+                                            QString::null,
+                                            KDialogBase::Ok, KDialogBase::Ok, this );
+    kapp->setTopWidget( about );
+    about->setCaption( kapp->makeStdCaption( i18n( "About %1" ).arg( name ) ) );
+    about->setProduct( "", "", "", "" );
+    // Get rid of the confusing KDE version text
+    QLabel* product = static_cast<QLabel*>( about->mainWidget()->child( "version" ) );
+    if( product ) product->setText( i18n( "%1 amaroK Script" ).arg( name ) );
 
-    QFont font( "fixed" );
-    font.setFixedPitch( true );
-    font.setStyleHint( QFont::TypeWriter );
-    editor->setFont( font );
+    about->addTextPage( i18n( "About" ), readme.readAll(), true );
+    if( license.open( IO_ReadOnly ) )
+        about->addLicensePage( i18n( "License" ), license.readAll() );
 
-    QTextStream stream( &file );
-    editor->setText( stream.read() );
-    editor->setTextFormat( QTextEdit::PlainText );
-    editor->resize( 640, 480 );
-    editor->show();
+    about->setInitialSize( QSize( 500, 350 ) );
+    about->show();
 }
 
 
 void
 ScriptManager::slotShowContextMenu( QListViewItem* item, const QPoint& pos )
 {
-    DEBUG_BLOCK
-    if ( !item ) return;
+    if( !item ) return;
 
     // Look up script entry in our map
     ScriptMap::Iterator it;
     ScriptMap::Iterator end( m_scripts.end() );
-    for ( it = m_scripts.begin(); it != end; ++it )
-        if ( it.data().li == item ) break;
+    for( it = m_scripts.begin(); it != end; ++it )
+        if( it.data().li == item ) break;
 
     enum { SHOW_LOG, EDIT };
     KPopupMenu menu;
     menu.insertTitle( i18n( "Debugging" ) );
-    menu.insertItem( SmallIconSet( "history" ), i18n( "Show Output &Log" ), SHOW_LOG );
+    menu.insertItem( SmallIconSet( amaroK::icon( "clock" ) ), i18n( "Show Output &Log" ), SHOW_LOG );
     menu.insertItem( SmallIconSet( "edit" ), i18n( "&Edit" ), EDIT );
     menu.setItemEnabled( SHOW_LOG, it.data().process );
     const int id = menu.exec( pos );
 
-    switch ( id )
+    switch( id )
     {
         case EDIT:
             KRun::runCommand( "kwrite " + it.data().url.path() );
@@ -557,10 +640,10 @@ ScriptManager::slotShowContextMenu( QListViewItem* item, const QPoint& pos )
 
         case SHOW_LOG:
             QString line;
-            while ( it.data().process->readln( line ) != -1 )
+            while( it.data().process->readln( line ) != -1 )
                 it.data().log += line;
 
-            KTextEdit* editor = new KTextEdit();
+            KTextEdit* editor = new KTextEdit( it.data().log );
             kapp->setTopWidget( editor );
             editor->setCaption( kapp->makeStdCaption( i18n( "Output Log for %1" ).arg( it.key() ) ) );
             editor->setReadOnly( true );
@@ -570,57 +653,60 @@ ScriptManager::slotShowContextMenu( QListViewItem* item, const QPoint& pos )
             font.setStyleHint( QFont::TypeWriter );
             editor->setFont( font );
 
-            editor->setText( it.data().log );
             editor->setTextFormat( QTextEdit::PlainText );
-            editor->resize( 640, 480 );
+            editor->resize( 500, 380 );
             editor->show();
             break;
     }
 }
 
 
+/* This is just a workaround, some scripts crash for some people if stdout is not handled. */
 void
-ScriptManager::slotReceivedStdout( KProcess* process, char* buf, int len )
+ScriptManager::slotReceivedStdout( KProcess*, char* buf, int len )
 {
-    DEBUG_BLOCK
+    debug() << QString::fromLatin1( buf, len ) << endl;
+}
 
+
+void
+ScriptManager::slotReceivedStderr( KProcess* process, char* buf, int len )
+{
     // Look up script entry in our map
     ScriptMap::Iterator it;
     ScriptMap::Iterator end( m_scripts.end() );
-    for ( it = m_scripts.begin(); it != end; ++it )
-        if ( it.data().process == process ) break;
+    for( it = m_scripts.begin(); it != end; ++it )
+        if( it.data().process == process ) break;
 
-    it.data().log += QString::fromLatin1( buf, len );
+    const QString text = QString::fromLatin1( buf, len );
+    error() << it.key() << ":\n" << text << endl;
+
+    if( it.data().log.length() > 20000 )
+        it.data().log = "==== LOG TRUNCATED HERE ====\n";
+    it.data().log += text;
 }
 
 
 void
 ScriptManager::scriptFinished( KProcess* process ) //SLOT
 {
-    DEBUG_BLOCK
-
     // Look up script entry in our map
     ScriptMap::Iterator it;
     ScriptMap::Iterator end( m_scripts.end() );
-    for ( it = m_scripts.begin(); it != end; ++it )
-        if ( it.data().process == process ) break;
+    for( it = m_scripts.begin(); it != end; ++it )
+        if( it.data().process == process ) break;
 
     // Check if there was an error on exit
-    if ( process->normalExit() && process->exitStatus() != 0 ) {
-        // Read Stderr log
-        KProcIO* proc = static_cast<KProcIO*>( process );
-        QString line, details;
-        while ( proc->readln( line ) != -1 )
-            details += line;
-
+    if( process->normalExit() && process->exitStatus() != 0 )
         KMessageBox::detailedError( 0, i18n( "The script '%1' exited with error code: %2" )
-                                           .arg( it.key() ).arg( process->exitStatus() ), details );
-    }
+                                           .arg( it.key() ).arg( process->exitStatus() )
+                                           ,it.data().log );
 
     // Destroy script process
     delete it.data().process;
     it.data().process = 0;
-    it.data().li->setPixmap( 0, SmallIcon( "stop" ) );
+    it.data().log = QString::null;
+    it.data().li->setPixmap( 0, QPixmap() );
     slotCurrentChanged( m_gui->listView->currentItem() );
 }
 
@@ -629,18 +715,25 @@ ScriptManager::scriptFinished( KProcess* process ) //SLOT
 // private
 ////////////////////////////////////////////////////////////////////////////////
 
+void
+ScriptManager::terminateProcess( KProcIO** proc )
+{
+    if( *proc ) {
+        (*proc)->kill(); // Sends SIGTERM
+        (*proc)->detach();
+
+        delete *proc;
+        *proc = 0;
+    }
+}
+
+
 void
 ScriptManager::notifyScripts( const QString& message )
 {
-    DEBUG_BLOCK
-
-    debug() << "Sending notification: " << message << endl;
-
-    ScriptMap::Iterator it;
-    ScriptMap::Iterator end( m_scripts.end() );
-    for ( it = m_scripts.begin(); it != end; ++it ) {
+    foreachType( ScriptMap, m_scripts ) {
         KProcIO* const proc = it.data().process;
-        if ( proc ) proc->writeStdin( message );
+        if( proc ) proc->writeStdin( message );
     }
 }
 
@@ -648,21 +741,38 @@ ScriptManager::notifyScripts( const QString& message )
 void
 ScriptManager::loadScript( const QString& path )
 {
-    DEBUG_BLOCK
-
-    if ( !path.isEmpty() ) {
+    if( !path.isEmpty() ) {
         const KURL url = KURL::fromPathOrURL( path );
+        QString name = url.fileName();
+        QString type = "generic";
 
-        KListViewItem* li = new KListViewItem( m_gui->listView, url.fileName() );
-        li->setPixmap( 0, SmallIcon( "stop" ) );
+        // Read and parse .spec file, if exists
+        QFileInfo info( path );
+        const QString specPath = info.dirPath() + "/" + info.baseName() + ".spec";
+        if( QFile::exists( specPath ) ) {
+            KConfig spec( specPath, true, false );
+            if( spec.hasKey( "name" ) )
+                name = spec.readEntry( "name" );
+            if( spec.hasKey( "type" ) ) {
+                type = spec.readEntry( "type" );
+                if( type == "lyrics" )
+                    name.prepend( i18n( "Lyrics" ) + ": " );
+                if( type == "transcode" )
+                    name.prepend( i18n( "Transcoding" ) + ": " );
+            }
+        }
+
+        KListViewItem* li = new KListViewItem( m_gui->listView, name );
+        li->setPixmap( 0, QPixmap() );
 
         ScriptItem item;
         item.url = url;
+        item.type = type;
         item.process = 0;
         item.li = li;
 
-        m_scripts[url.fileName()] = item;
-        debug() << "Loaded: " << url.fileName() << endl;
+        m_scripts[name] = item;
+        debug() << "Loaded: " << name << endl;
 
         slotCurrentChanged( m_gui->listView->currentItem() );
     }
@@ -672,9 +782,7 @@ ScriptManager::loadScript( const QString& path )
 void
 ScriptManager::engineStateChanged( Engine::State state, Engine::State /*oldState*/ )
 {
-    DEBUG_BLOCK
-
-    switch ( state )
+    switch( state )
     {
         case Engine::Empty:
             notifyScripts( "engineStateChange: empty" );
