@@ -1,5 +1,7 @@
 // (c) 2004 Christian Muehlhaeuser <chris@chris.de>
 // (c) 2004 Sami Nieminen <sami.nieminen@iki.fi>
+// (c) 2006 Shane King <kde@dontletsstart.com>
+// (c) 2006 Iain Benson <iain@arctos.me.uk>
 // See COPYING file for licensing information
 
 #ifndef AMAROK_SCROBBLER_H
@@ -10,6 +12,7 @@
 #include <qobject.h>
 #include <qptrdict.h>
 #include <qptrlist.h>
+#include <qtimer.h>
 
 //some setups require this
 #undef PROTOCOL_VERSION
@@ -23,6 +26,8 @@ class ScrobblerSubmitter;
 
 class Scrobbler : public QObject, public EngineObserver
 {
+    friend class MediaDevice;
+
     Q_OBJECT
 
     public:
@@ -34,6 +39,9 @@ class Scrobbler : public QObject, public EngineObserver
     signals:
         void similarArtistsFetched( const QString& artist, const QStringList& suggestions );
 
+    public slots:
+        void subTrack( long currentPos, long startPos, long endPos ); // cuefiles can update length without track change
+
     protected:
         Scrobbler();
         ~Scrobbler();
@@ -47,10 +55,13 @@ class Scrobbler : public QObject, public EngineObserver
             KIO::Job* /*job*/, const QByteArray& /*data*/ );
 
     private:
-        QString m_similarArtistsBuffer;
+        QTimer m_timer; //works around xine bug 
+                        //http://sourceforge.net/tracker/index.php?func=detail&aid=1401026&group_id=9655&atid=109655
+        QByteArray m_similarArtistsBuffer;
         KIO::Job* m_similarArtistsJob;
         QString m_artist;
         bool m_validForSending;
+        long m_startPos;
         ScrobblerSubmitter* m_submitter;
         SubmitItem* m_item;
 };
@@ -58,19 +69,23 @@ class Scrobbler : public QObject, public EngineObserver
 
 class SubmitItem
 {
+    friend class ScrobblerSubmitter;
+
     public:
         SubmitItem(
             const QString& /*artist*/,
             const QString& /*album*/,
             const QString& /*title*/,
-            int /*length*/ );
+            int /*length*/,
+            bool now = true );
         SubmitItem( const QDomElement& /* domElement */ );
+        SubmitItem();
 
         bool operator==( const SubmitItem& item );
 
-        const void setArtist( const QString& artist ) { m_artist = artist; }
-        const void setAlbum( const QString& album ) { m_album = album; }
-        const void setTitle( const QString& title ) { m_title = title; }
+        void setArtist( const QString& artist ) { m_artist = artist; }
+        void setAlbum( const QString& album ) { m_album = album; }
+        void setTitle( const QString& title ) { m_title = title; }
         const QString artist() const { return m_artist; }
         const QString album() const { return m_album; }
         const QString title() const { return m_title; }
@@ -79,6 +94,8 @@ class SubmitItem
 
         QDomElement toDomElement( QDomDocument& /* document */ ) const;
 
+        bool valid() const { return !m_artist.isEmpty() && !m_title.isEmpty() && m_length >= 30; }
+
     private:
         QString m_artist;
         QString m_album;
@@ -108,18 +125,14 @@ class ScrobblerSubmitter : public QObject
         ScrobblerSubmitter();
         ~ScrobblerSubmitter();
 
-        void handshake();
         void submitItem( SubmitItem* /* item */ );
 
-        const QString username() const { return m_username; }
-        const QString password() const { return m_password; }
-        const bool enabled() const { return m_scrobblerEnabled; }
+        void configure( const QString& /*username*/, const QString& /* password*/, bool /*enabled*/ );
 
-        void setUsername( const QString& /*username*/ );
-        void setPassword( const QString& /* password*/ );
-        void setEnabled( bool /*enabled*/ );
+        void syncComplete();
 
     private slots:
+        void scheduledTimeReached();
         void audioScrobblerHandshakeResult( KIO::Job* /*job*/ );
         void audioScrobblerSubmitResult( KIO::Job* /*job*/ );
         void audioScrobblerSubmitData(
@@ -135,6 +148,15 @@ class ScrobblerSubmitter : public QObject
             SubmitItem* /* item */, int /* tracks */, bool /* success */ ) const;
         void saveSubmitQueue();
         void readSubmitQueue();
+        bool schedule( bool failure );
+        void performHandshake();
+        void performSubmit();
+
+        // on failure, start at MIN_BACKOFF, and double on subsequent failures
+        // until MAX_BACKOFF is reached
+        static const int MIN_BACKOFF = 60;
+        static const int MAX_BACKOFF = 60 * 60;
+
         QString m_submitResultBuffer;
         QString m_username;
         QString m_password;
@@ -142,11 +164,20 @@ class ScrobblerSubmitter : public QObject
         QString m_challenge;
         QString m_savePath;
         bool m_scrobblerEnabled;
+        bool m_holdFakeQueue;
+        bool m_inProgress;
+        bool m_needHandshake;
         uint m_prevSubmitTime;
         uint m_interval;
+        uint m_backoff;
+        uint m_lastSubmissionFinishTime;
+        uint m_fakeQueueLength;
 
         QPtrDict<SubmitItem> m_ongoingSubmits;
-        SubmitQueue m_submitQueue;
+        SubmitQueue m_submitQueue; // songs played by amaroK
+        SubmitQueue m_fakeQueue; // songs for which play times have to be faked (e.g. when submitting from media device)
+
+        QTimer m_timer;
 };
 
 
