@@ -1,8 +1,12 @@
-// (c) Pierpaolo Di Panfilo 2004
+/// (c) Pierpaolo Di Panfilo 2004
 // (c) Alexandre Pereira de Oliveira 2005
+// (c) 2005 Isaiah Damron <xepo@trifault.net>
 // See COPYING file for licensing information
 
+#define DEBUG_PREFIX "SmartPlaylistEditor"
+
 #include "amarok.h" //foreach
+#include "debug.h"
 #include "collectiondb.h"
 #include "metabundle.h"
 #include "smartplaylisteditor.h"
@@ -34,38 +38,57 @@ QStringList m_expandableDbFields;
 
 
 SmartPlaylistEditor::SmartPlaylistEditor( QString defaultName, QWidget *parent, const char *name )
-    : KDialogBase( parent, name, true, i18n("Create Smart Playlist"), Ok|Cancel)
+    : KDialogBase( parent, name, true, i18n("Create Smart Playlist"),
+      Ok|Cancel, Ok, true )
 {
     init(defaultName);
-    addCriteria();
+    addCriteriaAny();
+    addCriteriaAll();
 }
 
 
 SmartPlaylistEditor::SmartPlaylistEditor( QWidget *parent, QDomElement xml, const char *name)
-    : KDialogBase( parent, name, true, i18n("Edit Smart Playlist"), Ok|Cancel)
+    : KDialogBase( parent, name, true, i18n("Edit Smart Playlist"),
+      Ok|Cancel, Ok, true )
 {
     init( xml.attribute( "name" ) );
     // matches
     QDomNodeList matchesList =  xml.elementsByTagName( "matches" );
-    if ( matchesList.count() ) {
-        m_matchCheck->setChecked( true );
+    bool matchedANY = false, matchedALL = false;
+    
+    m_matchAllCheck->setChecked( true );
+    m_matchAnyCheck->setChecked( true );
 
-        QDomElement matches = matchesList.item(0).toElement(); // we only allow one matches node
+    for (int i = 0, m = matchesList.count(); i<m; i++) {
+        QDomElement matches = matchesList.item(i).toElement();
         QDomNodeList criteriaList =  matches.elementsByTagName( "criteria" );
-        for (int i = 0, c=criteriaList.count() ; i<c; ++i ) {
-
-            QDomElement criteria = criteriaList.item(i).toElement();
-            addCriteria( criteria );
+        
+        if ( criteriaList.count() ) {
+            for (int j = 0, c=criteriaList.count() ; j<c; ++j ) {
+                QDomElement criteria = criteriaList.item(j).toElement();
+              
+                if (matches.attribute( "glue" ) == "OR") {
+                    matchedANY = true;
+                    addCriteriaAny( criteria );
+                }
+                else {
+                    matchedALL = true;
+                    addCriteriaAll( criteria );
+                }
+            }
         }
-        if ( matches.attribute( "glue" ) == "AND" )
-            m_matchCombo->setCurrentItem( 0 );
-        else
-            m_matchCombo->setCurrentItem( 1 );
     }
-    else {
-        addCriteria();
-        m_matchCheck->setChecked( false );
+
+    if ( !matchedALL ) {
+        addCriteriaAll();
+        m_matchAllCheck->setChecked( false );
     }
+
+    if ( !matchedANY ) {
+        m_matchAnyCheck->setChecked( false );
+        addCriteriaAny( );
+    }
+
     // orderby
     QDomNodeList orderbyList =  xml.elementsByTagName( "orderby" );
     if ( orderbyList.count() ) {
@@ -73,7 +96,8 @@ SmartPlaylistEditor::SmartPlaylistEditor( QWidget *parent, QDomElement xml, cons
         QDomElement orderby = orderbyList.item(0).toElement(); // we only allow one orderby node
 
         //random is always the last one.
-        int dbfield = orderby.attribute( "field" ) == "random" ? m_dbFields.count() : m_dbFields.findIndex( orderby.attribute( "field" ) );
+        int dbfield = orderby.attribute( "field" ) == "random" ? m_dbFields.count()-1 : m_dbFields.findIndex( orderby.attribute( "field" ) );
+
         m_orderCombo->setCurrentItem( dbfield );
         updateOrderTypes( dbfield );
         if ( orderby.attribute( "order" ) == "DESC" || orderby.attribute( "order" ) == "weighted" )
@@ -105,14 +129,14 @@ void SmartPlaylistEditor::init(QString defaultName)
 
     m_fields.clear();
     m_fields << i18n("Artist") << i18n("Album") << i18n("Genre") << i18n("Title") << i18n("Length") << i18n("Track #") << i18n("Year")
-             << i18n("Comment") << i18n("Play Counter") << i18n("Score") << i18n("First Play") << i18n("Last Play")
+             << i18n("Comment") << i18n("Play Counter") << i18n("Score") << i18n( "Rating" ) << i18n("First Play") << i18n("Last Play")
              << i18n("Modified Date") << i18n("File Path");
 
     m_dbFields.clear();
     m_dbFields << "artist.name" << "album.name" << "genre.name" << "tags.title" << "tags.length"
                << "tags.track" << "year.name" << "tags.comment" << "statistics.playcounter"
-               << "statistics.percentage" << "statistics.createdate" << "statistics.accessdate"
-               << "tags.createdate" << "tags.url";
+               << "statistics.percentage" << "statistics.rating" << "statistics.createdate"
+               << "statistics.accessdate" << "tags.createdate" << "tags.url" << "tags.sampler";
 
     m_expandableFields.clear();
     m_expandableFields << i18n("Artist") << i18n("Album") << i18n("Genre") <<  i18n("Year");
@@ -128,18 +152,21 @@ void SmartPlaylistEditor::init(QString defaultName)
     QFrame *sep = new QFrame( mainWidget() );
     sep->setFrameStyle( QFrame::HLine | QFrame::Sunken );
 
-    //match box
-    QHBox *matchBox = new QHBox( mainWidget() );
-    m_matchCheck = new QCheckBox( i18n("Match the following condition" ), matchBox );
-    m_matchCheck->setChecked( true );
-    m_matchCombo = new KComboBox( matchBox );
-    m_matchCombo->insertItem( i18n("All") );
-    m_matchCombo->insertItem( i18n("Any") );
-    m_matchLabel = new QLabel( i18n(" of the following conditions"), matchBox );
-    matchBox->setStretchFactor( new QWidget( matchBox ), 1 );
+    //match box (any)
+    QHBox *matchAnyBox = new QHBox( mainWidget() );
+    m_matchAnyCheck = new QCheckBox( i18n("Match Any of the following conditions" ), matchAnyBox );
+    matchAnyBox->setStretchFactor( new QWidget( matchAnyBox ), 1 );
 
     //criteria box
-    m_criteriaGroupBox = new QVGroupBox( QString::null, mainWidget() );
+    m_criteriaAnyGroupBox = new QVGroupBox( QString::null, mainWidget() );
+
+    //match box (all)
+    QHBox *matchAllBox = new QHBox( mainWidget() );
+    m_matchAllCheck = new QCheckBox( i18n("Match All of the following conditions" ), matchAllBox );
+    matchAllBox->setStretchFactor( new QWidget( matchAllBox ), 1 );
+
+    //criteria box
+    m_criteriaAllGroupBox = new QVGroupBox( QString::null, mainWidget() );
 
     //order box
     QHBox *hbox2 = new QHBox( mainWidget() );
@@ -179,12 +206,16 @@ void SmartPlaylistEditor::init(QString defaultName)
     //add stretch
     static_cast<QHBox *>(mainWidget())->setStretchFactor(new QWidget(mainWidget()), 1);
 
-    connect( m_matchCheck, SIGNAL( toggled(bool) ), m_criteriaGroupBox, SLOT( setEnabled(bool) ) );
+    connect( m_matchAnyCheck, SIGNAL( toggled(bool) ), m_criteriaAnyGroupBox, SLOT( setEnabled(bool) ) );
+    connect( m_matchAllCheck, SIGNAL( toggled(bool) ), m_criteriaAllGroupBox, SLOT( setEnabled(bool) ) );
     connect( m_orderCheck, SIGNAL( toggled(bool) ), orderBox, SLOT( setEnabled(bool) ) );
     connect( m_limitCheck, SIGNAL( toggled(bool) ), limitBox, SLOT(  setEnabled(bool) ) );
     connect( m_expandCheck, SIGNAL( toggled(bool) ), expandBox, SLOT( setEnabled(bool) ) );
     connect( m_orderCombo, SIGNAL( activated(int) ), this, SLOT( updateOrderTypes(int) ) );
 
+    m_criteriaAnyGroupBox->setEnabled( false );
+    m_criteriaAllGroupBox->setEnabled( false );
+
     orderBox->setEnabled( false );
     limitBox->setEnabled( false );
     expandBox->setEnabled( false );
@@ -195,32 +226,52 @@ void SmartPlaylistEditor::init(QString defaultName)
 }
 
 
-void SmartPlaylistEditor::addCriteria()
+void SmartPlaylistEditor::addCriteriaAny()
 {
-    CriteriaEditor *criteria = new CriteriaEditor( this, m_criteriaGroupBox );
-    m_criteriaEditorList.append( criteria );
-    updateMatchWidgets();
-    m_criteriaEditorList.first()->enableRemove( m_criteriaEditorList.count() > 1 );
+    CriteriaEditor *criteria= new CriteriaEditor( this, m_criteriaAnyGroupBox, criteriaAny );
+    m_criteriaEditorAnyList.append( criteria );
+    m_criteriaEditorAnyList.first()->enableRemove( m_criteriaEditorAnyList.count() > 1 );
 }
 
-void SmartPlaylistEditor::addCriteria( QDomElement &xml )
+void SmartPlaylistEditor::addCriteriaAll()
 {
-    CriteriaEditor *criteria = new CriteriaEditor( this, m_criteriaGroupBox, xml );
-    m_criteriaEditorList.append( criteria );
-    updateMatchWidgets();
-    m_criteriaEditorList.first()->enableRemove( m_criteriaEditorList.count() > 1 );
+    CriteriaEditor *criteria= new CriteriaEditor( this, m_criteriaAllGroupBox, criteriaAll );
+    m_criteriaEditorAllList.append( criteria );
+    m_criteriaEditorAllList.first()->enableRemove( m_criteriaEditorAllList.count() > 1 );
 }
 
-
-void SmartPlaylistEditor::removeCriteria( CriteriaEditor *criteria )
+void SmartPlaylistEditor::addCriteriaAny( QDomElement &xml )
 {
-    m_criteriaEditorList.remove( criteria );
+    CriteriaEditor *criteria = new CriteriaEditor( this, m_criteriaAnyGroupBox, criteriaAny, xml );
+    m_criteriaEditorAnyList.append( criteria );
+    m_criteriaEditorAnyList.first()->enableRemove( m_criteriaEditorAnyList.count() > 1 );
+}
+
+void SmartPlaylistEditor::addCriteriaAll( QDomElement &xml )
+{
+    CriteriaEditor *criteria = new CriteriaEditor( this, m_criteriaAllGroupBox, criteriaAll, xml );
+    m_criteriaEditorAllList.append( criteria );
+    m_criteriaEditorAllList.first()->enableRemove( m_criteriaEditorAllList.count() > 1 );
+}
+
+void SmartPlaylistEditor::removeCriteriaAny( CriteriaEditor *criteria )
+{
+    m_criteriaEditorAnyList.remove( criteria );
     criteria->deleteLater();
-    updateMatchWidgets();
     resize( size().width(), sizeHint().height() );
+    
+    if( m_criteriaEditorAnyList.count() == 1 )
+	m_criteriaEditorAnyList.first()->enableRemove( false );
+}
 
-    if( m_criteriaEditorList.count() == 1 )
-        m_criteriaEditorList.first()->enableRemove( false );
+void SmartPlaylistEditor::removeCriteriaAll( CriteriaEditor *criteria )
+{
+    m_criteriaEditorAllList.remove( criteria );
+    criteria->deleteLater();
+    resize( size().width(), sizeHint().height() );
+    
+    if( m_criteriaEditorAllList.count() == 1 )
+	m_criteriaEditorAllList.first()->enableRemove( false );
 }
 
 void SmartPlaylistEditor::updateOrderTypes( int index )
@@ -258,19 +309,30 @@ QDomElement SmartPlaylistEditor::result() {
 
     nodeE.appendChild( smartplaylist );
     // Matches
-    if( m_matchCheck->isChecked() ) {
+    if( m_matchAnyCheck->isChecked() ) {
         QDomElement matches = doc.createElement("matches");
         smartplaylist.appendChild( matches );
         // Iterate through all criteria list
-        CriteriaEditor *criteriaeditor = m_criteriaEditorList.first();
-        for( int i=0; criteriaeditor; criteriaeditor = m_criteriaEditorList.next(), ++i ) {
+        CriteriaEditor *criteriaeditor = m_criteriaEditorAnyList.first();
+        for( int i=0; criteriaeditor; criteriaeditor = m_criteriaEditorAnyList.next(), ++i ) {
             matches.appendChild( doc.importNode( criteriaeditor->getDomSearchCriteria( doc ), true ) );
         }
-        if ( m_criteriaEditorList.count() > 1 ) {
-            matches.setAttribute( "glue", m_matchCombo->currentItem() == 0 ? "AND" : "OR" );
-        }
+        matches.setAttribute( "glue",  "OR" );
         smartplaylist.appendChild( matches );
     }
+
+    if( m_matchAllCheck->isChecked() ) {
+        QDomElement matches = doc.createElement("matches");
+        smartplaylist.appendChild( matches );
+        // Iterate through all criteria list
+        CriteriaEditor *criteriaeditor = m_criteriaEditorAllList.first();
+        for( int i=0; criteriaeditor; criteriaeditor = m_criteriaEditorAllList.next(), ++i ) {
+            matches.appendChild( doc.importNode( criteriaeditor->getDomSearchCriteria( doc ), true ) );
+        }
+        matches.setAttribute( "glue",  "AND" );
+        smartplaylist.appendChild( matches );
+    }
+    
     // Order By
     if( m_orderCheck->isChecked() ) {
         QDomElement orderby = doc.createElement("orderby");
@@ -302,50 +364,69 @@ QDomElement SmartPlaylistEditor::result() {
 
 void SmartPlaylistEditor::buildQuery()
 {
+    DEBUG_BLOCK
 
-    QString joins = "tags INNER JOIN year ON year.id=tags.year INNER JOIN genre ON genre.id=tags.genre"
-                    " INNER JOIN artist ON artist.id=tags.artist INNER JOIN album ON album.id=tags.album";
+    QString joins = "tags LEFT JOIN year ON year.id=tags.year LEFT JOIN genre ON genre.id=tags.genre"
+                    " LEFT JOIN artist ON artist.id=tags.artist LEFT JOIN album ON album.id=tags.album";
     QString whereStr;
     QString criteriaListStr;
     QString orderStr;
     QString limitStr;
-
+    
     //where expression
-    if( m_matchCheck->isChecked() ) {
-        criteriaListStr += " (";
-
-        CriteriaEditor *criteria = m_criteriaEditorList.first();
-        for( int i=0; criteria; criteria = m_criteriaEditorList.next(), i++ ) {
-
-            QString str = criteria->getSearchCriteria();
-            //add the table used in the search expression to tables
-            QString table = str.left( str.find('.') );
-             if( !joins.contains( table ) ) {
-                // that makes it possible to search for tracks never played. it looks ugly but is works
-                if( str.contains(" OR statistics.playcounter IS NULL"))
+    if( m_matchAnyCheck->isChecked() || m_matchAllCheck->isChecked() ) {
+        int i = 0;
+        
+        if( m_matchAnyCheck->isChecked() ) {
+            criteriaListStr += "( (";
+            
+            CriteriaEditor *criteria = m_criteriaEditorAnyList.first();
+            for( i=0; criteria; criteria = m_criteriaEditorAnyList.next(), i++ ) {
+                
+                QString str = criteria->getSearchCriteria();
+                if( str.contains( "statistics." ) && !joins.contains( "statistics" ) )
                     joins += " LEFT JOIN statistics ON statistics.url=tags.url";
-                else
-                    joins += " INNER JOIN statistics ON statistics.url=tags.url";
+                
+                if( i ) //multiple conditions
+                    str.prepend( " OR (");
+                
+                criteriaListStr += str+")";
             }
-            if( i ) { //multiple conditions
-                QString op = m_matchCombo->currentItem() == 0 ? "AND" : "OR";
-                str.prepend( " " + op + " (");
-
-            }
-            criteriaListStr += str+")";
+            
+            criteriaListStr += " )"; // we want our ORs all in one bracket. :-)
         }
-        whereStr = " WHERE" + criteriaListStr;
+        
+        if( m_matchAllCheck->isChecked() ) {
+            if ( i ) // conditions exist from above
+                criteriaListStr += " AND ";
+            
+            criteriaListStr += "( (";
+            
+            CriteriaEditor *criteria2 = m_criteriaEditorAllList.first();
+            for( i=0; criteria2; criteria2 = m_criteriaEditorAllList.next(), i++ ) {
+                
+                QString str = criteria2->getSearchCriteria();
+                if( str.contains( "statistics." ) && !joins.contains( "statistics" ) )
+                    joins += " LEFT JOIN statistics ON statistics.url=tags.url";
+                
+                if( i ) //multiple conditions
+                    str.prepend( " AND (");
+                
+                criteriaListStr += str+")";
+            }
+            criteriaListStr += " )";
+        }
+        
+        whereStr = " WHERE " + criteriaListStr;
     }
-
+    
     //order by expression
     if( m_orderCheck->isChecked() ) {
         if( m_orderCombo->currentItem() != m_orderCombo->count()-1 ) {
             QString field = m_dbFields[ m_orderCombo->currentItem() ];
-            QString table = field.left( field.find('.') );
-            if( !joins.contains( table ) ) {
-                if( table=="statistics")
-                    joins += " INNER JOIN statistics ON statistics.url=tags.url";
-            }
+            if( field.contains( "statistics." ) && !joins.contains( "statistics" ) )
+                joins += " LEFT JOIN statistics ON statistics.url=tags.url";
+
             QString orderType = m_orderTypeCombo->currentItem() == 1 ? " DESC" : " ASC";
             orderStr = " ORDER BY " +  field + orderType;
         }
@@ -367,57 +448,42 @@ void SmartPlaylistEditor::buildQuery()
             */
            orderStr = " ORDER BY POWER(" + CollectionDB::instance()->randomFunc() + ",1.0/(statistics.percentage+1)) DESC";
             if( !joins.contains( "statistics" ) ) {
-                joins += " INNER JOIN statistics ON statistics.url=tags.url";
+                joins += " LEFT JOIN statistics ON statistics.url=tags.url";
             }
         }
     }
 
-    if( m_limitCheck->isChecked() )
-        limitStr = " LIMIT 0," + QString::number( m_limitSpin->value() );
+    if( m_limitCheck->isChecked() ) 
+        limitStr = " LIMIT " + QString::number( m_limitSpin->value() )+" OFFSET 0 ";
+    
 
-    // album / artist / genre / title / year / comment / track / bitrate / length / samplerate / path
-    m_query = "SELECT album.name, artist.name, genre.name, tags.title, year.name, "
-                    "tags.comment, tags.track, tags.bitrate, tags.length, tags.samplerate, tags.url"
+    // album / artist / genre / title / year / comment / track / bitrate / length / samplerate / path / compilation
+    m_query = "SELECT album.name, artist.name, genre.name, tags.title, year.name, tags.comment, tags.track, "
+                    "tags.bitrate, tags.length, tags.samplerate, tags.filesize, tags.url, tags.sampler"
                     " FROM " + joins + whereStr + orderStr + limitStr + ";";
 
     if( m_expandCheck->isChecked() ) { //We use "(*ExpandString*)" as a marker, if a artist/track/album has this bizarre name, it won't work.
         QString field = m_expandableDbFields[ m_expandCombo->currentItem() ];
         QString table = field.left( field.find('.') );
         if( !joins.contains( table ) ) {
-            joins += " INNER JOIN statistics ON statistics.url=tags.url";
+            joins += " LEFT JOIN statistics ON statistics.url=tags.url";
         }
         if ( !criteriaListStr.isEmpty() )
             whereStr = QString(" WHERE (%1) AND %2 = '(*ExpandString*)'").arg(criteriaListStr).arg(field);
         else
             whereStr = QString("WHERE %1 = '(*ExpandString*)'").arg(field);
         m_expandQuery = "SELECT album.name, artist.name, genre.name, tags.title, year.name, "
-                            "tags.comment, tags.track, tags.bitrate, tags.length, tags.samplerate, tags.url"
+                            "tags.comment, tags.track, tags.bitrate, tags.length, tags.samplerate, tags.filesize, tags.url, tags.sampler"
                             " FROM " + joins + whereStr + orderStr + limitStr + ";";
     }
 }
 
 
-void SmartPlaylistEditor::updateMatchWidgets()
-{
-    int count = m_criteriaEditorList.count();
-    if( count > 1 ) {
-        m_matchCheck->setText( i18n("Match") );
-        m_matchCombo->show();
-        m_matchLabel->show();
-    }
-    else {
-        m_matchCheck->setText( i18n("Match the following conditions" ) );
-        m_matchCombo->hide();
-        m_matchLabel->hide();
-    }
-}
-
-
 /////////////////////////////////////////////////////////////////////////////
 //    CLASS CriteriaEditor
 ////////////////////////////////////////////////////////////////////////////
 
-CriteriaEditor::CriteriaEditor( SmartPlaylistEditor *editor, QWidget *parent, QDomElement criteria )
+CriteriaEditor::CriteriaEditor( SmartPlaylistEditor *editor, QWidget *parent, int criteriaType, QDomElement criteria )
     : QHBox( parent )
     , m_playlistEditor( editor )
     , m_currentValueType( -1 )
@@ -442,8 +508,14 @@ CriteriaEditor::CriteriaEditor( SmartPlaylistEditor *editor, QWidget *parent, QD
 
     connect( m_fieldCombo,    SIGNAL( activated(int) ), SLOT( slotFieldSelected(int) ) );
     connect( m_criteriaCombo, SIGNAL( activated(int) ), SLOT( loadEditWidgets() ) );
-    connect( m_addButton, SIGNAL( clicked() ), editor, SLOT( addCriteria() ) );
-    connect( m_removeButton, SIGNAL( clicked() ), SLOT( slotRemoveCriteria() ) );
+    if (criteriaType == SmartPlaylistEditor::criteriaAny) {
+	connect( m_addButton, SIGNAL( clicked() ), editor, SLOT( addCriteriaAny() ) );
+	connect( m_removeButton, SIGNAL( clicked() ), SLOT( slotRemoveCriteriaAny() ) );
+    }
+    else {
+	connect( m_addButton, SIGNAL( clicked() ), editor, SLOT( addCriteriaAll() ) );
+	connect( m_removeButton, SIGNAL( clicked() ), SLOT( slotRemoveCriteriaAll() ) );
+    }
 
     if ( !criteria.isNull() ) {
         int field = m_dbFields.findIndex( criteria.attribute( "field" ) );
@@ -481,6 +553,13 @@ CriteriaEditor::CriteriaEditor( SmartPlaylistEditor *editor, QWidget *parent, QD
                     m_intSpinBox2->setValue( values.last().toInt() );
                 break;
             }
+            case Rating:
+            {
+                m_comboBox->setCurrentItem( ratingToIndex( values.first().toInt() ) );
+                if( condition == i18n("is between") )
+                    m_comboBox2->setCurrentItem( ratingToIndex( values.last().toInt() ) );
+                break;
+            }
             case Date:
             {
                 if( condition == i18n("is in the last") || condition == i18n("is not in the last") ) {
@@ -494,9 +573,13 @@ CriteriaEditor::CriteriaEditor( SmartPlaylistEditor *editor, QWidget *parent, QD
                         m_dateCombo->setCurrentItem(2);
                 }
                 else {
-                    m_dateEdit1->setDate( QDate::fromString( values.first() ) );
-                    if( condition == i18n("is between") )
-                        m_dateEdit2->setDate( QDate::fromString( values.last() ) );
+                    QDateTime dt;
+                    dt.setTime_t( values.first().toUInt() );
+                    m_dateEdit1->setDate( dt.date() );
+                    if( condition == i18n("is between") ) {
+                        dt.setTime_t( values.last().toUInt() );
+                        m_dateEdit2->setDate( dt.date() );
+                    }
                 }
                 break;
             }
@@ -531,10 +614,19 @@ QDomElement CriteriaEditor::getDomSearchCriteria( QDomDocument &doc )
             break;
          case Year: // fall through
          case Number:
+         {
             values << QString::number( m_intSpinBox1->value() );
             if( condition == i18n("is between")  )
                 values << QString::number( m_intSpinBox2->value() );
             break;
+         }
+         case Rating:
+         {
+            values << QString::number( indexToRating( m_comboBox->currentItem() ) );
+            if( condition == i18n("is between")  )
+                    values << QString::number( indexToRating( m_comboBox2->currentItem() ) );
+            break;
+         }
          case Date:
          {
             if( condition == i18n("is in the last") || condition == i18n("is not in the last") ) {
@@ -571,7 +663,10 @@ QString CriteriaEditor::getSearchCriteria()
     if( field.isEmpty() )
         return QString::null;
 
-    searchCriteria += field;
+    if ( ( field=="statistics.playcounter" || field=="statistics.rating" || field=="statistics.percentage" || field=="statistics.accessdate" || field=="statistics.createdate") )
+        searchCriteria += "COALESCE(" + field + ",0)";
+    else
+        searchCriteria += field;
 
     QString value;
     switch( getValueType( m_fieldCombo->currentItem() ) ) {
@@ -585,6 +680,13 @@ QString CriteriaEditor::getSearchCriteria()
             if( criteria == i18n("is between")  )
                 value += " AND " + QString::number( m_intSpinBox2->value() );
             break;
+        case Rating:
+        {
+            value = QString::number( indexToRating( m_comboBox->currentItem() ) );
+            if( criteria == i18n("is between")  )
+                value += " AND " + QString::number( indexToRating( m_comboBox2->currentItem() ) );
+            break;
+        }
         case Date:
         {
             if( criteria == i18n("is in the last") || criteria == i18n("is not in the last") ) {
@@ -614,9 +716,9 @@ QString CriteriaEditor::getSearchCriteria()
 
 
     if( criteria == i18n("contains") )
-        searchCriteria += " LIKE '%" + value + "%'";
+        searchCriteria += CollectionDB::likeCondition( value, true, true );
     else if( criteria == i18n("does not contain") )
-        searchCriteria += " NOT LIKE '%" + value + "%'";
+        searchCriteria += " NOT " + CollectionDB::likeCondition( value, true, true );
     else if( criteria == i18n("is") ) {
         if( m_currentValueType == Date )
             searchCriteria += " BETWEEN ";
@@ -625,9 +727,6 @@ QString CriteriaEditor::getSearchCriteria()
         if( m_currentValueType == String || m_currentValueType == AutoCompletionString )
             value.prepend("'").append("'");
         searchCriteria += value;
-        if (field=="statistics.playcounter" && value=="0") {
-            searchCriteria += " OR statistics.playcounter IS NULL";
-        }
     }
     else if( criteria == i18n("is not") ) {
         if( m_currentValueType == Date )
@@ -639,9 +738,9 @@ QString CriteriaEditor::getSearchCriteria()
         searchCriteria += value;
     }
     else if( criteria == i18n("starts with") )
-        searchCriteria += " LIKE '" + value + "%'";
+        searchCriteria += CollectionDB::likeCondition( value, false, true );
     else if( criteria == i18n("ends with") )
-        searchCriteria += " LIKE '%" + value + "'";
+        searchCriteria += CollectionDB::likeCondition( value, true, false );
     else if( criteria == i18n("is greater than") || criteria == i18n("is after") )
         searchCriteria += " > " + value;
     else if( criteria == i18n("is smaller than") || criteria == i18n("is before" ) )
@@ -667,11 +766,25 @@ void CriteriaEditor::enableRemove( bool enable )
 }
 
 
-void CriteriaEditor::slotRemoveCriteria()
+void CriteriaEditor::slotRemoveCriteriaAny()
 {
-    m_playlistEditor->removeCriteria( this );
+    m_playlistEditor->removeCriteriaAny( this );
 }
 
+void CriteriaEditor::slotRemoveCriteriaAll()
+{
+    m_playlistEditor->removeCriteriaAll( this );
+}
+
+void CriteriaEditor::slotAddCriteriaAny()
+{
+    m_playlistEditor->addCriteriaAny();
+}
+
+void CriteriaEditor::slotAddCriteriaAll()
+{
+    m_playlistEditor->addCriteriaAll();
+}
 
 void CriteriaEditor::slotFieldSelected( int field )
 {
@@ -692,7 +805,7 @@ void CriteriaEditor::slotFieldSelected( int field )
         else if( currentField == 1 ) //album
            items = CollectionDB::instance()->albumList();
         else  //genre
-           items = MetaBundle::genreList();
+           items = CollectionDB::instance()->genreList();
 
         m_comboBox->insertStringList( items );
         m_comboBox->completionObject()->insertItems( items );
@@ -721,7 +834,7 @@ void CriteriaEditor::loadEditWidgets()
 
     QObjectList* list = m_editBox->queryList( "QWidget" );
     for( QObject *obj = list->first(); obj; obj = list->next()  )
-        ((QWidget*)obj)->deleteLater();
+        static_cast<QWidget*>(obj)->deleteLater();
 
     delete list;
 
@@ -738,8 +851,9 @@ void CriteriaEditor::loadEditWidgets()
         case AutoCompletionString:    //artist, album, genre
         {
             m_comboBox = new KComboBox( true, m_editBox );
-            m_lineEdit = (KLineEdit*)m_comboBox->lineEdit();
+            m_lineEdit = static_cast<KLineEdit*>( m_comboBox->lineEdit() );
             m_lineEdit->setFocus();
+            m_comboBox->setMinimumSize( QSize( 240, 20 ) );
             m_comboBox->show();
             break;
         }
@@ -774,6 +888,24 @@ void CriteriaEditor::loadEditWidgets()
             break;
         }
 
+        case Rating:
+        {
+            const QStringList list = MetaBundle::ratingList();
+            m_comboBox = new KComboBox( false, m_editBox );
+            m_comboBox->insertStringList( list );
+            m_comboBox->show();
+
+            if( m_criteriaCombo->currentText() == i18n("is between") ) {
+                m_rangeLabel = new QLabel( i18n("and"), m_editBox );
+                m_rangeLabel->setAlignment( AlignCenter );
+                m_rangeLabel->show();
+                m_comboBox2 = new KComboBox( false, m_editBox );
+                m_comboBox2->insertStringList( list );
+                m_comboBox2->show();
+            }
+            break;
+        }
+
         case Date:
         {
             if( m_criteriaCombo->currentText() == i18n("is in the last") ||
@@ -823,6 +955,7 @@ void CriteriaEditor::loadCriteriaList( int valueType, QString condition )
                   << i18n( "starts with" ) << i18n( "ends with" );
             break;
 
+        case Rating:
         case Number:
             items << i18n( "is" ) << i18n( "is not" ) << i18n( "is greater than" ) << i18n( "is smaller than" )
                   << i18n( "is between" );
@@ -859,7 +992,7 @@ int CriteriaEditor::getValueType( int index )
             break;
         case 3:
         case 7:
-        case 13:
+        case 14:
             valueType = String;
             break;
         case 4:
@@ -868,6 +1001,9 @@ int CriteriaEditor::getValueType( int index )
         case 9:
             valueType = Number;
             break;
+        case 10:
+            valueType = Rating;
+            break;
         case 6:
             valueType = Year;
             break;
