@@ -23,21 +23,25 @@
 #include "amarokcore/amarokconfig.h"
 #include "debug.h"
 #include "enginecontroller.h"
+#include "queueLabel.h"
+#include "metabundle.h"
+#include "sliderwidget.h"
+#include "statusbar.h"
+
 #include <kiconloader.h>
 #include <klocale.h>
-#include "metabundle.h"
+
 #include <qhbox.h>
 #include <qlayout.h>
 #include <qtimer.h>
-#include "sliderwidget.h"
-#include "statusbar.h"
-#include "queueLabel.h"
 
 // stuff that must be included last
 //#include "startupTips.h"
 #include "timeLabel.h"
 #include "toggleLabel.h"
 #include "toggleLabel.moc"
+#include "selectLabel.h"
+#include "selectLabel.moc"
 
 
 namespace amaroK {
@@ -67,7 +71,7 @@ StatusBar::StatusBar( QWidget *parent, const char *name )
     QWidget *positionBox = new QWidget( this, "positionBox" );
     QBoxLayout *box = new QHBoxLayout( positionBox, 1, 3 );
 
-    m_slider = new amaroK::Slider( Qt::Horizontal, positionBox );
+    m_slider = new amaroK::MixedSlider( positionBox );
     m_timeLabel = new TimeLabel( positionBox );
     m_slider->setMinimumWidth( m_timeLabel->width() );
 
@@ -76,9 +80,8 @@ StatusBar::StatusBar( QWidget *parent, const char *name )
     QBoxLayout *layout = new QHBoxLayout( hbox, 0, 2 );
     layout->addSpacing( 3 );
     layout->addWidget( m_queueLabel = new QueueLabel( hbox ) );
-    layout->addWidget( new ToggleLabel( (KToggleAction*)amaroK::action( "random_mode" ), hbox ) );
-    layout->addWidget( new ToggleLabel( (KToggleAction*)amaroK::action( "repeat_playlist" ), hbox ) );
-    layout->addWidget( new ToggleLabel( (KToggleAction*)amaroK::action( "dynamic_mode" ), hbox ) );
+    layout->addWidget( new SelectLabel( static_cast<amaroK::SelectAction*>( amaroK::action( "repeat" ) ), hbox ) );
+    layout->addWidget( new SelectLabel( static_cast<amaroK::SelectAction*>( amaroK::action( "random_mode" ) ), hbox ) );
     layout->addSpacing( 3 );
 
     //TODO reimplement insertChild() instead
@@ -198,23 +201,27 @@ StatusBar::slotItemCountChanged( int newCount, int newLength,  //total
 {
     const bool hasSel = ( selCount > 1 ), hasVis = ( visCount != newCount );
 
-    QString text = ( hasSel && hasVis ) ? i18n( "%1 selected of %2 visible of %3 tracks" )
-                                          .arg( selCount ).arg( visCount ).arg( newCount )
+    QString text = ( hasSel && hasVis ) ? i18n( "%1 selected of %2 visible tracks" )
+                                          .arg( selCount ).arg( visCount )
                  : ( hasVis && newCount == 1 ) ? i18n( "0 visible of 1 track" )
                  : ( hasVis ) ? i18n( "%1 visible of %2 tracks" ).arg( visCount).arg( newCount )
                  : ( hasSel ) ? i18n( "%1 selected of %2 tracks" ).arg( selCount ).arg( newCount )
                  : i18n( "1 track", "%n tracks", newCount );
 
-    QString time = ( hasSel && hasVis ) ? i18n( " - [ %1 / %2 / %3 ]" )
-                                          .arg( MetaBundle::prettyLength( selLength, true ) )
-                                          .arg( MetaBundle::prettyLength( visLength, true ) )
-                                          .arg( MetaBundle::prettyLength( newLength, true ) )
-                 : ( ( hasSel || hasVis ) && visCount > 0 ) ? i18n( " - [ %1 / %2 ]" )
-                                                  .arg( hasVis ? MetaBundle::prettyLength( visLength, true )
-                                                               : MetaBundle::prettyLength( selLength, true ) )
-                                                  .arg( MetaBundle::prettyLength( newLength, true ) )
-                 : ( newCount ) ? i18n( " - [ %1 ]" ).arg( MetaBundle::prettyLength( newLength, true ) )
-                 : "";
+    QString time = QString::null;
+    int getValue = 0;
+
+    if( hasSel )
+        getValue = selLength;
+
+    else if( hasVis )
+        getValue = visLength;
+
+    else
+        getValue = newLength;
+
+    if( getValue )
+        time = i18n( " (%2)" ).arg( MetaBundle::prettyLength( getValue, true ) );
 
     m_itemCountLabel->setText( ' ' + text + time + ' ' );
 }
@@ -228,6 +235,12 @@ StatusBar::engineTrackPositionChanged( long position, bool /*userSeek*/ )
         drawTimeDisplay( position );
 }
 
+void
+StatusBar::engineTrackLengthChanged( long length )
+{
+    m_slider->setMaxValue( length * 1000 );
+}
+
 void
 StatusBar::drawTimeDisplay( int ms )  //SLOT
 {
