@@ -25,59 +25,41 @@ namespace amaroK
     static QPixmap
     loadOverlay( const char *iconName )
     {
-        return QImage( locate( "data", QString( "amarok/images/b_%1.png" ).arg( iconName ) ), "PNG" ).smoothScale( 12, 12 );
+        return QImage( locate( "data", QString( "amarok/images/b_%1.png" ).arg( iconName ) ), "PNG" ).smoothScale( 10, 10 );
     }
 }
 
 
 amaroK::TrayIcon::TrayIcon( QWidget *playerWidget )
-  : KSystemTray( playerWidget )
-  , trackLength( 0 )
-  , mergeLevel( -1 )
-  , overlay( 0 )
-  , blinkTimerID( 0 )
-  , overlayVisible( false )
+        : KSystemTray( playerWidget )
+        , EngineObserver( EngineController::instance() )
+        , trackLength( 0 )
+        , mergeLevel( -1 )
+        , overlay( 0 )
+        , blinkTimerID( 0 )
+        , overlayVisible( false )
 {
     KActionCollection* const ac = amaroK::actionCollection();
 
     setAcceptDrops( true );
 
-    ac->action( "prev"  )->plug( contextMenu() );
-    ac->action( "play"  )->plug( contextMenu() );
-    ac->action( "pause" )->plug( contextMenu() );
-    ac->action( "stop"  )->plug( contextMenu() );
-    ac->action( "next"  )->plug( contextMenu() );
-
-    QPopupMenu &p = *contextMenu();
-    QStringList shortcuts; shortcuts << "" << "Z" << "X" << "C" << "V" << "B";
-    const QString body = "|&%1| %2";
-
-    for( uint index = 1; index < 6; ++index )
-    {
-        int id = p.idAt( index );
-        p.changeItem( id, body.arg( *shortcuts.at( index ), p.text( id ) ) );
-    }
+    ac->action( "prev"        )->plug( contextMenu() );
+    ac->action( "play_pause"  )->plug( contextMenu() );
+    ac->action( "stop"        )->plug( contextMenu() );
+    ac->action( "next"        )->plug( contextMenu() );
 
     //seems to be necessary
     KAction *quit = actionCollection()->action( "file_quit" );
     quit->disconnect();
-    connect( quit, SIGNAL( activated() ), kapp, SLOT( quit() ) );
+    connect( quit, SIGNAL(activated()), kapp, SLOT(quit()) );
 
     baseIcon     = KSystemTray::loadIcon( "amarok" );
     playOverlay  = amaroK::loadOverlay( "play" );
     pauseOverlay = amaroK::loadOverlay( "pause" );
-    stopOverlay  = amaroK::loadOverlay( "stop" );
     overlayVisible = false;
+
     //paintIcon();
     setPixmap( baseIcon );
-
-    // attach to get notified about engine events
-    EngineController::instance()->attach( this );
-}
-
-amaroK::TrayIcon::~TrayIcon( )
-{
-    EngineController::instance()->detach( this );
 }
 
 bool
@@ -101,15 +83,6 @@ amaroK::TrayIcon::event( QEvent *e )
             paintIcon( mergeLevel, true );
         }
 
-        // if we're stopped return to default state after the first tick
-        else if ( overlay == &stopOverlay )
-        {
-            killTimer( blinkTimerID );
-            blinkTimerID = 0;
-            overlayVisible = false;
-            paintIcon( -1, true );
-        }
-
         return true;
 
     case QEvent::MouseButtonPress:
@@ -128,7 +101,7 @@ amaroK::TrayIcon::event( QEvent *e )
 }
 
 void
-amaroK::TrayIcon::engineStateChanged( Engine::State state )
+amaroK::TrayIcon::engineStateChanged( Engine::State state, Engine::State /*oldState*/ )
 {
     // stop timer
     if ( blinkTimerID )
@@ -149,14 +122,18 @@ amaroK::TrayIcon::engineStateChanged( Engine::State state )
 
     case Engine::Playing:
         overlay = &playOverlay;
-        if( AmarokConfig::animateTrayIcon() ) blinkTimerID = startTimer( 1500 );  // start 'blink' timer
-        else paintIcon( mergeLevel, true ); // repaint the icon
+        if( AmarokConfig::animateTrayIcon() )
+           blinkTimerID = startTimer( 1500 );  // start 'blink' timer
+
+        paintIcon( mergeLevel, true ); // repaint the icon
         break;
 
-    default: // idle/stopped case
-        overlay = &stopOverlay;
-        if( AmarokConfig::animateTrayIcon() ) blinkTimerID = startTimer( 2500 );  // start 'blink' timer
-        else paintIcon( -1, true ); // repaint the icon
+    case Engine::Empty:
+        overlayVisible = false;
+        paintIcon( -1, true ); // repaint the icon
+
+    default:
+        ;
     }
 }
 
@@ -167,7 +144,7 @@ amaroK::TrayIcon::engineNewMetaData( const MetaBundle &bundle, bool /*trackChang
 }
 
 void
-amaroK::TrayIcon::engineTrackPositionChanged( long position )
+amaroK::TrayIcon::engineTrackPositionChanged( long position, bool /*userSeek*/ )
 {
     mergeLevel = trackLength ? ((baseIcon.height() + 1) * position) / trackLength : -1;
     paintIcon( mergeLevel );
@@ -247,6 +224,11 @@ amaroK::TrayIcon::blendOverlay( QPixmap &sourcePixmap )
     QPixmap sourceCropped( opW, opH, sourcePixmap.depth() );
     copyBlt( &sourceCropped, 0,0, &sourcePixmap, opX,opY, opW,opH );
 
+    //speculative fix for a bactrace we received
+    //crash was in covertToImage() somewhere in this function
+    if( sourceCropped.isNull() )
+        return setPixmap( sourcePixmap );
+
     // blend the overlay image over the cropped rectangle
     QImage blendedImage = sourceCropped.convertToImage();
     QImage overlayImage = overlay->convertToImage();
