@@ -1,17 +1,20 @@
 // Author: Max Howell (C) Copyright 2004
-// Copyright: See COPYING file that comes with this distribution
+// (c) 2005 Jeff Mitchell <kde-dev@emailgoeshere.com>
+// See COPYING file that comes with this distribution
 //
 
 #ifndef THREADWEAVER_H
 #define THREADWEAVER_H
 
+#include "debug.h"
 #include <qevent.h>   //baseclass
 #include <qguardedptr.h>
 #include <qmap.h>
 #include <qobject.h>
 #include <qthread.h>
 #include <qvaluelist.h>
-
+#include <qmutex.h>
+#include "debug.h"
 
 #define DISABLE_GENERATED_MEMBER_FUNCTIONS_3( T ) \
     T( const T& ); \
@@ -83,11 +86,10 @@ protected:    bool m_aborted;
 
 class ThreadWeaver : public QObject
 {
+public:
     class Thread;
     friend class Thread;
     typedef QValueList<Thread*> ThreadList;
-
-public:
     class Job;
     friend class Job;
     typedef QValueList<Job*> JobList;
@@ -95,6 +97,10 @@ public:
     static ThreadWeaver *instance();
     static void deleteInstance();
 
+    static volatile uint getNewThreadId();
+    static QMutex *threadIdMutex;
+    static volatile uint threadIdCounter;
+
     /**
      * If the ThreadWeaver is already handling a job of this type then the job
      * will be queued, otherwise the job will be processed immediately. Allocate
@@ -157,7 +163,19 @@ private:
 
     virtual bool event( QEvent* );
 
+    /// checks the pool for an available thread, creates a new one if required
+    Thread *gimmeThread();
 
+    /// safe disposal for threads that may not have finished
+    void dispose( Thread* );
+
+    /// all pending and running jobs
+    JobList m_jobs;
+
+    /// a thread-pool, ready for use or running jobs currently
+    ThreadList m_threads;
+
+public:
     /**
      * Class Thread
      */
@@ -173,9 +191,15 @@ private:
 
         Job *job() const { return m_job; }
 
+        static QThread* getRunning();
+        static QString  threadId();
+        const uint localThreadId() const { return m_threadId; }
+
     private:
         Job *m_job;
 
+        uint m_threadId;
+
         //private so I don't break something in the distant future
         ~Thread();
 
@@ -185,21 +209,7 @@ private:
     protected:
         DISABLE_GENERATED_MEMBER_FUNCTIONS_3( Thread )
     };
-
-
-    /// checks the pool for an available thread, creates a new one if required
-    Thread *gimmeThread();
-
-    /// safe disposal for threads that may not have finished
-    void dispose( Thread* );
-
-    /// all pending and running jobs
-    JobList m_jobs;
-
-    /// a thread-pool, ready for use or running jobs currently
-    ThreadList m_threads;
-
-public:
+    
     /**
      * @class Job
      * @short A small class for doing work in a background thread
@@ -294,6 +304,8 @@ public:
          */
         //void setVisible( bool );
 
+        uint parentThreadId() { return m_parentThreadId; }
+
     protected:
         /**
          * Executed inside the thread, this should be reimplemented to do the
@@ -320,6 +332,7 @@ public:
         uint m_percentDone;
         uint m_progressDone;
         uint m_totalSteps;
+        uint m_parentThreadId;
 
         QString m_description;
         QString m_status;
@@ -379,6 +392,13 @@ protected:
     ThreadWeaver &operator=( const ThreadWeaver& );
 };
 
+//useful debug thingy
+#define DEBUG_THREAD_FUNC_INFO kdDebug() << Debug::indent() << k_funcinfo << "thread: " << ThreadWeaver::Thread::threadId() << endl;
+
+#define SHOULD_BE_GUI if( ThreadWeaver::Thread::getRunning() ) warning() \
+    << "Should not be Threaded, but is running in" << \
+    ThreadWeaver::Thread::getRunning() <<endl;
+
 inline ThreadWeaver*
 ThreadWeaver::instance()
 {
@@ -393,4 +413,14 @@ ThreadWeaver::deleteInstance()
     delete instance();
 }
 
+inline volatile uint
+ThreadWeaver::getNewThreadId()
+{
+    uint temp;
+    threadIdMutex->lock();
+    temp = threadIdCounter++;
+    threadIdMutex->unlock();
+    return temp;
+}
+
 #endif
