@@ -15,13 +15,18 @@
   email:     illissius@gmail.com
 */
 
-#include "tracktooltip.h"
+#include "amarok.h"
+#include "debug.h"
 #include "metabundle.h"
 #include "collectiondb.h"
 #include "playlist.h"
 #include "playlistitem.h"
-#include <qtooltip.h>
+#include "podcastbundle.h"
 #include <qapplication.h>
+#include <kstandarddirs.h>
+
+
+#include "tracktooltip.h"
 
 
 TrackToolTip *TrackToolTip::instance()
@@ -34,7 +39,13 @@ TrackToolTip::TrackToolTip(): m_haspos( false )
 {
     connect( CollectionDB::instance(), SIGNAL( coverChanged( const QString &, const QString & ) ),
              this, SLOT( slotCoverChanged( const QString &, const QString & ) ) );
-    connect( Playlist::instance(), SIGNAL( columnsChanged() ), this, SLOT( slotColumnsChanged() ) );
+    connect( CollectionDB::instance(), SIGNAL( imageFetched( const QString & ) ),
+             this, SLOT( slotImageChanged( const QString & ) ) );
+    connect( Playlist::instance(), SIGNAL( columnsChanged() ), this, SLOT( slotUpdate() ) );
+    connect( CollectionDB::instance(), SIGNAL( scoreChanged( const QString&, int ) ),
+             this, SLOT( slotUpdate( const QString& ) ) );
+    connect( CollectionDB::instance(), SIGNAL( ratingChanged( const QString&, int ) ),
+             this, SLOT( slotUpdate( const QString& ) ) );
     clear();
 }
 
@@ -43,7 +54,7 @@ void TrackToolTip::addToWidget( QWidget *widget )
     if( widget && !m_widgets.containsRef( widget ) )
     {
         m_widgets.append( widget );
-        QToolTip::add( widget, tooltip() );
+        amaroK::ToolTip::add( this, widget );
     }
 }
 
@@ -51,7 +62,7 @@ void TrackToolTip::removeFromWidget( QWidget *widget )
 {
     if( widget && m_widgets.containsRef( widget ) )
     {
-        QToolTip::remove( widget );
+        amaroK::ToolTip::remove( widget );
         m_widgets.removeRef( widget );
     }
 }
@@ -69,7 +80,7 @@ void TrackToolTip::setTrack( const MetaBundle &tags, bool force )
         QString filename = "", title = ""; //special case these, put the first one encountered on top
 
         Playlist *playlist = Playlist::instance();
-        const int n = playlist->visibleColumns();
+        const int n = playlist->numVisibleColumns();
         for( int i = 0; i < n; ++i )
         {
             const int column = playlist->mapToLogicalColumn( i );
@@ -83,7 +94,27 @@ void TrackToolTip::setTrack( const MetaBundle &tags, bool force )
                     left << playlist->columnText( column );
                 }
             }
-            else if( column == PlaylistItem::Playcount )
+            else if( column == PlaylistItem::Rating )
+            {
+                const int rating = CollectionDB::instance()->getSongRating( tags.url().path() );
+                if( rating > 0 )
+                {
+                    QString s;
+                    for( int i = 0; i < rating / 2; ++i )
+                        s += QString( "<img src=\"%1\" height=\"%2\" width=\"%3\">" )
+                             .arg( locate( "data", "amarok/images/star.png" ) )
+                             .arg( QFontMetrics( QToolTip::font() ).height() )
+                             .arg( QFontMetrics( QToolTip::font() ).height() );
+                    if( rating % 2 )
+                        s += QString( "<img src=\"%1\" height=\"%2\" width=\"%3\">" )
+                             .arg( locate( "data", "amarok/images/smallstar.png" ) )
+                             .arg( QFontMetrics( QToolTip::font() ).height() )
+                             .arg( QFontMetrics( QToolTip::font() ).height() );
+                    right << s;
+                    left << playlist->columnText( column );
+                }
+            }
+            else if( column == PlaylistItem::PlayCount )
             {
                 const int count = CollectionDB::instance()->getPlayCount( tags.url().path() );
                 if( count > 0 )
@@ -92,13 +123,19 @@ void TrackToolTip::setTrack( const MetaBundle &tags, bool force )
                     left << playlist->columnText( column );
                 }
             }
+            else if( column == PlaylistItem::LastPlayed )
+            {
+                const uint lastPlayed = CollectionDB::instance()->getLastPlay( tags.url().path() ).toTime_t();
+                right << amaroK::verboseTimeSince( lastPlayed );
+                left << playlist->columnText( column );
+            }
             else if( column == PlaylistItem::Filename && title.isEmpty() )
-                filename = tags.infoByColumn( column, true );
+                filename = tags.prettyText( column );
             else if( column == PlaylistItem::Title && filename.isEmpty() )
-                title = tags.infoByColumn( column, true );
+                title = tags.prettyText( column );
             else if( column != PlaylistItem::Length )
             {
-                const QString tag = tags.infoByColumn( column, true );
+                const QString tag = tags.prettyText( column );
                 if( !tag.isEmpty() )
                 {
                     right << tag;
@@ -129,9 +166,13 @@ void TrackToolTip::setTrack( const MetaBundle &tags, bool force )
         m_tooltip += "<center><b>amaroK</b></center><table cellpadding='2' cellspacing='2' align='center'><tr>";
 
         m_tooltip += "%1"; //the cover gets substituted in, in tooltip()
-        m_cover = CollectionDB::instance()->albumImage( tags );
-        if ( m_cover == CollectionDB::instance()->notAvailCover() )
-            m_cover = QString::null;
+        m_cover = CollectionDB::instance()->podcastImage( tags );
+        if( m_cover.isEmpty() || m_cover.contains( "nocover" ) != -1 )
+        {
+            m_cover = CollectionDB::instance()->albumImage( tags );
+            if ( m_cover == CollectionDB::instance()->notAvailCover() )
+                m_cover = QString::null;
+        }
 
         m_tooltip += "<td><table cellpadding='0' cellspacing='0'>";
 
@@ -171,6 +212,11 @@ void TrackToolTip::clear()
     updateWidgets();
 }
 
+QPair<QString, QRect> TrackToolTip::toolTipText( QWidget*, const QPoint& ) const
+{
+    return QPair<QString, QRect>( tooltip(), QRect() );
+}
+
 void TrackToolTip::slotCoverChanged( const QString &artist, const QString &album )
 {
     if( artist == m_tags.artist() && album == m_tags.album() )
@@ -183,12 +229,34 @@ void TrackToolTip::slotCoverChanged( const QString &artist, const QString &album
     }
 }
 
-void TrackToolTip::slotColumnsChanged()
+void TrackToolTip::slotImageChanged( const QString &remoteURL )
 {
-    setTrack( m_tags, true );
+    PodcastEpisodeBundle peb;
+    if( CollectionDB::instance()->getPodcastEpisodeBundle( m_tags.url().url(), &peb ) )
+    {
+        PodcastChannelBundle pcb;
+        if( CollectionDB::instance()->getPodcastChannelBundle( peb.parent().url(), &pcb ) )
+        {
+            if( pcb.imageURL().url() == remoteURL )
+            {
+                m_cover = CollectionDB::instance()->podcastImage( remoteURL );
+                if( m_cover == CollectionDB::instance()->notAvailCover() )
+                    m_cover = QString::null;
+
+                updateWidgets();
+            }
+
+        }
+    }
 }
 
-QString TrackToolTip::tooltip()
+void TrackToolTip::slotUpdate( const QString &url )
+{
+    if( url.isNull() || url == m_tags.url().path() )
+        setTrack( m_tags, true );
+}
+
+QString TrackToolTip::tooltip() const
 {
     QString tip = m_tooltip;;
     if( !m_tags.isEmpty() )
@@ -207,8 +275,7 @@ QString TrackToolTip::tooltip()
 
 void TrackToolTip::updateWidgets()
 {
-    for( QWidget *widget = m_widgets.first(); widget; widget = m_widgets.next() )
-        QToolTip::add( widget, tooltip() );
+    amaroK::ToolTip::updateTip();
 }
 
 #include "tracktooltip.moc"
