@@ -8,6 +8,7 @@
 #include <dirent.h>
 #include <dlfcn.h>           //dlopen etc.
 #include <stdlib.h>
+#include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/un.h>
@@ -35,7 +36,7 @@ GtkWidget *mainwin = &dummy;      //required by msa visplugin
 using std::string;
 
 
-int    tryConnect();
+int    tryConnect( const char* );
 string getPlugin( int, char** );
 void   vis_disable_plugin( VisPlugin* ) { exit( 0 ); } //this is called by the plugin when it wants to quit
 
@@ -44,7 +45,7 @@ void   vis_disable_plugin( VisPlugin* ) { exit( 0 ); } //this is called by the p
 int
 main( int argc, char** argv )
 {
-    if( argc > 1 && strcmp( argv[1], "--list" ) == 0 )
+    if( argc <= 1 || strcmp( argv[1], "--list" ) == 0 )
     {
         struct dirent *ent;
         struct stat statbuf;
@@ -57,7 +58,8 @@ main( int argc, char** argv )
         while( (ent = readdir( dir )) )
         {
             filename = ent->d_name;
-            const uint index = filename.find_last_of( '.' );
+            string::size_type index;
+            index = filename.find_last_of( '.' );
 
             if( index == string::npos ) continue;
 
@@ -79,9 +81,13 @@ main( int argc, char** argv )
 
         std::exit( 0 );
     }
+    else if( argc != 3 ) {
+        std::cout << "usage: xmms_wrapper2 socket_path plugin\n";
+        std::exit( 1 );
+    }
 
     //connect to socket
-    const int sockfd = tryConnect();
+    const int sockfd = tryConnect( argv[1] );
     if( sockfd == -1 ) exit( 1 );
 
 
@@ -115,7 +121,7 @@ main( int argc, char** argv )
 
     while( nbytes != -1 )
     {
-        gtk_main_iteration_do( FALSE );
+        gtk_main_iteration_do( false );
 
         //set the time to wait, we have to do this everytime on linux
         tv.tv_sec  = 0;
@@ -135,15 +141,23 @@ main( int argc, char** argv )
             recv( sockfd, c, 16, 0 );
             std::string command( c );
 
-            if( command == "fullscreen" );
+            //TODO fix this next line
+            //if( command == "fullscreen" )
             if( command == "configure" ) wrap.configure();
         }
 
         ::send( sockfd, "PCM", 4, 0 );
-        nbytes = ::recv( sockfd, pcm_data, 512 * sizeof( gint16 ), 0 );
+        // get 1024 samples, interleaved PCM from amaroK
+        gint16 data[1024];
+        nbytes = ::recv( sockfd, data, 1024 * sizeof( gint16 ), 0 );
 
+        int i = 0;
         for( uint x = 0; x < 512; ++x )
-            pcm_data[1][x] = pcm_data[0][x];
+        {
+           pcm_data[1][x] = data[i];
+           pcm_data[0][x] = data[i+1];
+           i+=2;
+        }
 
         wrap.render( pcm_data );
     }
@@ -154,17 +168,8 @@ main( int argc, char** argv )
 }
 
 int
-tryConnect()
+tryConnect( const char *path )
 {
-    #ifndef MAX_PATH
-    #define MAX_PATH 256
-    #endif
-
-    char path[MAX_PATH];
-    FILE *amarok = popen( "amarok --vis-socket-path", "r" );
-    fread( path, sizeof(char), MAX_PATH, amarok );
-    pclose( amarok );
-
     int fd = ::socket( AF_UNIX, SOCK_STREAM, 0 );
 
     if( fd != -1 )
@@ -189,58 +194,11 @@ tryConnect()
 }
 
 std::string
-getPlugin( int argc, char **argv )
+getPlugin( int /*argc*/, char **argv )
 {
-    if( argc == 1 )
-    {
-        std::list<string> list;
-
-        //scan plugins
-        string dirname = XMMS_PLUGIN_PATH;
-        dirname.append( "/" );
-        DIR *dir;
-        struct dirent *ent;
-        struct stat statbuf;
-
-        dir = opendir( dirname.c_str() );
-        if ( !dir ) { std::cerr << "Please edit the PLUGIN_PATH in xmmswrapper.cpp\n"; exit( 1 ); }
-
-        while ( (ent = readdir( dir )) )
-        {
-            string filename = ent->d_name;
-            uint index = filename.find_last_of( '.' );
-            if ( index == string::npos ) continue;
-            string extension = filename.substr( index );
-            string fullpath = dirname + filename;
-
-            if ( !stat( fullpath.c_str(), &statbuf )
-                    && S_ISREG( statbuf.st_mode )
-                    && extension == SHARED_LIB_EXT )
-                list.push_back( filename );
-        }
-        closedir( dir );
-
-        std::cout << "Please select a plugin: \n";
-
-        std::vector<string> v( list.size() );
-        std::copy( list.begin(), list.end(), v.begin() );
-
-        for ( uint n = 0; n < v.size(); ++n )
-        {
-            std::cout << n + 1 << ": " << v[ n ] << '\n';
-        }
-
-        char c[8];
-        std::cin >> c;
-        uint selection = atoi( c );
-        if( selection > v.size() ) exit( 1 );
-
-        return v[ selection - 1 ];
-    }
-
     std::string
     plugin  = "lib";
-    plugin += argv[1];
+    plugin += argv[2];
     plugin += ".so";
 
     return plugin;
